﻿/* Author: Jim-Kyrre Benjaminsen AKA Romfarer
 * License: This work is licensed under a Creative Commons Attribution-NonCommercial-NoDerivs 3.0 Unported License. 
 * http://creativecommons.org/licenses/by-nc-nd/3.0/
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;
using KSP.IO;

namespace Romfarer {
    class CFGI_KPS {

        private LazorSystem system;
        public delegate bool Buttons(KPSmapLocation i);
        Dictionary<KPSmapLocation, Buttons> tasks = new Dictionary<KPSmapLocation, Buttons>();
        Dictionary<KPSmapLocation, Buttons> taskstasks = new Dictionary<KPSmapLocation, Buttons>();
        private enum CoordSystem { Decimal = 0, Sexagesimal = 1 };
        private CoordSystem currentCoordSystem = CoordSystem.Decimal;
        public List<KPSmapLocation> list;
        public List<Buttons> standardButtons;
        public KPSmapLocation edit; //the new one
        private static char[] delimiters = new char[] { '#', '\t' };
        private Dictionary<string, CelestialBody> cbDictionary;
        private Rect popupPos;
        private int index = 1;
        private string txtlat;
        private string txtlon;
        private bool txtlatset;
        private bool txtlonset;


        public CFGI_KPS(LazorSystem system) {
            this.system = system;
            list = new List<KPSmapLocation>();
            cbDictionary = new Dictionary<string, CelestialBody>();
            foreach (CelestialBody cb in FlightGlobals.Bodies)
                cbDictionary.Add(cb.name, cb);
            standardButtons = new List<Buttons>();
            standardButtons.Add(StdButtonEdit);
            standardButtons.Add(StdButtonDelete);
            popupPos = new Rect(Screen.width / 2, Screen.height / 2, 10, 10);
            Load();
        }

        public void Add(KPSmapLocation item) {
            list.Add(item);
        }

        //Add item if it doesn't exist in the list
        public bool AddExclusive(KPSmapLocation item) {
            if (!list.Exists(u => u.id == item.id)) {
                Add(item);
                return true;
            } return false;
        }

        //Replace any occurance or add item to the list
        public void AddReplacement(KPSmapLocation item) {
            list.RemoveAll(u => u.id == item.id);
            Add(item);
        }

        public void Remove(KPSmapLocation item) {
            list.RemoveAll(u => u.id == item.id);
        }

        public List<KPSmapLocation> Get() {
            return list;
        }

        public List<KPSmapLocation> GetByCelestialBody(CelestialBody cb) {
            return list.FindAll(x => x.cb == cb);
        }

        public int Load() {
            if (!File.Exists<CFGI_KPS>("kps.txt")) {
                AddExclusive(new KPSmapLocation("loc1", "Crater 1", 4.16064834714487d, -55.6260228246487d, cbDictionary["Mun"]));
                AddExclusive(new KPSmapLocation("loc2", "Crater 2", 63.5456339296924d, -32.3903335090882d, cbDictionary["Mun"]));
                AddExclusive(new KPSmapLocation("loc3", "Crater 3", 13.1034661976821d, 20.7120616562151d, cbDictionary["Mun"]));
                AddExclusive(new KPSmapLocation("loc4", "Crater 4", -9.00872987151143d, 81.8389789338056d, cbDictionary["Mun"]));
                AddExclusive(new KPSmapLocation("loc5", "Crater 5", -20.0523976065337d, 135.036532057894d, cbDictionary["Mun"]));
                AddExclusive(new KPSmapLocation("loc6", "Crater 6", 6.97189113955997d, -150.202351223027d, cbDictionary["Mun"]));
                AddExclusive(new KPSmapLocation("loc7", "Romfarer landing", -37.6715834126541d, 4.7115849947796d, cbDictionary["Mun"]));
                AddExclusive(new KPSmapLocation("loc8", "Romfarer was here", -23.7466988564638d, -342.025066661392d, cbDictionary["Minmus"]));
                AddExclusive(new KPSmapLocation("loc9", "Romfarer Bay", 62.6437630408871d, -117.351140052713d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc10", "KSC 2", 20.6616392056821d, -146.333447448622d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc11", "Booster Bay", 1d, -74d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc12", "Shoeshore Archipelago", 5, 62, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc13", "Mount Vildan", 62.680808d, 45.74196d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc14", "Borgnine Beach", -11.416667d, 169.911111d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc15", "Lac-en-Vert", 50.557778d, 175.237222d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc16", "Malbert's Peak", 0.527d, -33.487d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc17", "Tear Duct Glacier", 71.582d, 75.595d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc18", "Spaniel", 5.886d, -61.973d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc19", "Touchdown valley", 0.211d, 74.691d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc20", "South Island", -42.053d, -117.742d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc21", "Mt. Kurt", 0.999d, -79.198d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc22", "Harvester Mountains", 35d, -79.198d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc23", "Valley of Death", 41.532d, 84.183d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc24", "Aerotilt Isle", 1.959d, 109.019d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc25", "Pelóri mountains", -68.511d, -32.18d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc26", "Brain Pit Valley", 10.73d, -116.98d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc27", "Fox's Mesa", 30.136d, -43.272d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc28", "Kerbolius Gulf", -1.281d, -96.504d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc29", "B.B. Sea", -61.241d, -35.43d, cbDictionary["Kerbin"]));
                AddExclusive(new KPSmapLocation("loc30", "Red Rocket Valley", 0.654d, -78.371d, cbDictionary["Kerbin"]));
                Save();
                index = 31;
            } else {
                try {
                    TextReader tr = TextReader.CreateForType<CFGI_KPS>("kps.txt");
                    while (tr.Peek() != -1) {
                        string l = tr.ReadLine();
                        string[] item = l.Split(delimiters);
                        CelestialBody cb;
                        if (cbDictionary.TryGetValue(item[0], out cb))
                            AddExclusive(new KPSmapLocation("loc" + index, item[3], Double.Parse(item[1]), Double.Parse(item[2]), cb));
                        index++;
                    }
                    tr.Close();
                    tr.Dispose();
                } catch (IOException) { } catch (Exception) { }
            }
            return index;
        }

        public void Save() {
            try {
                TextWriter tw = TextWriter.CreateForType<CFGI_KPS>("kps.txt");
                for (int i = 0; i < list.Count; i++)
                    tw.WriteLine(list[i].cb.name + "#" + list[i].lat + "#" + list[i].lon + "#" + list[i].name);
                tw.Flush();
                tw.Close();
                tw.Dispose();
            } catch (IOException) { } catch (Exception) { }
        }

        public virtual bool StdButtonEdit(KPSmapLocation i) {
            if (GUILayout.Button("Edit")) {
                resetFlags();
                edit = new KPSmapLocation(i);
                tasks.Add(i, TaskPopupEdit);
            }
            return true;
        }

        public virtual bool StdButtonDelete(KPSmapLocation i) {
            if (GUILayout.Button("Delete")) {
                resetFlags();
                edit = new KPSmapLocation(i);
                tasks.Add(i, TaskPopupDelete);
            }
            return true;
        }

        public virtual bool StdButtonAdd(KPSmapLocation i) { //only call on button press
            resetFlags();
            edit = new KPSmapLocation(i);
            tasks.Add(i, TaskPopupAdd);
            return true;
        }

        public bool TaskPopupAdd(KPSmapLocation i) {
            popupPos = Util.ClampToScreen(GUILayout.Window(10, popupPos, delegate {
                GUILayout.BeginVertical(Util.Space_Compact);
                GUILayout.Space(5);
                NameLatLon();
                GUILayout.Space(5);
                GUILayout.BeginHorizontal();
                if (GUILayout.Button("Yes")) {
                    AddExclusive(new KPSmapLocation("loc" + index, edit.name, edit.lat, edit.lon, system.vessel.mainBody));
                    Save();
                    index++;
                    taskstasks.Add(i, tasks.Remove);
                }
                if (GUILayout.Button("NO"))
                    taskstasks.Add(i, tasks.Remove);
                GUILayout.EndHorizontal();
                GUILayout.EndVertical();
            }, "Add KPS Location", GUILayout.MinWidth(250)));
            return true;
        }
        
        public bool TaskPopupEdit(KPSmapLocation i) {
            popupPos = Util.ClampToScreen(GUILayout.Window(10, popupPos, delegate {
                GUILayout.BeginVertical(Util.Space_Compact);
                GUILayout.Space(5);
                NameLatLon();
                GUILayout.Space(5);
                GUILayout.BeginHorizontal();
                if (GUILayout.Button("Yes")) {
                    AddReplacement(new KPSmapLocation(edit.id, edit.name, edit.lat, edit.lon, edit.cb));
                    Save();
                    taskstasks.Add(i, tasks.Remove);
                }
                if (GUILayout.Button("NO"))
                    taskstasks.Add(i, tasks.Remove);
                GUILayout.EndHorizontal();
                GUILayout.EndVertical();
            }, "Edit KPS Location", GUILayout.MinWidth(250)));
            return true;
        }

        public bool TaskPopupDelete(KPSmapLocation i) {
            popupPos = Util.ClampToScreen(GUILayout.Window(10, popupPos, delegate {
                GUILayout.BeginVertical(Util.Space_Compact);
                GUILayout.Label("Delete " + i.name + "?");
                GUILayout.BeginHorizontal();
                if (GUILayout.Button("Yes")) {
                    Remove(i);
                    Save();
                    taskstasks.Add(i, tasks.Remove);
                }
                if (GUILayout.Button("NO"))
                    taskstasks.Add(i, tasks.Remove);
                GUILayout.EndHorizontal();
                GUILayout.EndVertical();
            }, "Delete KPS Location", GUILayout.MinWidth(250)));
            return true;
        }

        private void resetFlags() {
            txtlatset = txtlonset = false;
        }

        private void NameLatLon() {
            GUILayout.BeginHorizontal();
            GUILayout.Label("Name: ", GUILayout.Width(60));
            edit.name = GUILayout.TextField(edit.name).Replace("\n", "").Replace("\r", "");
            GUILayout.EndHorizontal();
            GUILayout.Space(5);
            currentCoordSystem = (CoordSystem)GUILayout.SelectionGrid((int)currentCoordSystem, Enum.GetNames(typeof(CoordSystem)), 2);
            if (currentCoordSystem == CoordSystem.Decimal) {
                LabelEvent("Latitude: ", ref txtlat, ref txtlatset, ref edit.lat);
                LabelEvent("Longitude: ", ref txtlon, ref txtlonset, ref edit.lon);
            } else if (currentCoordSystem == CoordSystem.Sexagesimal) {
                GUILayout.BeginHorizontal();
                GUILayout.Label("Latitude: ", GUILayout.Width(60));
                double lat_deg, lat_min, lat_sec;
                Util.DegToSex(edit.lat, out lat_deg, out lat_min, out lat_sec);
                lat_deg = double.Parse(GUILayout.TextField(lat_deg.ToString()).Replace("\n", "").Replace("\r", ""));
                lat_min = double.Parse(GUILayout.TextField(lat_min.ToString()).Replace("\n", "").Replace("\r", ""));
                lat_sec = double.Parse(GUILayout.TextField(lat_sec.ToString()).Replace("\n", "").Replace("\r", ""));
                int lat_sign = edit.lat > 0 ? 1 : -1;
                if (edit.lat > 0){
                    if (GUILayout.Button("N", GUILayout.Width(25)))
                        lat_sign = -1;
                }else{
                    if (GUILayout.Button("S", GUILayout.Width(25)))
                        lat_sign = 1;
                }
                edit.lat = lat_sign * Util.SexToDeg(lat_deg, lat_min, lat_sec);
                GUILayout.EndHorizontal();
                GUILayout.BeginHorizontal();
                GUILayout.Label("Longitude: ", GUILayout.Width(60));
                double lon_deg, lon_min, lon_sec;
                Util.DegToSex(edit.lon, out lon_deg, out lon_min, out lon_sec);
                lon_deg = double.Parse(GUILayout.TextField(lon_deg.ToString()).Replace("\n", "").Replace("\r", ""));
                lon_min = double.Parse(GUILayout.TextField(lon_min.ToString()).Replace("\n", "").Replace("\r", ""));
                lon_sec = double.Parse(GUILayout.TextField(lon_sec.ToString()).Replace("\n", "").Replace("\r", ""));
                int lon_sign = edit.lon > 0 ? 1 : -1;
                if (edit.lon > 0) {
                    if (GUILayout.Button("E", GUILayout.Width(25)))
                        lon_sign = -1;
                } else {
                    if (GUILayout.Button("W", GUILayout.Width(25)))
                        lon_sign = 1;
                }
                edit.lon = lon_sign * Util.SexToDeg(lon_deg, lon_min, lon_sec);
                GUILayout.EndHorizontal();
            }
        }

        private void LabelEvent(string name, ref string field, ref bool flag, ref double value){
            GUILayout.BeginHorizontal();
                GUILayout.Label(name, GUILayout.Width(60));
                if (!flag) {
                    field = value.ToString();
                    flag = true;
                }
                GUI.SetNextControlName(name);
                field = GUILayout.TextField(field).Replace("\n", "").Replace("\r", "");
                double fieldTest;
                if (!field.EndsWith(".") && Double.TryParse(field, out fieldTest)) {
                    value = fieldTest;
                    flag = false;
                }
                GUILayout.EndHorizontal();
        }

        private DoubleSwitch sendFront;
        public void ExecuteTasks() { //executed in guisubsystems

            foreach (KPSmapLocation task in tasks.Keys)
                tasks[task](task);
            foreach (KPSmapLocation task in taskstasks.Keys)
                taskstasks[task](task);
            taskstasks.Clear();

            sendFront.set = tasks.Count > 0;
            if (sendFront.In())
                GUI.BringWindowToFront(10);
            else if (sendFront.Out()) { }
        }

    }
}
