﻿/* Author: Jim-Kyrre Benjaminsen AKA Romfarer
 * License: This work is licensed under a Creative Commons Attribution-NonCommercial-NoDerivs 3.0 Unported License. 
 * http://creativecommons.org/licenses/by-nc-nd/3.0/
*/

using System;
using System.Collections.Generic;
using System.Text;
using UnityEngine;
using System.Timers;

namespace Romfarer {
    class CFG_blue : CFG, IDisposable {

        //private Collider track;
        //private Rect windowPos;



        //Transfer
        private double ejectV;
        private double ejectVdelta;
        private double window;
        private double ToF;
        private double VtxA;
        private double deltaVA = 0;
        private CelestialBody transfer_target;
        private CelestialBody transfer_reference;
        private HashSet<CelestialBody> transferBodies = new HashSet<CelestialBody>();
        private LinkedList<CelestialBody> transferQueue = new LinkedList<CelestialBody>();
        private bool toggleBlue_all {
            get { return system.guiOverlays.toggleBlue_all; }
            set { system.guiOverlays.toggleBlue_all = value; }
        }
        private double e {
            get { return system.guiOverlays.e; }
            set { system.guiOverlays.e = value; }
        }
        private float phase {
            get { return system.guiOverlays.phase; }
            set { system.guiOverlays.phase = value; }
        }
        private float phaseD {
            get { return system.guiOverlays.phaseD; }
            set { system.guiOverlays.phaseD = value; }
        }
        private float phaseD2 {
            get { return system.guiOverlays.phaseD2; }
            set { system.guiOverlays.phaseD2 = value; }
        }
        private float angle {
            get { return system.guiOverlays.angle; }
            set { system.guiOverlays.angle = value; }
        }
        private float angleD {
            get { return system.guiOverlays.angleD; }
            set { system.guiOverlays.angleD = value; }
        }
        private Vector3 pos_target {
            get { return system.guiOverlays.pos_target; }
            set { system.guiOverlays.pos_target = value; }
        }
        private Vector3 pos_reference {
            get { return system.guiOverlays.pos_reference; }
            set { system.guiOverlays.pos_reference = value; }
        }
        private Vector3 pos_origin {
            get { return system.guiOverlays.pos_origin; }
            set { system.guiOverlays.pos_origin = value; }
        }
        private Vector3 normal_reference {
            get { return system.guiOverlays.normal_reference; }
            set { system.guiOverlays.normal_reference = value; }
        }
        private Vector3 normal_target {
            get { return system.guiOverlays.normal_target; }
            set { system.guiOverlays.normal_target = value; }
        }
        private float insertionAlt = 100000;
        private TransferType transfer;

        

        public CFG_blue(LazorSystem system):base(system) {
            //track = GameObject.Find("Tracking Station").collider;
            //track = LazorGlobals.trackingStation;
            name = "Tracking Station Linkup";
            color = Color.blue;
            colorTexture = Util.LightTexture(Util.getTexture(new Color(color.r, color.g, color.b, 0.9f)), 0.6f);
        }

        // Scan attached lazor list and pick the first and best grouped or single lazor to do the job.
        override public bool Detect(Dictionary<int, List<LazorAimable>> lazors) {
            claimedLazors = StdDetect(Color.blue, lazors);
            return claimedLazors.Count > 0;
        }

        override public void ExecuteCommand(LazorAimable lazor) {
			if(lazor != null) {
				if(LazorGlobals.trackingStation == null)
					lazor.Lazor.Aim(LazorGlobals.GetBody("Kerbin").transform.position - lazor.Lazor.GetOrigin());
				else
					lazor.Lazor.Aim(LazorGlobals.trackingStation.transform.position - lazor.Lazor.GetOrigin());
			}
        }

        override public void OnEnabledFixedUpdate() {
            system.RequestEnergyDelta(0.15d);
			if (system.IsActiveLazorSystem())
                TransferCalculator();
        }
        public override void OnEnable() {
            //LazorGlobals.Init();
			system.localEvents.onSoIChange += OnSoIChange;
            OnSoIChange();

            //KSP targets
            /*Collider tmp = GameObject.Find("Tracking Station").collider;
            system.targetList.AddReplacement(new GUItargetsListItem("" + tmp.GetInstanceID(), TargetType.KSC, tmp.name, tmp, 0, 0));
            tmp = GameObject.Find("Launchpad Platform").collider;
            system.targetList.AddReplacement(new GUItargetsListItem("" + tmp.GetInstanceID(), TargetType.KSC, tmp.name, tmp, 0, 0));
            tmp = GameObject.Find("Vehicle Assembly Building").collider;
            system.targetList.AddReplacement(new GUItargetsListItem("" + tmp.GetInstanceID(), TargetType.KSC, tmp.name, tmp, 0, 0));
            tmp = GameObject.Find("SPH").collider;
            system.targetList.AddReplacement(new GUItargetsListItem("" + tmp.GetInstanceID(), TargetType.KSC, tmp.name, tmp, 0, 0));*/
            // Celestial bodies category on the list
            foreach (CelestialBody cb in FlightGlobals.Bodies)
                system.targetList.AddReplacement(new GUItargetsListItem("" + cb.GetInstanceID(), TargetType.Celestial_Body, cb.name, cb, 0, 0));
            // Vessels category on the list
            //foreach (Vessel v in FlightGlobals.Vessels)
            //    system.targetList.AddReplacement(new GUItargetsListItem("" + v.id, TargetType.Vessel, v.vesselName, v, 0, 0));//, ButtonSefDestruct));
        }
        public override void OnDisable() {
			system.localEvents.onSoIChange -= OnSoIChange;
            toggleBlue_all = false;
        }

        // The Button placed on the main GUI. Opens Gui and starts the lazors
        override public void DrawGUIbutton() {
            enabled.set = system.guiSubSystems.SubsystemButton(enabled, name, color, out docked, ContentGUI, 0);
        }

        private enum TransferType {
            PP = 1, //planet to planet, or moon to moon         | phaseangle + depangle
            PM = 2, //from planet to moon, or star to planet    | depangle
            MP = 3, //opposite of PM                            | depangle                  | insertion altitude txtfield
        }

        override public void ContentGUI(int windowID) {
            docked = system.guiSubSystems.Header(docked, rect, enabled, ref enabled);

            GUILayout.Label("Transfer: ");
            GUILayout.BeginHorizontal();
            if (GUILayout.Button("◄", Util.Buttons_arrows_mainPanel, GUILayout.Width(22))) {
                transferQueue.AddFirst(transfer_target);
                transfer_target = transferQueue.Last.Value;
                transferQueue.RemoveLast();
                SelectTransfer();
            }
            GUILayout.BeginVertical(Util.TxtArea);
            GUILayout.Label(transfer_target.name, Util.StdLabelSmallCustom(Color.green, TextAnchor.MiddleCenter), GUILayout.Height(17));
            GUILayout.EndVertical();
            if (GUILayout.Button("►", Util.Buttons_arrows_mainPanel, GUILayout.Width(22))) {
                transferQueue.AddLast(transfer_target);
                transfer_target = transferQueue.First.Value;
                transferQueue.RemoveFirst();
                SelectTransfer();
            }
            GUILayout.EndHorizontal();

            GUILayout.BeginVertical(Util.Space_Compact);
            if (transfer == TransferType.MP) { //going into the reference planet
                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("Insertion alt.: ", Util.StdLabelSmallCustom(Color.green));
                insertionAlt = float.Parse(GUILayout.TextField(insertionAlt.ToString("f0")).Replace("\n", "").Replace("\r", ""));
                GUILayout.EndHorizontal();

                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("ToF: ", Util.StdLabelSmallCustom(Color.green));
                GUILayout.Label(Util.SecondsAsTimeString(ToF), Util.StdLabelSmallCustom(Color.white));
                GUILayout.EndHorizontal();

                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("Angle: ", Util.StdLabelSmallCustom(Color.yellow));
                GUILayout.Label(angle.ToString("f0"), Util.StdLabelSmallCustom(Color.white));
                GUILayout.Label("°", Util.StdLabelSmallCustom(Color.yellow));
                GUILayout.EndHorizontal();

                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("Eject angle: ", Util.StdLabelSmallCustom(Color.yellow));
                GUILayout.Label(angleD.ToString("f2"), Util.StdLabelSmallCustom(Color.white));
                GUILayout.Label("°", Util.StdLabelSmallCustom(Color.yellow));
                GUILayout.EndHorizontal();

                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("V Eject: ", Util.StdLabelSmallCustom(Color.yellow));
                GUILayout.Label(ejectV.ToString("f0"), Util.StdLabelSmallCustom(Color.white));
                GUILayout.Label("m/s", Util.StdLabelSmallCustom(Color.yellow));
                GUILayout.EndHorizontal();

            } else if (transfer == TransferType.PM) {

                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("ToF: ", Util.StdLabelSmallCustom(Color.red));
                GUILayout.Label(Util.SecondsAsTimeString(ToF), Util.StdLabelSmallCustom(Color.white));
                GUILayout.EndHorizontal();

                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("Phase angle: ", Util.StdLabelSmallCustom(Color.red));
                GUILayout.Label(phase.ToString("f2"), Util.StdLabelSmallCustom(Color.white));
                GUILayout.Label("°", Util.StdLabelSmallCustom(Color.red));
                GUILayout.EndHorizontal();

                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("Departure median: ", Util.StdLabelSmallCustom(new Color(0.2f, 0.2f, 1f, 1f)));
                GUILayout.Label(phaseD.ToString("f2"), Util.StdLabelSmallCustom(Color.white));
                GUILayout.Label("°", Util.StdLabelSmallCustom(new Color(0.2f, 0.2f, 1f, 1f)));
                GUILayout.EndHorizontal();

                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("Departure angle: ", Util.StdLabelSmallCustom(Color.cyan));
                GUILayout.Label(phaseD2.ToString("f2"), Util.StdLabelSmallCustom(Color.white));
                GUILayout.Label("°", Util.StdLabelSmallCustom(Color.cyan));
                GUILayout.EndHorizontal();

                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("ΔV Departure: ", Util.StdLabelSmallCustom(new Color(0.2f, 0.2f, 1f, 1f)));
                GUILayout.Label(deltaVA.ToString("f0"), Util.StdLabelSmallCustom(Color.white));
                GUILayout.Label("m/s", Util.StdLabelSmallCustom(new Color(0.2f, 0.2f, 1f, 1f)));
                GUILayout.EndHorizontal();

                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("V Departure: ", Util.StdLabelSmallCustom(new Color(0.2f, 0.2f, 1f, 1f)));
                GUILayout.Label(VtxA.ToString("f0"), Util.StdLabelSmallCustom(Color.white));
                GUILayout.Label("m/s", Util.StdLabelSmallCustom(new Color(0.2f, 0.2f, 1f, 1f)));
                GUILayout.EndHorizontal();

            } else if (transfer == TransferType.PP) {

                if (system.vessel.mainBody.orbit != null) { //orbit is null when SUN
                    GUILayout.BeginVertical(Util.Parameters_normal);
                    GUILayout.BeginHorizontal(Util.Space_Compact);
                    GUILayout.Label("Departure Window : ", Util.StdLabelSmallCustom(Color.red));
                    GUILayout.EndHorizontal();

                    GUILayout.BeginHorizontal(Util.Space_Compact);
                    GUILayout.Label("T- ", Util.StdLabelSmallCustom(Color.red));
                    GUILayout.Label(Util.SecondsAsTimeString(window), Util.StdLabelSmallCustom(Color.white));
                    GUILayout.EndHorizontal();
                    GUILayout.EndVertical();
                }

                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("ToF: ", Util.StdLabelSmallCustom(Color.red));
                GUILayout.Label(Util.SecondsAsTimeString(ToF), Util.StdLabelSmallCustom(Color.white));
                GUILayout.EndHorizontal();

                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("Phase angle: ", Util.StdLabelSmallCustom(Color.red));
                GUILayout.Label(phase.ToString("f2"), Util.StdLabelSmallCustom(Color.white));
                GUILayout.Label("°", Util.StdLabelSmallCustom(Color.red));
                GUILayout.EndHorizontal();

                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("Departure median: ", Util.StdLabelSmallCustom(new Color(0.2f, 0.2f, 1f, 1f)));
                GUILayout.Label(phaseD.ToString("f2"), Util.StdLabelSmallCustom(Color.white));
                GUILayout.Label("°", Util.StdLabelSmallCustom(new Color(0.2f, 0.2f, 1f, 1f)));
                GUILayout.EndHorizontal();

                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("Departure angle: ", Util.StdLabelSmallCustom(Color.cyan));
                GUILayout.Label(phaseD2.ToString("f2"), Util.StdLabelSmallCustom(Color.white));
                GUILayout.Label("°", Util.StdLabelSmallCustom(Color.cyan));
                GUILayout.EndHorizontal();

                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("Angle: ", Util.StdLabelSmallCustom(Color.yellow));
                GUILayout.Label(angle.ToString("f0"), Util.StdLabelSmallCustom(Color.white));
                GUILayout.Label("°", Util.StdLabelSmallCustom(Color.yellow));
                GUILayout.EndHorizontal();

                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("Eject Angle: ", Util.StdLabelSmallCustom(Color.yellow));
                GUILayout.Label(angleD.ToString("f0"), Util.StdLabelSmallCustom(Color.white));
                GUILayout.Label("°", Util.StdLabelSmallCustom(Color.yellow));
                GUILayout.EndHorizontal();

                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("ΔV Eject: ", Util.StdLabelSmallCustom(Color.yellow));
                GUILayout.Label(ejectVdelta.ToString("f0"), Util.StdLabelSmallCustom(Color.white));
                GUILayout.Label("m/s", Util.StdLabelSmallCustom(Color.yellow));
                GUILayout.EndHorizontal();

                GUILayout.BeginHorizontal(Util.Parameters_normal);
                GUILayout.Label("V Eject: ", Util.StdLabelSmallCustom(Color.yellow));
                GUILayout.Label(ejectV.ToString("f0"), Util.StdLabelSmallCustom(Color.white));
                GUILayout.Label("m/s", Util.StdLabelSmallCustom(Color.yellow));
                GUILayout.EndHorizontal();
            }
            GUILayout.EndVertical();

            GUILayout.BeginHorizontal(Util.Space_Compact);
            toggleBlue_all = GUILayout.Toggle(toggleBlue_all, "Map overlay");
            GUILayout.EndHorizontal();
            if (!enabled)
                toggleBlue_all = false;

            system.guiSubSystems.Footer(docked, rect);
        }

        private void TransferCalculator() {
            pos_reference = transfer_reference.position;
            pos_target = transfer_target.position;
            if (transfer == TransferType.PM) {
                normal_target = Vector3.Cross(transfer_target.orbit.getPositionFromTrueAnomaly(Math.PI / 2) - transfer_target.position, transfer_target.orbit.getPositionFromTrueAnomaly(Math.PI) - transfer_target.position).normalized;
                normal_reference = Vector3.Cross(system.vessel.orbit.GetVel(), transfer_reference.position - system.vessel.transform.position).normalized;
                phase = Util.CalculatePhaseAngle(system.vessel, transfer_reference, transfer_target, normal_reference);
                pos_origin = system.vessel.transform.position;
                angle = -1;
            } else if (transfer == TransferType.PP) {
                normal_target = Vector3.Cross(transfer_reference.position - transfer_target.position, transfer_target.orbit.GetRelativeVel()).normalized;
                normal_reference = -Planetarium.up; //ok so apparently up is down.
                phase = Util.CalculatePhaseAngle(system.vessel.mainBody, transfer_reference, transfer_target, normal_reference); //phase angle in degrees
                pos_origin = system.vessel.mainBody.position;
                angle = Util.CalculatePhaseAngle(system.vessel, system.vessel.mainBody, transfer_reference, normal_reference);
            } else {
                normal_reference = -Planetarium.up; 
                phase = -1;
                angle = Util.CalculatePhaseAngle(system.vessel, system.vessel.mainBody, transfer_reference, normal_reference);
            }

            //GENERAL PARAMETERS
            Orbit rAo = system.vessel.orbit; //the vessel's orbit
            double rA = system.vessel.orbit.semiMajorAxis; //semi major axis of vessel (i.e average distance from radius)

            if (transfer != TransferType.PM &&  !LazorGlobals.IsStar(system.vessel.mainBody)){ //system.vessel.mainBody.orbit != null) {
                rA = system.vessel.mainBody.orbit.semiMajorAxis;
                rAo = system.vessel.mainBody.orbit;
            }
 
            double rB = transfer_target.orbit.semiMajorAxis; //semi major axis of destination
            double aTX = (rA + rB) / 2; //semi-major axis of transfer ellipse in meter
            double GM = transfer_reference.gravParameter; //gravitational parameter in m3/s2
            e = 1 - rA / aTX; //eccentricity of transfer ellipse
            
            /*double v = Math.Acos(Mathf.Clamp((float)((((aTX * (1 - (e * e))) / rB) - 1) / e), -1f, 1f)); //true anomaly at second burn in radians
            double E = Math.Atan2(Math.Sqrt(1 - e * e) * Math.Sin(v), e + Math.Cos(v)); //eccentric anomaly in radians
            double ToF = (E - e * Math.Sin(E)) * Math.Sqrt(aTX * aTX * aTX / GM); //time of flight in seconds
            double angularV = 360 / transfer_target.orbit.period; // angular frequency, degree per second
            phaseD = (float)Util.NormalizeAngle(((v * Mathf.Rad2Deg) - (angularV * ToF))); //departure phase angle
            double ViA = Math.Sqrt(GM / rA);
            double VfB = Math.Sqrt(GM / rB);
            double VtxA = Math.Sqrt(GM * (2 / rA - 1 / aTX));
            double VtxB = Math.Sqrt(GM * (2 / rB - 1 / aTX));
            deltaVA = VtxA - ViA;
            double deltaVB = VfB - VtxB;*/
            double ViA = Math.Sqrt(GM / rA); //mean orbital velocity
            VtxA = Math.Sqrt(GM * (2 / rA - 1 / aTX)); //required velocity
            if (transfer == TransferType.MP) {
                rB = insertionAlt + transfer_target.Radius;
                VtxA = Math.Sqrt(GM * (2 / rA - 1 / ((rA + rB) / 2))); //required velocity
                //VtxB = Math.Sqrt(GM * (2 / rB - 1 / ((rA + rB) / 2)));
            }
            ToF = Math.PI * Math.Sqrt(((rA + rB) * (rA + rB) * (rA + rB)) / (8 * GM)); //time of flight (i.e time in seconds to complete transfer ellipse)
            phaseD = (float)Util.NormalizeAngle(180 - (float)(Math.Sqrt(GM / rB) * (ToF / rB) * (180 / Math.PI))); //mean departure phase angle
            deltaVA = VtxA - ViA; //delta-V of transfer start

            //CALCULATE EJECT ANGLE
            double soi = system.vessel.mainBody.sphereOfInfluence; //vessel soi
            double oGM = system.vessel.mainBody.gravParameter; //local soi's grav parameter
            double parkR = (system.vessel.transform.position - system.vessel.mainBody.position).magnitude; //parking altitude
            ejectV = Math.Sqrt((parkR * (soi * deltaVA * deltaVA - 2 * oGM) + 2 * soi * oGM) / (parkR * soi)); //total velocity required for eject burn
            ejectVdelta = ejectV - Math.Sqrt(oGM / parkR); //delta-V of eject burn
            double epsilon = Math.Abs(ejectV * ejectV / 2 - oGM / parkR);
            double h = parkR * ejectV;
            double eF = Math.Sqrt(1 + ((2 * epsilon * h * h) / (oGM * oGM)));
            angleD = (float)Util.NormalizeAngle(270 - Math.Acos(1 / eF) * Mathf.Rad2Deg); //eject angle
            if (rA > rB)
                angleD = (float)Util.NormalizeAngle(angleD - 180); //eject angle

            //take ECCENTRICITY into account when solving for phase angle
            double aT = Util.NormalizeAngle(phase - phaseD); //angle until estimated phase angle (extremely rough estimate, it will keep moving)
            double Te = transfer_target.orbit.eccentricity; //eccentricity of destination orbit

            //next departure window in seconds
            double nV = Math.Sqrt(GM / (rA * rA * rA)); //vessel mean motion
            double nT = Math.Sqrt(GM / (rB * rB * rB)); //target mean motion

            if (rA < rB) {
                double nM = nV - nT; //vessel motion relative to target in rad per sec
                window = (aT * Mathf.Deg2Rad) / nM;  //departure window.............
                double div;
                double Wt = Util.DoubleMod(window, transfer_target.orbit.period, out div); //time until phase alignment, not counting destination periods
                double M; //estimate of mean anomaly of destination at departure
                if (phase < phaseD)
                    M = transfer_target.orbit.meanAnomaly;
                else
                    M = transfer_target.orbit.meanAnomaly + (nT * Wt);
                double v = Util.NormalizeAngle((M + (2 * Te * Math.Sin(M)) + (1.25 * Te * Te * Math.Sin(2 * M))) * Mathf.Rad2Deg); //true anomaly at departure
                double Mi = M + (nT * ToF); //mean anomaly at intercept
                double vi = Util.NormalizeAngle((Mi + (2 * Te * Math.Sin(Mi)) + (1.25 * Te * Te * Math.Sin(2 * Mi))) * Mathf.Rad2Deg); //true anomaly at intercept
                phaseD2 = 180 - (float)Util.NormalizeAngle(vi - v); //estimated phase angle at intercept

            } else { //high to low
                aT = Util.NormalizeAngle(phaseD - phase);
                double nM = nT - nV; //target motion relative to vessel in rad per sec
                window = (aT * Mathf.Deg2Rad) / nM;  //departure window.............
                double div;
                double Wt = Util.DoubleMod(window, transfer_target.orbit.period, out div); //time until phase alignment, not counting destination periods
                double M; //estimate of mean anomaly of destination at departure
                if (phase > phaseD)
                    M = transfer_target.orbit.meanAnomaly;
                else
                    M = transfer_target.orbit.meanAnomaly + (nT * Wt);
                double v = Util.NormalizeAngle((M + (2 * Te * Math.Sin(M)) + (1.25 * Te * Te * Math.Sin(2 * M))) * Mathf.Rad2Deg); //true anomaly at departure
                double Mi = M + (nT * ToF); //mean anomaly at intercept
                double vi = Util.NormalizeAngle((Mi + (2 * Te * Math.Sin(Mi)) + (1.25 * Te * Te * Math.Sin(2 * Mi))) * Mathf.Rad2Deg); //true anomaly at intercept
                phaseD2 = (float)Util.NormalizeAngle(180 - (float)Util.NormalizeAngle(vi - v)); //estimated phase angle at intercept
            }

            /*double Tx = Math.Cos(v * Mathf.Deg2Rad);
            double E = Math.Acos((Te + Tx) / (1 + (Te * Tx)));
            double Me = E - (Te * Math.Sin(E)); //mean anomaly at eject
            double Mi = Me + (nT * ToF); //mean anomaly at intercept*/
            
            /*double To = transfer_target.orbit.trueAnomaly;
            double Te = Util.NormalizeAngle(To + aT); //true anomaly at eject
            double Tx = Math.Cos(Te * Mathf.Deg2Rad);
            double E = Math.Acos((eT + Tx) / (1 + (eT * Tx)));
            double Me = E - (eT * Math.Sin(E)); //mean anomaly at eject
            double n = Math.Sqrt(GM / (rB * rB * rB)); // velocity of target in rad per sec
            double M = Me + (n * ToF); //mean anomaly at intercept
            double v = M + (2 * eT * Math.Sin(M)) + (1.25 * eT * eT * Math.Sin(2 * M)); //true anomaly at intercept
            double T = Util.NormalizeAngle((Util.NormalizeAngle(v * Mathf.Rad2Deg) - Te)); //how many degrees the target will travel from Te*/

            /*double Mo = transfer_target.orbit.meanAnomaly;
            double M = Mo + Math.Sqrt(GM / (rB * rB * rB)) * ToF;
            double eT = transfer_target.orbit.eccentricity;
            double v = M + (2 * eT * Math.Sin(M)) + (1.25 * eT * eT * Math.Sin(2 * M)); //true anomaly at intercept
            double To = transfer_target.orbit.trueAnomaly;
            double T = v * Mathf.Rad2Deg - To; */


            /*MonoBehaviour.print(
                " rb" + rB.ToString("f2") +
                //" rb'" + (transfer_target.position - transfer_reference.position).magnitude.ToString("f2") +
                " rA" + rA.ToString("f2") +
                //" rA'" + (system.vessel.transform.position - transfer_reference.position).magnitude.ToString("f2") +
                //" n" + n.ToString("f9") +
                //" E" + E.ToString("f2") +
                //" M" + M.ToString("f2") +
                //" Mi" + Mi.ToString("f2") +
                //" " + Util.SecondsAsTimeString(t) +
                //" v" + v.ToString("f2") +
               // " vi" + vi.ToString("f2") +
                //" P " + phaseD2.ToString("f2") +
                //" Wt" + Util.SecondsAsTimeString(Wt) +
                "");*/

        }






        override public void Dispose() {
            //if(timer!=null)timer.Dispose();
            //if (timer2 != null) timer2.Dispose();
        }




        public void OnSoIChange() {
            if (system.vessel.mainBody != null) {
                transferBodies.Clear();
                transferQueue.Clear();
                if (LazorGlobals.IsStar(system.vessel.mainBody)) {
                    foreach (CelestialBody cb in system.vessel.mainBody.orbitingBodies) {
                        transferBodies.Add(cb);
                        transferQueue.AddLast(cb);
                    }
                } else if (LazorGlobals.IsPlanet(system.vessel.mainBody)) {
                    foreach (CelestialBody cb in system.vessel.mainBody.referenceBody.orbitingBodies) //iterate planets
                        if (cb != system.vessel.mainBody) {
                            transferBodies.Add(cb);
                            transferQueue.AddLast(cb);
                        }
                    foreach (CelestialBody cb in system.vessel.mainBody.orbitingBodies) { //iterate moons
                        transferBodies.Add(cb);
                        transferQueue.AddLast(cb);
                    }
                } else if (LazorGlobals.IsMoon(system.vessel.mainBody)) {
                    foreach (CelestialBody cb in system.vessel.mainBody.referenceBody.orbitingBodies) //iterate moons
                        if (cb != system.vessel.mainBody) {
                            transferBodies.Add(cb);
                            transferQueue.AddLast(cb);
                        }
                    transferBodies.Add(system.vessel.mainBody.referenceBody);
                    transferQueue.AddLast(system.vessel.mainBody.referenceBody);
                }
                transfer_target = transferQueue.First.Value;
                transferQueue.RemoveFirst();

                SelectTransfer();
            }
        }

        private void SelectTransfer() {
            transfer_reference = system.vessel.mainBody.referenceBody; //rule-p-p         //Planetarium.fetch.Sun; 
            transfer = TransferType.PP;
            if (LazorGlobals.IsPlanet(system.vessel.mainBody) && LazorGlobals.IsMoon(transfer_target) || //rule-p-m
                LazorGlobals.IsStar(system.vessel.mainBody) && LazorGlobals.IsPlanet(transfer_target)) {
                transfer_reference = system.vessel.mainBody;
                transfer = TransferType.PM;
            } else if (LazorGlobals.IsMoon(system.vessel.mainBody) && LazorGlobals.IsPlanet(transfer_target) || //rule-m-p    ...ref=target
                       LazorGlobals.IsPlanet(system.vessel.mainBody) && LazorGlobals.IsStar(transfer_target)) {
                transfer_reference = system.vessel.mainBody.referenceBody;
                transfer = TransferType.MP;
            }
        }

        override public void onFlightStateLoad(Dictionary<string, KSPParseable> parsedData) {
            onFlightStateLoadX(parsedData, "CFG_Blue");
        }

        override public void onFlightStateSave(Dictionary<string, KSPParseable> partDataCollection) {
            onFlightStateSaveX(partDataCollection, "CFG_Blue");
        }

    }
}


/*if (!didit) {
                MonoBehaviour.print("-------------------------------\n" + FlightGlobals.FindObjectsOfTypeIncludingAssets(typeof(Collider)).Length); //162   x
                MonoBehaviour.print("-------------------------------\n" + FlightGlobals.FindObjectsOfTypeIncludingAssets(typeof(GameObject)).Length); //6357
                MonoBehaviour.print("-------------------------------\n" + FlightGlobals.FindObjectsOfTypeIncludingAssets(typeof(Rigidbody)).Length); //14   x
                MonoBehaviour.print("-------------------------------\n" + FlightGlobals.FindObjectsOfTypeIncludingAssets(typeof(Part)).Length); //found 42  x
                MonoBehaviour.print("-------------------------------\n" + FlightGlobals.Bodies.Length); //found 3
                MonoBehaviour.print("-------------------------------\n" + FlightGlobals.Vessels); //found 1
                MonoBehaviour.print("-------------------------------\n" + Resources.FindObjectsOfTypeAll(typeof(Part)).Length); //found 42
                MonoBehaviour.print("-------------------------------\n" + GameObject.Find("Tracking Station")); //FOUND IT hahahaha
                MonoBehaviour.print("-------------------------------\n" + (GameObject.FindObjectsOfType(typeof(GameObject)) as GameObject[]).Count<GameObject>()); //found 1394
                didit = true;
            }*/



