﻿/* Author: Jim-Kyrre Benjaminsen AKA Romfarer
 * License: This work is licensed under a Creative Commons Attribution-NonCommercial-NoDerivs 3.0 Unported License. 
 * http://creativecommons.org/licenses/by-nc-nd/3.0/
*/

using System;
using System.Collections.Generic;
using System.Text;
using UnityEngine;

namespace Romfarer {
    class CFG_green : CFG {

        private enum ScanMode {Random = 0, H90V90=1, Horizon=2};
        private ScanMode scanMode = ScanMode.H90V90;


        public CFG_green(LazorSystem system) : base(system) {
            name = "LIDAR";
            color = Color.green;
            colorTexture = Util.LightTexture(Util.getTexture(new Color(color.r, color.g, color.b, 0.9f)), 0.6f);
        }

        override public bool Detect(Dictionary<int, List<LazorAimable>> lazors) {
            claimedLazors = StdDetectAll(color, lazors);
            return claimedLazors.Count > 0;
        }

        override public void OnEnabledFixedUpdate() {
            system.RequestEnergyDelta(0.15d);
        }

        // The Button placed on the main GUI. Opens Gui and starts the lazors
        override public void DrawGUIbutton() {
            enabled.set = system.guiSubSystems.SubsystemButton(enabled, name, color, out docked, ContentGUI, 0); //GUILayout.Toggle(enabled, "LIDAR", Util.StdStyle, GUILayout.ExpandWidth(true));
        }

        public override void OnEnable() {
            system.guiTargets.Enabled = true;
        }

        override public void ContentGUI(int windowID) {
            docked = system.guiSubSystems.Header(docked, rect, enabled, ref enabled); 
            scanMode = (ScanMode)GUILayout.SelectionGrid((int)scanMode, Enum.GetNames(typeof(ScanMode)), 3);
            system.guiSubSystems.Footer(docked, rect);
        }

        // What the lazors will do every frame.
        override public void ExecuteCommand(LazorAimable lazor) {
            //MonoBehaviour.print("x");
            switch (scanMode) {
                case ScanMode.Random:
                    lazor.Lazor.Aim(UnityEngine.Random.insideUnitSphere);
                    break;
                case ScanMode.H90V90:
					lazor.Lazor.Aim(Quaternion.LookRotation(lazor.transform.up, (system.vessel.rootPart.transform.position - system.vessel.mainBody.position).normalized) * Quaternion.Euler(90, 0, 0) *
				                Quaternion.Inverse(Quaternion.Euler(Mathf.Sin(Time.time * 2.5f) * Mathf.Rad2Deg, 0, Mathf.Sin(Time.time / 9) * Mathf.Rad2Deg)) * Vector3.up);
                    break;
                case ScanMode.Horizon:
                    //lazor.Lazor.GetTransform().rotation = Quaternion.LookRotation(lazor.Lazor.horizontalAim, (system.vessel.rootPart.transform.position - system.vessel.mainBody.position).normalized) * Quaternion.Euler(90, 0, 0) *
                    //                        Quaternion.Inverse(Quaternion.Euler(Mathf.Sin(Time.time / 9) / 9 * Mathf.Rad2Deg, 0, Mathf.Sin(Time.time * 2) * Mathf.Rad2Deg));
					lazor.Lazor.Aim(Quaternion.LookRotation(lazor.Lazor.horizontalAim, (system.vessel.rootPart.transform.position - system.vessel.mainBody.position).normalized) * Quaternion.Euler(90, 0, 0) *
				                Quaternion.Inverse(Quaternion.Euler(Mathf.Sin(Time.time / 9) / 9 * Mathf.Rad2Deg, 0, Mathf.Sin(Time.time * 2) * Mathf.Rad2Deg)) * Vector3.up);
					break;
            }
            system.AddToTargetList(lazor.Lazor);
        }

        public void ButtonScan(GUItargetsListItem i) {
        }

        override public void DrawGUIbuttonLeader() {
            base.DrawGUIbuttonLeader();                          
        }

        override public void onFlightStateLoad(Dictionary<string, KSPParseable> parsedData) {
            onFlightStateLoadX(parsedData, "CFG_Green");
            //enabled.set = true;
            //docked = false;
            //MonoBehaviour.print("CFG_Green docked load " + (parsedData["CFG_Green_docked"].value_bool ? "True" : "False") + "  " + (docked ? "True" : "False"));
            //MonoBehaviour.print("CFG_Green enabled load " + (parsedData["CFG_Green_enabled"].value_bool ? "True" : "False") + "  " + (enabled ? "True" : "False"));
        }

        override public void onFlightStateSave(Dictionary<string, KSPParseable> partDataCollection) {
            onFlightStateSaveX(partDataCollection, "CFG_Green");
            //MonoBehaviour.print("CFG_Green docked save " + (partDataCollection["CFG_Green_docked"].value_bool ? "True" : "False") + "  " + (docked ? "True" : "False"));
            //MonoBehaviour.print("CFG_Green enabled save " + (partDataCollection["CFG_Green_enabled"].value_bool ? "True" : "False") + "  " + (enabled ? "True" : "False"));
        }

    }
}
