﻿/* Author: Jim-Kyrre Benjaminsen AKA Romfarer
 * License: This work is licensed under a Creative Commons Attribution-NonCommercial-NoDerivs 3.0 Unported License. 
 * http://creativecommons.org/licenses/by-nc-nd/3.0/
*/

using System;
using System.Collections.Generic;
using System.Text;
using UnityEngine;

namespace Romfarer {
    class CFG_magenta : CFG {

		private MVSPanel mvs; //movement controller for claimed lazors
		private List<LazorAimable> lazorRefs = new List<LazorAimable>(); //lazors operated by this system, (actual lazors not groups as in claimedLazors)

		//private PIDclamp pidHover { get { return LazorGlobals.GLOBALCONFIG_pid_tractor_hover; }
		//	set { LazorGlobals.GLOBALCONFIG_pid_tractor_hover = value; } } //new PIDclamp("pid hover", 3f, 0.06f, 5);
		private PIDclamp pidHover = new PIDclamp("pid hover", 10f, 0.75f, 20);

        private PID pidLift = new PID(3f, 0.05f, 5);
        //private PID pidSpeed = new PID(3f, 0.05f, 5);
		private PIDclamp pidHover2 = new PIDclamp("pid hover 2", 2f, 0f, 0.3f);

        //auto
        private const float MAX_FORCE = 30;
        private float MAX_FORCE_CLAMP = 30;
        private Vector3 hoverGravityAdjust = Vector3.zero;
        private float forcePb = 0f;
        private float rangeAdjust = 1f;
        private float maxHoverForce = 0;
        private float hoverGravityAdjust_TotalForcePerLazor = 0;
        private UtilAverage display_forcePb = new UtilAverage(10);
        private UtilAverage display_forceTot = new UtilAverage(10);
        private DoubleSwitch toggleModeManual;
        private DoubleSwitch toggleModeHover;
        private DoubleSwitch toggleModeLift;
        private UtilAverage energyUseTotal  = new UtilAverage(25); //the total average amount of energy the beams are drawing
        private DoubleSwitch toggleStabilizor;
        private float stabilizor_max;
        private float stabilizor_min;
        private Vector3 stabilizorForceYield; //amount of force actually used by stabilizor

        //hover system
        private DoubleSwitch toggleHover;
        private float height = 5;
        private float hoverForce = 0;

        //lock system
        private DoubleSwitch toggleLift;
        private float heightLock = 5;
        private float lazorLiftForce = 0;
        private Vessel target;


        public CFG_magenta(LazorSystem system) : base(system) {
            name = "Tractor Beam";
            nameEVA = "EVA stabilizer";
            color = Color.magenta;
            colorTexture = Util.LightTexture(Util.getTexture(new Color(color.r, color.g, color.b, 0.9f)), 0.6f);
            hasEvaSystem = false;
            toggleModeManual.set = true;
        }

        public override void swapConfig() {
            if (configuredEVA) {
                docked = true;
                forcePb = 0f;
                toggleHover.set = false;
                enabledMain.set = false;
                toggleLift.set = false;
                target = null;
            } else
                enabledEVA.set = false;
        }

        public override void Dispose() {
            if(toggleHover)
                system.sysLock_HOVER = false;
        }

        override public bool Detect(Dictionary<int, List<LazorAimable>> lazors) {
            claimedLazors = StdDetectAll(color, lazors);
			lazorRefs.Clear();
			foreach (int i in claimedLazors) //show lazor beams
				lazorRefs.AddRange(lazors[i]); //system.config and -.LazorCounterparts not made yet
			mvs = new MVSPanel(new MVSnormal(system, lazorRefs));
            return claimedLazors.Count > 0;
        }

		override public bool ClaimLazors(int i) {
			lazorRefs.AddRange(system.config.LazorCounterparts[i]); 
			return base.ClaimLazors(i);
		}

		override public bool UnclaimLazors(int i) {
			lazorRefs.RemoveAll(x => system.config.LazorCounterparts[i].Contains(x)); 
			return base.UnclaimLazors(i);
		}

		override public void Reset() {
			mvs = new MVSPanel(new MVSnormal(system, lazorRefs));
		}

		override public void RemoveLazorRef(LazorAimable l) { 
			lazorRefs.Remove(l);
		}
	
        override public void DrawGUIbutton() {
            enabled.set = system.guiSubSystems.SubsystemButton(enabled, "Tractor Beam", color, out docked, ContentGUI, 0);
            if (!enabled)
                forcePb = 0;
        }

        override public void OnEnabledFixedUpdate() {
            system.sysLock_HOVER = toggleHover;
			mvs.MoveWithLeader(); //move assigned lazors after the leader in chase and lock mode
            if (toggleHover.In())
                system.VI.AddTask(HoverForcePerPart);
            if (toggleHover.Out()) {
                system.VI.RemoveTask(HoverForcePerPart);
                hoverForce = 0;
                hoverGravityAdjust_TotalForcePerLazor = 0;
                hoverGravityAdjust = Vector3.zero;
            }
            if (toggleModeHover) {
                double alt = system.vessel.GetHeightFromTerrain() == -1 ? system.vessel.orbit.altitude : system.vessel.GetHeightFromTerrain();
                if (alt > 1000) //calculate max range
                    rangeAdjust = Mathf.Clamp((5000 - (float)alt) / 4000, 0, 1); // 1000 to 2500 effective range
                else
                    rangeAdjust = 1;
            } else if (toggleModeLift) {
                if (system.Target != null && target != null) {
                    double alt = (target.transform.position - system.vessel.transform.position).magnitude;
                    if (alt > 1000) //calculate max range
                        rangeAdjust = Mathf.Clamp((5000 - (float)alt) / 4000, 0, 1); // 1000 to 2500 effective range
                } else
                    rangeAdjust = 1;
            }else if(!toggleModeHover && !toggleModeLift && !toggleModeManual)
                rangeAdjust = 1;

            if (!configuredEVA) {
                if (toggleHover) {

					float r = 0.5f;
					Vector3 g = FlightGlobals.getGeeForceAtPosition(system.vessel.transform.position);
					hoverGravityAdjust = -g * rangeAdjust * r; //the force to be added to each part, to make the craft float
					float cancelG = system.CRAFT_MASS * g.magnitude / claimedLazorsCount;
					hoverGravityAdjust_TotalForcePerLazor = Mathf.Clamp(r * rangeAdjust * cancelG, -MAX_FORCE_CLAMP, MAX_FORCE_CLAMP);
					maxHoverForce = MAX_FORCE_CLAMP - hoverGravityAdjust_TotalForcePerLazor;
					if (maxHoverForce < 0)
						maxHoverForce = 0;
					float pidAdjust = (system.CRAFT_MASS / (MAX_FORCE_CLAMP * claimedLazorsCount)) * 100;
					pidHover.ReinitializePIDsOnly(6 * pidAdjust, 1, 16 * pidAdjust);
					pidHover.Clamp(maxHoverForce);
					float dist = height - system.vessel.GetHeightFromTerrain();

					//float spd = 50 - (float)Math.Abs(system.vessel.verticalSpeed);
					//hoverForce = Mathf.Clamp(pidHover.UpdatePD((float)(system.CRAFT_MASS * spd * spd) / (2 * dist), Time.deltaTime, 7.5f) * rangeAdjust, -maxHoverForce, maxHoverForce);

					hoverForce = Mathf.Clamp(hoverGravityAdjust_TotalForcePerLazor + pidHover.UpdatePD(dist , Time.deltaTime, 7.5f) * rangeAdjust, -maxHoverForce, maxHoverForce);


					//clamp max velocity
					float breakingForce = 0;
					pidHover2.Clamp(maxHoverForce * 2);
					if(dist * system.vessel.verticalSpeed > 0) //the object is going towards target distance, speed should be clamped
						if(Math.Abs(system.vessel.verticalSpeed) > 20)
							if(dist * hoverForce > 0) //being dragged towards center
								breakingForce = pidHover2.Update(Mathf.Clamp(dist/4, -50, 50) - (float)system.vessel.verticalSpeed, Time.deltaTime);
					hoverForce = Mathf.Clamp(hoverForce + breakingForce, -maxHoverForce, maxHoverForce);


                    /*float r = 0.5f;
					Vector3 g = FlightGlobals.getGeeForceAtPosition(system.vessel.transform.position);
                    hoverGravityAdjust = -g * rangeAdjust * r; //the force to be added to each part, to make the craft float
					float cancelG = system.CRAFT_MASS * g.magnitude / claimedLazorsCount;
					hoverGravityAdjust_TotalForcePerLazor = Mathf.Clamp(r * rangeAdjust * cancelG, -MAX_FORCE_CLAMP, MAX_FORCE_CLAMP);
                    maxHoverForce = MAX_FORCE_CLAMP - hoverGravityAdjust_TotalForcePerLazor;
                    if (maxHoverForce < 0)
                        maxHoverForce = 0;
					float pidAdjust = (system.CRAFT_MASS / (MAX_FORCE_CLAMP * claimedLazorsCount)) * 100;
					pidHover.ReinitializePIDsOnly(8 * pidAdjust, 1, 16 * pidAdjust);
					pidHover.Clamp(maxHoverForce);
					float dist = (height - system.vessel.GetHeightFromTerrain());
					hoverForce = Mathf.Clamp(pidHover.UpdatePD(dist, Time.deltaTime, 5f) * rangeAdjust, -maxHoverForce, maxHoverForce);
					int mod = 1;
					if(dist * system.vessel.verticalSpeed > 0) //the object is going towards target distance, speed should be clamped
						if(Math.Abs(system.vessel.verticalSpeed) > 20)
							mod = -1;
					float breakingForce = 0;
					pidHover2.Clamp(maxHoverForce * 2);
					if(dist * hoverForce > 0  && mod < 0)
						breakingForce = pidHover2.Update(Mathf.Clamp(dist/4, -50, 50) - (float)system.vessel.verticalSpeed, Time.deltaTime);
					hoverForce = Mathf.Clamp(hoverForce + breakingForce, -maxHoverForce, maxHoverForce);
*/




                    if (toggleStabilizor) {
                        float torque = maxHoverForce - hoverForce;
                        torque = Mathf.Clamp(torque, stabilizor_min, stabilizor_max);
                        stabilizorForceYield = Stabilize(torque * claimedLazorsCount);
                    }
                } else if (toggleLift) {
                    float disp = heightLock - (target.transform.position - system.vessel.transform.position).magnitude;
                    lazorLiftForce = Mathf.Clamp(pidLift.Update(disp, Time.deltaTime), -MAX_FORCE_CLAMP, MAX_FORCE_CLAMP);
                } else {
                    if (toggleStabilizor) {
                        float torque = MAX_FORCE_CLAMP;
                        torque = Mathf.Clamp(torque, stabilizor_min, stabilizor_max);
                        stabilizorForceYield = Stabilize(torque * claimedLazorsCount);
                    }
                }
            }

			
			//force readouts
			if (toggleModeManual) {
				if (toggleStabilizor) {
					display_forcePb.Average(stabilizorForceYield.magnitude / claimedLazorsCount);
					display_forceTot.Average(stabilizorForceYield.magnitude);
				} else {
					display_forcePb.Average(forcePb);
					display_forceTot.Average(forcePb * claimedLazorsCount);
				}
			}else if (toggleModeHover) {
				float forcePerLazor = 0;
				forcePerLazor = hoverForce + hoverGravityAdjust_TotalForcePerLazor;
				//MonoBehaviour.print(hoverForce + " " + hoverGravityAdjust_TotalForcePerLazor);
				display_forcePb.Average(forcePerLazor);
				display_forceTot.Average(forcePerLazor * claimedLazorsCount);
			} else if (toggleModeLift) {
				display_forcePb.Average(lazorLiftForce);
				display_forceTot.Average(lazorLiftForce * claimedLazorsCount);
			} else {
				display_forcePb.Reset();
				display_forceTot.Reset();
			}

			//energy drain
			double d = system.RequestEnergyDelta(0.015d * Math.Abs(display_forceTot));// / Time.deltaTime;
            if(d != double.NaN)
                energyUseTotal.Average(d); //Calculate and draw average energy
            //MonoBehaviour.print(d + " " + Time.deltaTime);
        }


        //delegate added to vesseliterator to add force to all parts
        private void HoverForcePerPart(Part p) {
            if (p.physicalSignificance == Part.PhysicalSignificance.FULL && p.rigidbody != null)
                p.rigidbody.AddForce(hoverGravityAdjust * p.rigidbody.mass);
        }


        override public void ExecuteCommand(LazorAimable lazor) {
            if (lazor != null) {
                if (configuredEVA) {
                    if (enabledEVA)
                        if (system.Target != null)
                            lazor.Lazor.Aim(system.Target.transform.position - lazor.Lazor.GetOrigin());
                } else {
                    if (enabled) {
                        if (toggleHover) {
                            if (!lazor.Lazor.LazorIsHittingOwnVessel)
                                if (lazor.Lazor.LazorIsHittingSomething)
                                    lazor.rigidbody.AddForce(-lazor.Lazor.GetAim() * hoverForce);
                        } else if (toggleLift) {
                            lazor.Lazor.Aim(target.transform.position - lazor.Lazor.GetOrigin());
                            if (!lazor.Lazor.LazorIsHittingOwnVessel && lazor.Lazor.LazorIsHittingVessel) {
                                target.rigidbody.AddForce(lazor.Lazor.GetAim() * lazorLiftForce * rangeAdjust);
                                lazor.rigidbody.AddForce(-lazor.Lazor.GetAim() * lazorLiftForce * rangeAdjust);
                            }
                        } else {
                            if (!lazor.Lazor.LazorIsHittingOwnVessel || !system.SafeMode) {
                                if (!toggleStabilizor) {
                                    double alt = lazor.Lazor.HitDistance();
                                    if (alt > 1000) //calculate max range
                                        rangeAdjust = Mathf.Clamp((5000 - (float)alt) / 4000, 0, 1); // 1000 to 2500 effective range
                                    else
                                        rangeAdjust = 1;
                                    if (lazor.Lazor.LazorIsHittingVessel) {
                                        if (lazor.Lazor.GetHit().collider.attachedRigidbody != null) {
                                            Part hitPart = lazor.Lazor.GetHit().collider.attachedRigidbody.GetComponent<Part>();
                                            if (hitPart != null)
                                                hitPart.rigidbody.AddForce(lazor.Lazor.GetAim() * forcePb * rangeAdjust);
                                        }
                                    }
                                    if (lazor.Lazor.LazorIsHittingSomething)
                                        lazor.rigidbody.AddForce(-lazor.Lazor.GetAim() * forcePb * rangeAdjust);
                                }
                            }
                        }
                    }
                }
            }
        }

        override public void Inject_EVA() {
            if (configuredEVA) {
                if (system.Target != null && system.Target is Vessel && ((Vessel)system.Target).isEVA)
                    enabledEVA.set = GUILayout.Toggle(enabledEVA, "Stabilizer beam");
                else {
                    GUI.enabled = false;
                    enabledEVA.set = GUILayout.Toggle(enabledEVA, "Stabilizer beam");
                    GUI.enabled = true;
                }
                if (enabledEVA) {
                    if (system.Target != null) {
                        foreach (Part p in ((Vessel)system.Target).parts)
                            if (p.rigidbody != null && p.physicalSignificance == Part.PhysicalSignificance.FULL)
                                p.rigidbody.AddForce(p.rigidbody.mass * system.vessel.acceleration - p.rigidbody.mass * FlightGlobals.getGeeForceAtPosition(system.Target.transform.position) + (p.rigidbody.mass * (system.nonRotatingFrameVelocity - ((Vessel)system.Target).orbit.GetVel())));
                    }
                }
                enabled = enabledEVA;
            }
        }





		//*****************************************************
		// GUI
		//*****************************************************





        override public void ContentGUI(int windowID) {
            docked = system.guiSubSystems.Header(docked, rect, enabled, ref enabled);

            GUILayout.Space(4);


			mvs.GUIPanel();


            GUILayout.Label("Max Force: |" + MAX_FORCE_CLAMP.ToString("f1") + "| kN");
            MAX_FORCE_CLAMP = Mathf.Clamp(GUILayout.HorizontalSlider(MAX_FORCE_CLAMP, 0, MAX_FORCE), 0, MAX_FORCE);


            GUILayout.BeginHorizontal();
            toggleModeManual.set = GUILayout.Toggle(toggleModeManual, "Manual", Util.Button_Toggle);
            toggleModeHover.set = GUILayout.Toggle(toggleModeHover, "Hover", Util.Button_Toggle);
            toggleModeLift.set = GUILayout.Toggle(toggleModeLift, "Lift", Util.Button_Toggle);
            GUILayout.EndHorizontal();
            if (toggleModeManual.In()) {
                toggleModeHover.set = false;
                toggleModeLift.set = false;
            } else if (toggleModeManual.Out())
                forcePb = 0; //reset manual beam force
            if (toggleModeHover.In()) {
                toggleModeManual.set = false;
                toggleModeLift.set = false;
            } else if (toggleModeHover.Out())
                toggleHover.set = false;
            if (toggleModeLift.In()) {
                toggleModeManual.set = false;
                toggleModeHover.set = false;
            } else if (toggleModeLift.Out())
                toggleLift.set = false;
            if (toggleStabilizor.In()) {
                stabilizor_max = MAX_FORCE_CLAMP / 3f;
                stabilizor_min = 1;
            } else if (toggleStabilizor.Out())
                stabilizor_max = 0;
            if(!toggleModeManual && !toggleModeHover)
                toggleStabilizor.set = false;

            GUILayout.Space(3);

            //MANUAL:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
            if (toggleModeManual) {
                GUILayout.Label("Force: " + forcePb.ToString("f1") + " kN" + (toggleStabilizor ? " (Lock)" : ""));
                forcePb = Mathf.Clamp(GUILayout.HorizontalSlider(forcePb, -MAX_FORCE_CLAMP, MAX_FORCE_CLAMP), -MAX_FORCE_CLAMP, MAX_FORCE_CLAMP);

                //STABILIZOR
                GUILayout.Space(5);
                toggleStabilizor.set = GUILayout.Toggle(toggleStabilizor.set, "STABILIZOR");
                if (toggleStabilizor) {
                    GUILayout.Space(5);
                    forcePb = 0;
                    GUILayout.Label("Max Force: |" + stabilizor_max.ToString("f1") + "| kN");
                    stabilizor_max = Mathf.Clamp(GUILayout.HorizontalSlider(stabilizor_max, 0, MAX_FORCE_CLAMP), 0, MAX_FORCE_CLAMP);
                    GUILayout.Label("Min Force: |" + stabilizor_min.ToString("f1") + "| kN");
                    stabilizor_min = Mathf.Clamp(GUILayout.HorizontalSlider(stabilizor_min, 0, stabilizor_max), 0, stabilizor_max);
                }
            } 
            //HOVER::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
            else if (toggleModeHover) {
                if (system.sysLock_HOVER && !toggleHover || toggleLift)
                    GUI.enabled = false;
                GUILayout.BeginHorizontal();
                toggleHover.set = GUILayout.Toggle(toggleHover.set, "Hover");
                
                if (!toggleHover)
                    height = system.vessel.GetHeightFromTerrain();
                height = float.Parse(GUILayout.TextField(height.ToString("f2")).Replace("\n", "").Replace("\r", ""));

                if (GUILayout.RepeatButton("▲", GUILayout.Width(30)))  //●▼▲
                    height += 0.05f;
                GUILayout.EndHorizontal();
                GUILayout.BeginHorizontal();
                GUILayout.FlexibleSpace();
                if (GUILayout.RepeatButton("▼", GUILayout.Width(30)))  //●▼▲
                    height -= 0.05f;
                GUILayout.EndHorizontal();
                GUI.enabled = true;

                //STABILIZOR
                GUILayout.Space(5);
                toggleStabilizor.set = GUILayout.Toggle(toggleStabilizor.set, "STABILIZOR");
                if (toggleStabilizor.In())
                    stabilizor_max = MAX_FORCE_CLAMP / 3f;
                if (toggleStabilizor.Out())
                    stabilizor_max = 0;
                if (toggleStabilizor) {
                    GUILayout.Space(5);
                    GUILayout.Label("Max Force: " + stabilizor_max.ToString("f1") + " kN");
                    stabilizor_max = Mathf.Clamp(GUILayout.HorizontalSlider(stabilizor_max, 0, MAX_FORCE_CLAMP), 0, MAX_FORCE_CLAMP);

                    GUILayout.Label("Min Force: " + stabilizor_min.ToString("f1") + " kN");
                    stabilizor_min = Mathf.Clamp(GUILayout.HorizontalSlider(stabilizor_min, 0, stabilizor_max), 0, stabilizor_max);
                }
            }
            //LIFT::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
            else if (toggleModeLift) {
                 //lift system
                GUILayout.Space(5);
                GUILayout.Label("Target: " + ((target != null && target is Vessel) ? target.name : system.Target!=null?system.Target.name:"Pending"));
                GUILayout.BeginHorizontal();
                if (toggleHover || system.Target == null || !(system.Target is Vessel))
                    GUI.enabled = false;
                toggleLift.set = GUILayout.Toggle(toggleLift, "Lift");
                if(toggleLift.In())
                    target = (Vessel)system.Target;
                else if(toggleLift.Out())
                    target = null;
                if (!toggleLift && system.Target != null)
                    heightLock = (system.Target.transform.position - system.vessel.transform.position).magnitude;
                heightLock = float.Parse(GUILayout.TextField(heightLock.ToString("f2")).Replace("\n", "").Replace("\r", ""));

                if (GUILayout.RepeatButton("▲", GUILayout.Width(30)))  //●▼▲
                    heightLock += 0.05f;
                GUILayout.EndHorizontal();
                GUILayout.BeginHorizontal();
                GUILayout.Label(((target != null && target is Vessel) ? "Lifting" : "Idle"));
                GUILayout.FlexibleSpace();
                if (GUILayout.RepeatButton("▼", GUILayout.Width(30)))  //●▼▲
                    heightLock -= 0.05f;
                GUILayout.EndHorizontal();

                GUI.enabled = true; 
            }

            GUILayout.Space(5);
            GUILayout.BeginVertical();
            GUILayout.BeginHorizontal(Util.Parameters_normal);
            GUILayout.Label("Force applied: ", Util.StdLabelSmallCustom(Color.magenta));
            GUILayout.EndHorizontal();

            GUILayout.BeginHorizontal(Util.Parameters_normal);
            GUILayout.Label("Per beam: ", Util.StdLabelSmallCustom(Color.magenta));
            GUILayout.Label((display_forcePb * rangeAdjust).ToString("f2") + " / " + (MAX_FORCE_CLAMP * rangeAdjust).ToString("f2") + " kN");
            GUILayout.EndHorizontal();

            GUILayout.BeginHorizontal(Util.Parameters_normal);
            GUILayout.Label("Total: ", Util.StdLabelSmallCustom(Color.magenta));
            GUILayout.Label((display_forceTot * rangeAdjust).ToString("f2") + " / " + (MAX_FORCE_CLAMP * claimedLazorsCount * rangeAdjust).ToString("f2") + " kN");
            GUILayout.EndHorizontal();

            GUILayout.BeginHorizontal(Util.Parameters_normal);
            GUILayout.Label("Efficiency: ", Util.StdLabelSmallCustom(Color.magenta));
            GUILayout.Label((rangeAdjust * 100).ToString("f2"));
            GUILayout.Label(" %", Util.StdLabelSmallCustom(Color.magenta));
            GUILayout.EndHorizontal();

            GUILayout.BeginHorizontal(Util.Parameters_normal);
            GUILayout.Label("Energy use: ", Util.StdLabelSmallCustom(Color.green));
            GUILayout.Label(energyUseTotal.value.ToString("f4"));
            GUILayout.Label(" U/s", Util.StdLabelSmallCustom(Color.green));
            GUILayout.EndHorizontal();
            GUILayout.EndVertical();
            GUILayout.Space(5);

            /*GUILayout.Label("Advanced Options:", Util.StdLabelSmall);
            GUILayout.BeginHorizontal();
            kp = float.Parse(GUILayout.TextField(kp.ToString("f4"), Util.StdTextFieldSmall, GUILayout.Width(55)).Replace("\n", "").Replace("\r", ""));
            ki = float.Parse(GUILayout.TextField(ki.ToString("f4"), Util.StdTextFieldSmall, GUILayout.Width(55)).Replace("\n", "").Replace("\r", ""));
            kd = float.Parse(GUILayout.TextField(kd.ToString("f4"), Util.StdTextFieldSmall, GUILayout.Width(55)).Replace("\n", "").Replace("\r", ""));
            GUILayout.EndHorizontal();
            if (GUILayout.Button("Reinitialize", Util.StdButtonSmall))
                pid.Reinitialize(kp, ki, kd);
            */

			/*
			#region options plane
			toggleAdvanced = GUILayout.Toggle(toggleAdvanced, "Advanced Controller Options");
			if (toggleAdvanced) {
				PIDclamp.DebugGuiPanel(pidHover);
				PIDclamp.DebugGuiPanel(pidHover2);
			}
			#endregion
*/
            system.guiSubSystems.Footer(docked, rect);
        }
		//private bool toggleAdvanced;




        private Vector3 Stabilize(float torque) {
            float needed_pitch = 0;
            float needed_yaw = 0;
            float needed_roll = 0;
            float stop_pitch = 0;
            float stop_yaw = 0;
            float stop_roll = 0;
            float yaw = 0;
            float pitch = 0;
            float roll = 0;

            Vector3 target_angle = /*system.vessel.ReferenceTransform.rotation **/ (system.vessel.transform.position - system.vessel.mainBody.position).normalized;
			//Vector3 av = system.vessel.angularVelocity; //angular velocity
			//Vector3 am = system.vessel.angularMomentum; //angular momentum
			//Vector3 av = system.vessel.rigidbody.angularVelocity; //angular velocity
			Vector3 av = Quaternion.Inverse(system.vessel.transform.rotation) * system.vessel.rigidbody.angularVelocity;
			/*Vector3 am = new Vector3(system.vessel.MOI.x * av.x, 
			                         system.vessel.MOI.y * av.y, 
			                         system.vessel.MOI.z * av.z);*/

            if (system.IsHorizontal) {
                needed_pitch = system.actualMark * Mathf.Deg2Rad;
                needed_roll = -system.actualRoll * Mathf.Deg2Rad;
                //stop_pitch = (system.vessel.angularVelocity.x * system.vessel.angularVelocity.x) / ((2 * torque * system.vessel.angularVelocity.x) / system.vessel.angularMomentum.x);
                //stop_roll = (system.vessel.angularVelocity.y * system.vessel.angularVelocity.y) / ((2 * torque * system.vessel.angularVelocity.y) / system.vessel.angularMomentum.y);
				stop_pitch = (system.vessel.angularVelocity.x * system.vessel.angularMomentum.x) / (2 * torque);
				stop_roll = (system.vessel.angularVelocity.y * system.vessel.angularMomentum.y) / (2 * torque);
				if (stop_pitch < Math.Abs(needed_pitch))
                    pitch = (needed_pitch - system.vessel.angularVelocity.x) * (system.vessel.MOI.x + torque);
                else
                    pitch = (-system.vessel.angularVelocity.x) * (system.vessel.MOI.x + torque);
                if (stop_roll < Math.Abs(needed_roll))
                    roll = (-needed_roll - system.vessel.angularVelocity.y) * (system.vessel.MOI.y + torque);
                else
                    roll = (-system.vessel.angularVelocity.y) * (system.vessel.MOI.y + torque);
            } else {
				Util.PitchYawAngle(system.vessel.transform, target_angle, out needed_pitch, out needed_yaw);
                needed_pitch *= Mathf.Deg2Rad;
                needed_yaw *= Mathf.Deg2Rad;
				stop_pitch = (av.x * av.x) / ((2 * torque) / system.vessel.MOI.x);
				stop_yaw = (av.z * av.z) / ((2 * torque) / system.vessel.MOI.z);
                //find the axis with the shortest path to 0 or |180|
                float p = 0;
                float y = 0;
                float a = Math.Abs(needed_pitch);
                float ai = Mathf.PI - a;
                float b = Math.Abs(needed_yaw);
                float bi = Mathf.PI - b;
                float min = Math.Min(Math.Min(a, ai), Math.Min(b, bi));
                if (min == a) {
                    p = needed_pitch;
                    y = needed_yaw;
                } else if (min == b) {
                    p = needed_pitch;
                    y = needed_yaw;
                } else if (min == ai) {
                    p = -needed_pitch;
                    y = needed_yaw;
                } else if (min == bi) {
                    p = needed_pitch;
                    y = -needed_yaw;
                }
                if (stop_yaw < b)
                    yaw = Mathf.Clamp((-y - av.z) * (torque), -torque, torque);
                else
                    yaw = Mathf.Clamp((-av.z) * (torque), -torque, torque);

                if (stop_pitch < a)
                    pitch = Mathf.Clamp((p - av.x) * (torque), -torque, torque);
                else
                    pitch = Mathf.Clamp((-av.x) * (torque), -torque, torque);

                if (!(Input.GetKey(GameSettings.ROLL_LEFT.primary) || Input.GetKey(GameSettings.ROLL_RIGHT.primary)))
                    roll = Mathf.Clamp(-av.y * (torque), -torque, torque);
            }

			system.vessel.transform.rigidbody.AddRelativeTorque(pitch, roll, yaw);
            return new Vector3(pitch, roll, yaw);
            //MonoBehaviour.print("t" + torque.ToString("f2") + " y" + yaw.ToString("f2") + " p" + pitch.ToString("f2") + " r" + roll.ToString("f2") + "  " + system.vessel.angularVelocity + "  " + system.vessel.MOI);
        }

        override public void onFlightStateLoad(Dictionary<string, KSPParseable> parsedData) {
            onFlightStateLoadX(parsedData, "CFG_Magenta");
        }

        override public void onFlightStateSave(Dictionary<string, KSPParseable> partDataCollection) {
            onFlightStateSaveX(partDataCollection, "CFG_Magenta");
        }

    }
}

