﻿/* Author: Jim-Kyrre Benjaminsen AKA Romfarer
 * License: This work is licensed under a Creative Commons Attribution-NonCommercial-NoDerivs 3.0 Unported License. 
 * http://creativecommons.org/licenses/by-nc-nd/3.0/
*/

using System;
using System.Collections.Generic;
using System.Text;
using UnityEngine;
using KSP.IO;
using System.Linq;


namespace Romfarer {
    class Datastructures { }

	public delegate R Funx<R>();
	public delegate R Funx<T, R>(T t);
	public delegate void Axtion();


    class GUItargetsList {

        public delegate void Buttons(GUItargetsListItem i);
        Dictionary<GUItargetsListItem, Buttons> tasks;
        public List<GUItargetsListItem> list;


        public GUItargetsList() {
            list = new List<GUItargetsListItem>();
            tasks = new Dictionary<GUItargetsListItem, Buttons>();
        }

        public void Add(GUItargetsListItem item) {
            list.Add(item);
        }

        //Add item if it doesn't exist in the list
        public bool AddExclusive(GUItargetsListItem item) {
            if (!list.Exists(u => u.id == item.id)) {
                Add(item);
                return true;
            } return false;
        }

        //Replace any occurance or add item to the list
        public void AddReplacement(GUItargetsListItem item) {
            list.RemoveAll(u => u.id == item.id);
            Add(item);
        }

        public void Remove(GUItargetsListItem item) {
            list.RemoveAll(u => u.id == item.id);
        }

        public List<GUItargetsListItem> GetByTargetType(TargetType t) {
            return list.FindAll(x => x.type == t);
        }

        public void AddTask(GUItargetsListItem tli, Buttons b) {
            if (tasks.ContainsKey(tli))
                MonoBehaviour.print("GUItargetsList, AddTask() task already contains: " + tli.name);
            else
                tasks.Add(tli, b);
        }

        public void ExecuteTasks() {
            foreach (GUItargetsListItem tli in tasks.Keys) {
                tasks[tli](tli);
            }
            tasks.Clear();
        }
    }




 

    enum TargetIndicatorType {
        Target = 1,
        Tracking = 2
    }

    enum TargetType {
        Vessel = 1,
        KSC = 2,
        Celestial_Body = 3,
        Object = 4,
        //Location = 5,
    }

    enum StandardDirections {
        FORWARD_NORTH = 1,
        RIGHT_EAST = 2,
        BACK_SOUTH = 3,
        LEFT_WEST = 4,
        UP = 5,
        DOWN = 6,
    }



    struct GUItargetsListItem {

        public string id;
        public TargetType type;
        public string name;
        public Component c;
        public double lat;
        public double lon;
        public GUItargetsList.Buttons[] buttons;

        public GUItargetsListItem(string id, TargetType type, string name, Component c, double lat, double lon, params GUItargetsList.Buttons[] buttons) {
            this.id = id;
            this.type = type;
            this.name = name;
            this.c = c;
            this.lat = lat;
            this.lon = lon;
            this.buttons = buttons;
        }
    }

    struct KPSmapLocation {

        public string id;
        public string name;
        public double lat;
        public double lon;
        public CelestialBody cb;

        public KPSmapLocation(string id, string name, double lat, double lon, CelestialBody cb) {
            this.id = id;
            this.name = name;
            this.lat = lat;
            this.lon = lon;
            this.cb = cb;
        }

        public KPSmapLocation(KPSmapLocation k) {
            this.id = k.id;
            this.name = k.name;
            this.lat = k.lat;
            this.lon = k.lon;
            this.cb = k.cb;
        }
    }
    
    

    

    struct DoubleSwitch {
        public bool set;
        public bool on;
        public DoubleSwitch(bool set, bool on) {
            this.set = set;
            this.on = on;
        }
        public bool In() {
            if (set && !on) {
                on = true;
                return true;
            }
            return false;
        }
        public bool Out() {
            if (!set && on) {
                on = false;
                return true;
            }
            return false;
        }

        public static implicit operator bool(DoubleSwitch a) {
            return a.set;
        }
    }

	class EnumToggleGroup<E> where E: IConvertible{
		E fallback;
		public EnumToggleGroup(E fallback){
			this.fallback = fallback;
		}
		public E SelectMode(E current, E test, string s, int width) {
			bool on = current.Equals(test);
			if (on) {
				on = GUILayout.Toggle(on, s, GUILayout.Width(width));
				if (!on)
					return fallback;
				return test;
			} else {
				on = GUILayout.Toggle(on, s, GUILayout.Width(width));
				if(on)
					return test;
			}
			return current;
		}
	}



	class AimingDevice : MonoBehaviour{
		private GameObject aimer;
		private float speedSlow;
		private float speedFast;
		public Quaternion rotation{ get { return aimer.transform.rotation; } }
		public Vector3 position {
			get { return aimer.transform.position; } 
			set { aimer.transform.position = value;}}
		public Vector3 forward{ get { return aimer.transform.forward; } }
		public Vector3 up{ get { return aimer.transform.up; } }
		public new Transform transform{ get { return aimer.transform; } }
		Transform ref_base;
		Transform ref_origin;
		private float deadZone;
		private bool fixedRotation;
		private Quaternion unFixedRotation;
		/*private LineRenderer lf;
		private LineRenderer lu;
		private LineRenderer lr;*/


		public AimingDevice(Transform ref_base, Transform ref_origin, Vector3 baseAim, Vector3 baseOrigin, 
		                    Vector3 baseNormal, float speedSlow, float speedFast, float deadZone = 55f){
			this.ref_base = ref_base;
			this.ref_origin = ref_origin;
			this.speedSlow = speedSlow;
			this.speedFast = speedFast;
			this.deadZone = deadZone;
			aimer = new GameObject("LazorAimingDevice");
			aimer.transform.parent = ref_base;
			aimer.transform.position = baseOrigin;
			aimer.transform.rotation = Quaternion.LookRotation(baseAim, baseNormal);

			/*lf = Util.GenericLazor(aimer.transform, Vector3.zero, 0.05f, Color.green);
			lf.SetPosition(1, aimer.transform.InverseTransformDirection(aimer.transform.forward) * 2);
			lu = Util.GenericLazor(aimer.transform, Vector3.zero, 0.05f, Color.red);
			lu.SetPosition(1, aimer.transform.InverseTransformDirection(aimer.transform.up) * 2);
			lr = Util.GenericLazor(aimer.transform, Vector3.zero, 0.05f, Color.blue);
			lr.SetPosition(1, aimer.transform.InverseTransformDirection(aimer.transform.right) * 2);*/
		}
		
		public bool CorrectDeadzone(){
			if(fixedRotation){
				//MonoBehaviour.print(Vector3.Angle(aimer.transform.forward, -ref_base.up));
				if(Vector3.Angle(aimer.transform.forward, -ref_base.up) < deadZone) {
					Vector3 up = aimer.transform.up;
					float amount = (deadZone - Vector3.Angle(aimer.transform.forward, -ref_base.up)) * Mathf.Deg2Rad;
					aimer.transform.forward = Vector3.RotateTowards(aimer.transform.forward, ref_base.up, amount, amount);
					aimer.transform.rotation = Quaternion.LookRotation(aimer.transform.forward, up);
					return true;
				}
			} else{ 
				aimer.transform.position = ref_origin.position;
				if(Vector3.Angle(unFixedRotation * Vector3.forward, -ref_base.up) < deadZone) {
					Vector3 up = unFixedRotation * Vector3. up;
					float amount = (deadZone - Vector3.Angle(unFixedRotation * Vector3.forward, -ref_base.up)) * Mathf.Deg2Rad;
					aimer.transform.forward = Vector3.RotateTowards(unFixedRotation * Vector3.forward, ref_base.up, amount, amount);
					aimer.transform.rotation = Quaternion.LookRotation(aimer.transform.forward, up);
					return true;
				}else{
					aimer.transform.rotation = unFixedRotation;
				}
			}
			return false;
		}

		public void SetLockDirection(Vector3 a, Vector3 up){
			unFixedRotation = Quaternion.LookRotation(a, up);
		}

		private float Td() {
			return FlightInputHandler.fetch.precisionMode ? speedSlow : speedFast;
		}

		public void Right(){
			aimer.transform.Rotate(Vector3.up, -Td());
			unFixedRotation = aimer.transform.rotation;
		}
		
		public void Left() {
			aimer.transform.Rotate(Vector3.up, Td());
			unFixedRotation = aimer.transform.rotation;
		}
		
		public void Up() {
			aimer.transform.Rotate(Vector3.right, -Td());
			unFixedRotation = aimer.transform.rotation;
		}
		
		public void Down() {
			aimer.transform.Rotate(Vector3.right, Td());
			unFixedRotation = aimer.transform.rotation;
		}

		public void RollRight() {
			aimer.transform.Rotate(Vector3.forward, Td());
			unFixedRotation = aimer.transform.rotation;
		}
		
		public void RollLeft() {
			aimer.transform.Rotate(Vector3.forward, -Td());
			unFixedRotation = aimer.transform.rotation;
		}
		
		public void Fix(bool fix){
			if(fixedRotation = fix) {
				aimer.transform.parent = ref_base;
				aimer.transform.position = ref_origin.position;
			} else {
				aimer.transform.parent = null;
				unFixedRotation = aimer.transform.rotation;
			}
		}

	}





















	//temp fix for pardmodule persistence
	/*static class PartmodulePersistenceFix{

		public static void Sync(){
			Dictionary<string, KSPParseable> data = new Dictionary<string, KSPParseable>();
			data.Add("EvaToVessel", new KSPParseable(Util.DictionaryToString(LazorGlobals.evaToVessel),  KSPParseable.Type.STRING)); //data needs to have the keys to be loaded
			Util.Load("persistent.data", ref data);
			MonoBehaviour.print ("syncing "+data.Count);
			if(data.ContainsKey("EvaToVessel")) {
				MonoBehaviour.print ("data1 "+data["EvaToVessel"].value);
				//LazorGlobals.evaToVessel = LazorGlobals.evaToVessel.Union(Util.StringToDictionary(data["EvaToVessel"].value)).ToDictionary(pair => pair.Key, pair => pair.Value);
				LazorGlobals.evaToVessel = Util.MergeStringDictionary(LazorGlobals.evaToVessel, Util.StringToDictionary(data["EvaToVessel"].value));
				MonoBehaviour.print ("data2 "+Util.DictionaryToString(LazorGlobals.evaToVessel));
			}
			IntegrityCheck();
		}
		
		private static void IntegrityCheck(){
			Dictionary<string, string> acc = new Dictionary<string, string>();
			foreach(KeyValuePair<string, string> pair in LazorGlobals.evaToVessel)
				if(FlightGlobals.Vessels.Exists(v => v.name == pair.Key) || FlightGlobals.Vessels.Exists(v => v.id.ToString() == pair.Value))
					acc.Add(pair.Key, pair.Value);
			LazorGlobals.evaToVessel = acc;
		}

		//LazorSystem flightstart
		public static void EvaluateEVAs(){
			foreach(Vessel v in FlightGlobals.Vessels) {
				//check LazorGlobals.evaToVessel
				if(v.isEVA && v.rootPart != null && !v.rootPart.Modules.Contains("LazorEVA")) { //restore partmodule from lazorglobals dictionary evaToVessel
					MonoBehaviour.print("[LazorSystem: Attempting to restore memory on " + v.vesselName + "]");
					if(LazorGlobals.evaToVessel.ContainsKey(v.vesselName)) {
						PartModule mem = v.rootPart.AddModule("LazorEVA");
						mem.Fields.SetValue("myVessel", LazorGlobals.evaToVessel[v.vesselName]);
						MonoBehaviour.print("[LazorSystem: Memory restored on " + v.vesselName + "]");
					}
				}
			}
		}

		//RuntimeLogic
		public static void AddMemory(){
			if(!LazorGlobals.evaToVessel.ContainsKey(FlightGlobals.ActiveVessel.vesselName)) {
				LazorGlobals.evaToVessel.Add(FlightGlobals.ActiveVessel.vesselName, LazorGlobals.LastVesselWithLazor.id.ToString());
			}
		}

		//LazorEVA and LazorSystem
		public static void SaveEVA(){
			IntegrityCheck();
			Save();
		}

		public static void Save(){
			Dictionary<string, KSPParseable> data = new Dictionary<string, KSPParseable>();
			data.Add("EvaToVessel", new KSPParseable(Util.DictionaryToString(LazorGlobals.evaToVessel),  KSPParseable.Type.STRING));
			Util.Save("persistent.data", data);
		}

	}*/




}

