﻿/* Author: Jim-Kyrre Benjaminsen AKA Romfarer
 * License: This work is licensed under a Creative Commons Attribution-NonCommercial-NoDerivs 3.0 Unported License. 
 * http://creativecommons.org/licenses/by-nc-nd/3.0/
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Romfarer {
    static class Events {

        static public void Reset() {
            onVesselChange = delegate { };
            onKerbalKilled = delegate { };
            onVesselDestroyed = delegate { };
        }

        static public event OnVesselChange onVesselChange = delegate { };
        public delegate void OnVesselChange();
        static public void RegisterVesselChange(string from, string to) {
			RuntimeLogic.Instance.Log("[Romfarer.Events: Vessel changed from " + from + " to " + to + "]", true);
            onVesselChange();
        }

        static public event OnKerbalKilled onKerbalKilled = delegate { };
        public delegate void OnKerbalKilled();
        static public void RegisterKerbalKilled(string name) {
            MonoBehaviour.print("[Romfarer.Events: " + name + " was killed in action]");
            onKerbalKilled();
        }
        static public event OnVesselDestroyed onVesselDestroyed = delegate { };
        public delegate void OnVesselDestroyed();
        static public void RegisterVesselDestroyed(string name) {
            MonoBehaviour.print("[Romfarer.Events: " + name + " was destroyed]");
            onVesselDestroyed();
        }
        
    }


	class LocalEvents {
		//Vessel stage changed
		public event OnStageChange onStageChange = delegate { };
		public delegate void OnStageChange(VesselIteratorOwn vi);
		public void RegisterStageChange(VesselIteratorOwn vi){
			MonoBehaviour.print("[Romfarer.Events: Stage change on vessel " + vi.vessel.name + "]");
			onStageChange(vi);
		}

		//Situation changed
		public event OnSituationChange onSituationChange = delegate { };
		public delegate void OnSituationChange();
		public void RegisterSituationChange(string from, string to) {
			MonoBehaviour.print("Romfarer.Events: [Situation changed from " + from + " to " + to + "]");
			onSituationChange();
		}

		//SoI changed
		public event OnSoIChange onSoIChange = delegate { };
		public delegate void OnSoIChange();
		public void RegisterSoIChange(string from, string to) {
			MonoBehaviour.print("[Romfarer.Events: SoI changed from " + from + " to " + to + "]");
			onSoIChange();
		}


	}


}
