﻿/* Author: Jim-Kyrre Benjaminsen AKA Romfarer
 * License: This work is licensed under a Creative Commons Attribution-NonCommercial-NoDerivs 3.0 Unported License. 
 * http://creativecommons.org/licenses/by-nc-nd/3.0/
*/

using System;
using System.Collections.Generic;
using System.Text;
using UnityEngine;
using System.Linq;
//using RomfarerConnection;

namespace Romfarer {
    class GUIconfig : GUItab {

        private Vector2 scrollArea = Vector2.zero;
        private Rect area;
        private bool focusedLeader;
        public bool toggleIdentify;
        private Texture2D textureNormal = Util.getTexture(new Color(0.1f, 0.1f, 0.1f, 0.9f));
        private Texture2D textureHighlight = Util.getTexture(new Color(0.7f, 0.7f, 0.7f, 1));
        private Texture2D textureHover = Util.getTexture(new Color(0.2f, 0.2f, 0.2f, 0.9f));
        private GUIStyle expandButtons;
        private GUIStyle expandButtonsRight;
        private GUIStyle listItem;
        private Texture2D genericHeaderBg = Util.getTexture(new Color(0.4f, 0.4f, 0.4f, 1));

        //Queued tasks
        private delegate bool ClaimOperation(int i);
        private delegate bool ClaimOperation2(System.Object[] o);
        private Dictionary<int, ClaimOperation> taskClaimOperation = new Dictionary<int, ClaimOperation>();
        private Dictionary<System.Object[], ClaimOperation2> taskClaimOperation2 = new Dictionary<System.Object[], ClaimOperation2>();


        public GUIconfig(LazorSystem system) : base(system) {}

        override public string Name { get { return "Config"; } }
        override public string GetTabName() {
            return "Config";
        }

        override public void DrawGUI(string name) {
            GUI.skin = Util.LazorSkin;
            windowPos = GUILayout.Window(id, windowPos, WindowGUI, name, Util.Window_border);
            if (!docked)
                windowPos = Util.ClampToScreen(windowPos);
        }

        override public void WindowGUI(int windowID) {
            Header();
            if (expandButtons == null) {
                expandButtons = Util.Label(13, Color.white, Color.green, Color.clear);
                expandButtons.alignment = TextAnchor.UpperCenter;
            }
            if (expandButtonsRight == null) {
                expandButtonsRight = Util.Label(13, Color.white, Color.green, Color.clear);
                expandButtonsRight.alignment = TextAnchor.UpperRight;
            }
            if (listItem == null) {
                listItem = new GUIStyle(Util.ListItem_std);
                listItem.normal.background = textureNormal;
                listItem.hover.background = textureHover;
            }
            //if (GUILayout.Button("Reset to default", Util.StdButtonSmall)) { }

            ///// LIST START /////
            float height = area.height > 400 ? 410 : area.height + 10;
            scrollArea = GUILayout.BeginScrollView(scrollArea, false, false, GUILayout.MinHeight(height), GUILayout.ExpandHeight(false));
            GUILayout.BeginVertical(GUILayout.MaxWidth(windowPos.width - 22));
            if (focusedLeader) { /////////////////// LEADER
                listItem.normal.background = system.GetLeaderTextureColor();
                listItem.hover.background = system.GetLeaderTextureColor();
            } else {
                listItem.normal.background = textureNormal;
                listItem.hover.background = textureHover;
            }
            GUILayout.BeginHorizontal();
            expandButtons.normal.background = system.GetLeaderTextureColor();
            GUILayout.Label("", expandButtons, GUILayout.Width(13));
            if (GUILayout.Button("Leader", listItem))
                focusedLeader = !focusedLeader;
            GUILayout.EndHorizontal();

            foreach (CFG cfg in system.config.cfgActive) { /////////////////// CFG's   title of each cfg, or its EVA system title
                GUILayout.BeginHorizontal();
                expandButtons.normal.background = cfg.ColorTexture;
                expandButtons.hover.background = cfg.ColorTexture;
                if (cfg.hasEvaSystem) {
                    if (GUILayout.Button("►", expandButtons, GUILayout.Width(15))) {
                        cfg.ConfiguredEVA = !cfg.ConfiguredEVA;
                    }
                }
                listItem.normal.background = cfg.ColorTexture;
                listItem.hover.background = cfg.ColorTexture;
                GUILayout.Label(cfg.ConfiguredEVA?cfg.nameEVA:cfg.name, listItem);
                GUILayout.EndHorizontal();
                foreach (int i in cfg.GetClaimedLazors()) { /////////////////// GROUPS  list of lazor groups assigned to each cfg
                    ListGroup(i, true);
                }
            }

            /////////////////// UNASSIGNED GROUPS
            if (system.config.unassignedLazorGroups.Count > 0) {
                listItem.normal.background = genericHeaderBg;
                listItem.hover.background = genericHeaderBg;
                GUILayout.Label("Unassigned Lazor Groups: " + system.config.unassignedLazorGroups.Count, listItem);
                foreach (int i in system.config.unassignedLazorGroups) { /////////////////// GROUPS
                    ListGroup(i, false);
                }
            }

            GUILayout.EndVertical();
            if (Event.current.type == EventType.Repaint)
                area = GUILayoutUtility.GetLastRect();
            GUILayout.EndScrollView();

            //Execute List Tasks
            foreach (int i in taskClaimOperation.Keys) {
                taskClaimOperation[i](i);
            }
            taskClaimOperation.Clear();
            foreach (System.Object[] o in taskClaimOperation2.Keys) {
                taskClaimOperation2[o](o);
            }
            taskClaimOperation2.Clear();

            ///// LIST END /////
            GUILayout.BeginVertical(Util.Space_Formatted);
            toggleIdentify = GUILayout.Toggle(toggleIdentify, "Identify Selection");
            GUILayout.Space(2);
            if (GUILayout.Button("Clear Selection")) {
                focusedLeader = false;
                system.config.focusedListGroups.Clear();
                system.config.focusedListItems.Clear();
            }

			//IMPACT/EXIT MARKERS

            GUIoverlays over = system.guiOverlays;
            GUILayout.BeginHorizontal();
            GUILayout.Label("Impact ", GUILayout.Width(50));
            over.toggleImpactDist = GUILayout.Toggle(over.toggleImpactDist, "Dist.");
            over.toggleImpactAngle = GUILayout.Toggle(over.toggleImpactAngle, "Angle");
            GUILayout.EndHorizontal();
            GUILayout.BeginHorizontal();
            GUILayout.Label("Exit ", GUILayout.Width(50));
            over.toggleExitDist = GUILayout.Toggle(over.toggleExitDist, "Dist.");
            over.toggleExitAngle = GUILayout.Toggle(over.toggleExitAngle, "Angle");
            GUILayout.EndHorizontal();

			//OPTIONS

            GUILayout.Space(5);
            GUILayout.Label("Local Options: ", Util.StdLabelSmallCustom(Color.yellow));
            system.guiOverlays.toggleKPSlocations = GUILayout.Toggle(system.guiOverlays.toggleKPSlocations, "Show KPS locations on map");
			system.guiOverlays.toggleHeading = GUILayout.Toggle(system.guiOverlays.toggleHeading, "Show vessel heading on map");
            GUILayout.EndVertical();

            Footer();
        }


        //GUI methods
        private void ListGroup(int i, bool assigned) {
			//MonoBehaviour.print(assigned);
            if (system.config.focusedListGroups.Contains(i)) { //color selected list items
                listItem.normal.background = system.config.LazorCounterparts[i][0].Lazor.textureColor;
                listItem.hover.background = system.config.LazorCounterparts[i][0].Lazor.textureColor;
            } else {
                listItem.normal.background = textureNormal;
                listItem.hover.background = textureHover;
            }
            GUILayout.BeginHorizontal();
            expandButtons.normal.background = system.config.LazorCounterparts[i][0].Lazor.textureColor;
            expandButtons.hover.background = system.config.LazorCounterparts[i][0].Lazor.textureColor;
            if (system.config.expandedListGroups.Contains(i)) {
                if (GUILayout.Button("-", expandButtons, GUILayout.Width(15)))
                    system.config.expandedListGroups.Remove(i);
            } else {
                if (GUILayout.Button("+", expandButtons, GUILayout.Width(15)))
                    system.config.expandedListGroups.Add(i);
            }
            GUILayout.Label("▣", Util.StdLabelSmallCustom(system.config.LazorCounterparts[i][0].Lazor.GetColor()));
            if (GUILayout.Button("Lazor Group " + i, listItem)) {
                if (system.config.focusedListGroups.Contains(i))
                    system.config.focusedListGroups.Remove(i);
                else {
                    system.config.focusedListGroups.Add(i);
                    system.config.focusedListItems.Clear();
                }
                foreach (LazorAimable l in system.config.LazorCounterparts[i])
                    system.config.Identify(l);
            }
            expandButtonsRight.normal.background = system.config.LazorCounterparts[i][0].Lazor.textureColor;
            expandButtonsRight.hover.background = system.config.LazorCounterparts[i][0].Lazor.textureColor;
            if (assigned) {
                if (GUILayout.Button("▼", expandButtonsRight, GUILayout.Width(15))){
                    taskClaimOperation.Add(i, system.config.TryRemove);
                }
            } else {
                if (system.config.focusedListGroups.Count == 1 && !system.config.focusedListGroups.Overlaps(system.config.unassignedLazorGroups) && system.config.LazorCounterparts[system.config.focusedListGroups.ToArray<int>()[0]][0].Lazor.GetColor() == system.config.LazorCounterparts[i][0].Lazor.GetColor()) {
                    if (GUILayout.Button("∈", expandButtonsRight, GUILayout.Width(15))) {
                        CFG cfg = null;
						foreach (CFG c in system.config.cfgActive)
                            if (c.GetClaimedLazors().Contains(system.config.focusedListGroups.ToArray<int>()[0])) {
                                cfg = c;
                                break;
                            }
                        taskClaimOperation2.Add(new System.Object[] { i, cfg }, system.config.TryAddToExistingSpecific);
                    }
                } else {
                    if (GUILayout.Button("★", expandButtonsRight, GUILayout.Width(15))) {
                        taskClaimOperation.Add(i, system.config.TryAddCFG);
                    }
                    if (GUILayout.Button("▲", expandButtonsRight, GUILayout.Width(15))) {
						if (system.config.cfgActive.FindAll(x => x.Claimable(system.config.LazorCounterparts[i][0].Lazor.GetColor())).Count == 1) {
                            taskClaimOperation.Add(i, system.config.TryAddToExisting);
						} else if (system.config.cfgActive.FindAll(x => x.Claimable(system.config.LazorCounterparts[i][0].Lazor.GetColor())).Count == 0) {
                            taskClaimOperation.Add(i, system.config.TryAddCFG);
                        }
                    }
                }
            }
            GUILayout.EndHorizontal();


            if (system.config.expandedListGroups.Contains(i)) {
                foreach (LazorAimable l in system.config.LazorCounterparts[i]) { /////////////////// GROUP ITEMS
                    if (system.config.focusedListItems.Contains(l.GetInstanceID())) {
                        listItem.normal.background = l.Lazor.textureColor;// textureHighlight;
                        listItem.hover.background = l.Lazor.textureColor;// textureHighlight;
                    } else {
                        listItem.normal.background = textureNormal;
                        listItem.hover.background = textureHover;
                    }
                    GUILayout.BeginHorizontal();
                    GUILayout.Label("", expandButtons, GUILayout.Width(15));
                    GUILayout.Label("●", Util.StdLabelSmallCustom(l.Lazor.GetColor()));
                    if (GUILayout.Button(l.GetInstanceID().ToString(), listItem)) {
                        if (system.config.focusedListItems.Contains(l.GetInstanceID()))
                            system.config.focusedListItems.Remove(l.GetInstanceID());
                        else {
                            system.config.focusedListItems.Add(l.GetInstanceID());
                            system.config.focusedListGroups.Clear();
                        }
                        system.config.Identify(system.config.lazorsById[l.GetInstanceID()]);
                    }
                    GUILayout.EndHorizontal();
                }
            }
        }

        public void onFlightStateLoad(Dictionary<string, KSPParseable> parsedData) {
            onFlightStateLoad(parsedData, "GUIconfig");
        }
        public void onFlightStateSave(Dictionary<string, KSPParseable> partDataCollection) {
            onFlightStateSave(partDataCollection, "GUIconfig");
        }
     


    }
}
