﻿/* Author: Jim-Kyrre Benjaminsen AKA Romfarer
 * License: This work is licensed under a Creative Commons Attribution-NonCommercial-NoDerivs 3.0 Unported License. 
 * http://creativecommons.org/licenses/by-nc-nd/3.0/
*/

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using UnityEngine;

namespace Romfarer {
    class GUImodules : GUItab{

        private bool fireSunbeam;
        private List<Config.ModuleInfo> modulesT2Active {
            get { return system.config.modulesT2Active; }
        }
        private List<Config.ModuleInfo> modulesT1Active {
            get { return system.config.modulesT1Active; }
        }
        private Dictionary<int, List<Part>> modulesCounterparts {
            get { return system.config.modulesCounterparts; }
        }
        private Dictionary<Part, Config.PidControllers> pids {
            get { return system.config.pids; }
        }
        /* Control modules ::::::::::::::::::::::::
         * LazorControl-t1 :...fixed............... sunbeam 
         * [KSPField] public bool underControl;                 // in, set by LazorSystem. Should be set to "false" before LazorSystem goes off-line, decoupled or destroyed.
         * [KSPField] public bool isAimed;                      // out; may be "false" when turret is processing "Aim" command (or if it cannot process it at all).
         * [KSPField] public Vector3 aimingOrigin;              // out; in part's Local Space.
         * [KSPEvent] public void Aim(BaseEventData data){}     // "data" contains Vector3 "vector" (in World Space), otherwise aim's dropped.
         * 
         * Behaviours: LazorTrigger, LazorTriggerContinous
         * 
         * LazorControl-t2 :...remote.............. lazor missile
         * [KSPField] public bool receiveControl;               // out
         * [KSPField] public float maxTorque;                   // out constant
         * [KSPField] public float maxThrust;                   // out constant
         * [KSPField] public float ejectionForce;               // out constant
         * [KSPField] public float burnTime;                    // out constant
         * public void AddTorque(BaseEventData data) {}         // "data" contains Vector3 "vector"
         * 
         * Behaviours: LazorWarhead
         * 
         * Behaviour Modules ::::::::::::::::::::::::
         * LazorWarhead :............................
         * [KSPField] public float blastradius;                 // out constant
         * [KSPEvent] public void Detonate(){}       remove and use trigger? warhead not behaviour but a property? 
         * 
         * LazorTrigger :............................
         * [KSPField] public bool triggerName;                  // out constant, the name of the trigger button in LazorSystem (Fire, Trigger, Eject, Launch etc.)
         * [KSPEvent] public void Trigger(){}                   // fire once.
         * 
         * LazorTriggerContinous :...................
         * [KSPField] public bool triggerName;                  // out constant, the name of the trigger button in LazorSystem (Fire, Trigger, On, Active etc.)
         * [KSPField] public bool triggerOn;                    // in, true = continous fire
         */

        public GUImodules(LazorSystem system) : base(system) {
            if (system.config.modulesT1Active.Count + system.config.modulesT2Active.Count == 0) //no modules on vessel?
                hasContent = false;
        }

        override public string Name { get { return "Modules"; } }
        override public string GetTabName() {
            return "Modules";
        }

        override public void DrawGUI(string name) {
            GUI.skin = Util.LazorSkin;
            windowPos = GUILayout.Window(id, windowPos, WindowGUI, name);
            if (!docked)
                windowPos = Util.ClampToScreen(windowPos);
        }

        
        override public void WindowGUI(int windowID) {
            Header();
            for (int i = 0; i < modulesT2Active.Count();i++) {  
                modulesT2Active[i].toggle = GUILayout.Toggle(modulesT2Active[i].toggle, modulesT2Active[i].title, Util.Button_bold, GUILayout.Height(26));
                if (modulesT2Active[i].toggle) {
                    if(modulesT2Active[i].behaviours.Contains("LazorWarhead")){
                        GUIsubsystems.DividerTop();
                        modulesT2Active[i].proximitydetonate = GUILayout.Toggle(modulesT2Active[i].proximitydetonate, "Proximity Detonation");
                        if (modulesT2Active[i].proximitydetonate) {
                            GUILayout.BeginHorizontal();
                            GUILayout.Label("Range: ");
                            modulesT2Active[i].proximitydetonate_range = Mathf.Clamp(float.Parse(GUILayout.TextField(modulesT2Active[i].proximitydetonate_range.ToString()).Replace("\n", "").Replace("\r", "").Replace(".", "")), 0, modulesT2Active[i].proximitydetonate_maxrange);
                            GUILayout.Label("Max= " + modulesT2Active[i].proximitydetonate_maxrange);
                            GUILayout.EndHorizontal();
                        }
                        GUIsubsystems.DividerBottom();
                    }
                }
            }
            for (int i = 0; i < modulesT1Active.Count(); i++) {
                modulesT1Active[i].toggle = GUILayout.Toggle(modulesT1Active[i].toggle, modulesT1Active[i].title, Util.Button_bold, GUILayout.Height(26));
                if (modulesT1Active[i].toggle) {
                    if (modulesT1Active[i].behaviours.Contains("LazorTriggerContinous")) {
                        GUIsubsystems.DividerTop();
                        modulesT1Active[i].triggerOn = GUILayout.RepeatButton(modulesT1Active[i].triggerName) || fireSunbeam;
                        system.SunbeamLock = GUILayout.Toggle(system.SunbeamLock, "Toggle MouseLook");
                        if (system.SunbeamLock)
                            GUILayout.Label("Hit hotkey \"1\" or \"2\" to exit this mode");
                        GUIsubsystems.DividerBottom();
                    }
                }
            }
            Footer();
        }

        
        public void ModuleTriggerOn(bool trigger){
            fireSunbeam = trigger;
        }

        override public void OnFixedUpdate() {
            //send messages to 3rd party modules
            //missiles
            for (int i = 0; i < modulesT2Active.Count(); i++) {
                Config.ModuleInfo mi = modulesT2Active[i];
                if (mi.toggle) {
                    foreach (int ii in mi.counterparts) {
                        foreach (Part mod in modulesCounterparts[ii]) {
							if (mod != null && !mod.packed && !mod.isConnected && mod.Modules["LazorControlT2"].Fields.GetValue<bool>("receiveControl")) {
								Vector3 torqueOut = Controllers.MissileControl(mod, system.Target, system.Leader.GetAim(), pids[mod].yaw, pids[mod].pitch, 
								                                               modulesT2Active[i].proximitydetonate
								                                               ?modulesT2Active[i].proximitydetonate_range
								                                               :0);
								BaseEventData data = new BaseEventData(BaseEventData.Sender.USER);
								data.Set<Vector3>("vector", torqueOut);
								mod.Modules.Send("AddTorque", data);
                            }
                        }
                    }
                }
            }
            //sunbeam
            for (int i = 0; i < modulesT1Active.Count(); i++) {
                if (modulesT1Active[i].toggle) {
                    foreach (int ii in modulesT1Active[i].counterparts) {
                        foreach (Part mod in modulesCounterparts[ii]) {
                            if (mod != null) {
                                mod.Modules["LazorControlT1"].Fields.SetValue<bool>("underControl", true);
                                BaseEventData data = new BaseEventData(BaseEventData.Sender.USER);
                                Vector3 aim = system.Leader.GetAim();
                                if (system.Target != null) {
                                    Vector3 a = system.Target.transform.position;
                                    Vector3 b = mod.transform.TransformPoint(mod.Modules["LazorControlT1"].Fields.GetValue<Vector3>("aimingOrigin"));
                                    aim = (a - b).normalized;
                                } else if (system.Leader.LazorIsHittingSomething) {
                                    Vector3 a = system.Leader.GetHitPoint();
                                    Vector3 b = mod.transform.TransformPoint(mod.Modules["LazorControlT1"].Fields.GetValue<Vector3>("aimingOrigin"));
                                    aim = (a - b).normalized;
                                }
                                data.Set<Vector3>("vector", aim);
                                mod.Modules.Send("Aim", data);

                                mod.Modules["LazorTriggerContinous"].Fields.SetValue<bool>("triggerOn", modulesT1Active[i].triggerOn); //FIRE!
                            }
                        }
                    }
                } else {
                    foreach (int ii in modulesT1Active[i].counterparts)
                        foreach (Part mod in modulesCounterparts[ii]) {
                            if (mod) {
                                mod.Modules["LazorControlT1"].Fields.SetValue<bool>("underControl", false);
                                mod.Modules["LazorTriggerContinous"].Fields.SetValue<bool>("triggerOn", false); //FIRE!
                            }
                        }
                }
            }
        }

        public void onFlightStateLoad(Dictionary<string, KSPParseable> parsedData) {
            onFlightStateLoad(parsedData, "GUImodules");
        }
        public void onFlightStateSave(Dictionary<string, KSPParseable> partDataCollection) {
            onFlightStateSave(partDataCollection, "GUImodules");
        }




    }
}

