/* Author: Jim-Kyrre Benjaminsen AKA Romfarer
 * License: This work is licensed under a Creative Commons Attribution-NonCommercial-NoDerivs 3.0 Unported License. 
 * http://creativecommons.org/licenses/by-nc-nd/3.0/
*/

using System;
using System.Collections.Generic;
using System.Text;
using UnityEngine;

namespace Romfarer {

    class GUIoverlays : MonoBehaviour, IDisposable {

        private LazorSystem system;
        private Texture2D targetIndicator;
        private Texture2D targetIndicatorTracker;
        private Texture2D targetIndicatorConnection;
        private Texture2D crossHair;
        private Texture2D crossHairGreen;
        private Texture2D crossHairBlue;
        private Texture2D crossHairBlueConnection;
        private Texture2D crossHairWhite;

        private Material material;
        private GUIStyle mapTextLabel;
        private GUIStyle mapTextLabelBlue;
        private GUIStyle mapTextLabelBlueSolid;
        private Vector3 posTarget; //pos, dir, p, d, g,
        private double posTargetLat, posTargetLon;
        private string lat, lat2, lon, lon2;
        public bool toggleLGF;
        public bool toggleLGFtarget;
        public bool toggleLGFnewBearing;
        private double newBearing, newDistance, newBearingTarget, newDistanceTarget;
        float sinT30 = (float)Math.Sin(30 * Mathf.Deg2Rad);
        float cosT30 = (float)Math.Cos(30 * Mathf.Deg2Rad);

        //from CFG_Red
        public bool toggleBlue_all;
        public double e = 0;
        public float phase = 0;
        public float phaseD = 0; //departure angle
        public float phaseD2 = 0; //departure angle Bob's estimate
        public float angle = 0;
        public float angleD = 0;
        public Vector3 pos_target;
        public Vector3 pos_reference;
        public Vector3 pos_origin;
        public Vector3 normal_reference;
        public Vector3 normal_target;

        public Dictionary<string, Vector3d> normals = new Dictionary<string, Vector3d>();

        //velocity arrows
        public bool toggleHeading = true;
        public bool toggleSpeedSrf;
        public bool toggleSpeedObt;
        public LineRenderer lineSpeedSrf;
        public Vector3 lineSpeedSrf_endpos;
        public LineRenderer lineSpeedObt;
        public Vector3 lineSpeedObt_endpos;

        //parameter toggle
        public DoubleSwitch toggleMapGrid;

        //impact and exit angles/distance
        public bool toggleImpactDist;
        public bool toggleImpactAngle;
        public bool toggleExitDist;
        public bool toggleExitAngle;

        //axis
        public DoubleSwitch toggleAxisLocal;
        public DoubleSwitch toggleAxisGlobal;
        public Dictionary<StandardDirections, LineRenderer> axisLocal = new Dictionary<StandardDirections, LineRenderer>();
        public Dictionary<StandardDirections, LineRenderer> axisGlobal = new Dictionary<StandardDirections, LineRenderer>();

        //mapselect
        private double latitudeDecimal = 0;
        private double longitudeDecimal = 0;
        private string latitudeSexagesimal = "?";
        private string longitudeSexagesimal = "?";
        public double latitudeDecimalF = 0;
        public double longitudeDecimalF = 0;
        public string latitudeSexagesimalF = "?";
        public string longitudeSexagesimalF = "?";

        //kps
        public bool toggleKPSlocations = true;


        public GUIoverlays(LazorSystem system) {
            this.system = system;
            targetIndicator = Util.crossHairTarget;
            targetIndicatorConnection = Util.crossHairTargetBlueConnection;
            targetIndicatorTracker = Util.crossHairTracker;
            crossHair = Util.crossHairAim;
            crossHairGreen = Util.crossHairAimGreen;
            crossHairBlue = Util.crossHairAimBlue;
            crossHairBlueConnection = Util.crossHairAimBlueConnection;
            crossHairWhite = Util.crossHairAimWhite;
            material = new Material(Shader.Find("Particles/Additive"));

            //surface velocity indicator
            lineSpeedSrf = Util.GenericLazor(system.transform, Vector3.zero, 0.05f, Color.yellow);
            lineSpeedSrf.SetPosition(1, system.vessel.transform.InverseTransformDirection(system.vessel.transform.forward) * 12);
            lineSpeedSrf.enabled = false;
            //orbit velocity indicator
            lineSpeedObt = Util.GenericLazor(system.transform, Vector3.zero, 0.05f, Color.blue);
            lineSpeedObt.SetPosition(1, system.vessel.transform.InverseTransformDirection(system.vessel.transform.forward) * 12);
            lineSpeedObt.enabled = false;

            //Setup axis overlays
            axisLocal.Add(StandardDirections.FORWARD_NORTH, Util.GenericLazor(system.transform, Vector3.zero, 0.05f, Color.blue));
            axisLocal[StandardDirections.FORWARD_NORTH].SetPosition(1, system.vessel.transform.InverseTransformDirection(system.vessel.transform.forward) * 10);
            axisLocal.Add(StandardDirections.RIGHT_EAST, Util.GenericLazor(axisLocal[StandardDirections.FORWARD_NORTH].transform, Vector3.zero, 0.05f, Color.red));
            axisLocal[StandardDirections.RIGHT_EAST].SetPosition(1, system.vessel.transform.InverseTransformDirection(system.vessel.transform.right) * 10);
            axisLocal.Add(StandardDirections.BACK_SOUTH, Util.GenericLazor(axisLocal[StandardDirections.FORWARD_NORTH].transform, Vector3.zero, 0.05f, Color.blue));
            axisLocal[StandardDirections.BACK_SOUTH].SetPosition(1, system.vessel.transform.InverseTransformDirection(-system.vessel.transform.forward) * 10);
            axisLocal.Add(StandardDirections.LEFT_WEST, Util.GenericLazor(axisLocal[StandardDirections.FORWARD_NORTH].transform, Vector3.zero, 0.05f, Color.red));
            axisLocal[StandardDirections.LEFT_WEST].SetPosition(1, system.vessel.transform.InverseTransformDirection(-system.vessel.transform.right) * 10);
            axisLocal.Add(StandardDirections.UP, Util.GenericLazor(axisLocal[StandardDirections.FORWARD_NORTH].transform, Vector3.zero, 0.05f, Color.green));
            axisLocal[StandardDirections.UP].SetPosition(1, system.vessel.transform.InverseTransformDirection(system.vessel.transform.up) * 10);
            axisLocal.Add(StandardDirections.DOWN, Util.GenericLazor(axisLocal[StandardDirections.FORWARD_NORTH].transform, Vector3.zero, 0.05f, Color.green));
            axisLocal[StandardDirections.DOWN].SetPosition(1, system.vessel.transform.InverseTransformDirection(-system.vessel.transform.up) * 10);
            foreach (LineRenderer l in axisLocal.Values)
                l.enabled = false;
            axisGlobal.Add(StandardDirections.FORWARD_NORTH, Util.GenericLazor(system.transform, Vector3.zero, 0.05f, new Color(0.3f, 0.3f, 1, 1)));
            axisGlobal[StandardDirections.FORWARD_NORTH].SetPosition(1, system.vessel.transform.InverseTransformDirection(system.vessel.transform.forward) * 11);
            axisGlobal.Add(StandardDirections.RIGHT_EAST, Util.GenericLazor(axisGlobal[StandardDirections.FORWARD_NORTH].transform, Vector3.zero, 0.05f, new Color(1, 0.3f, 0.3f, 1)));
            axisGlobal[StandardDirections.RIGHT_EAST].SetPosition(1, system.vessel.transform.InverseTransformDirection(system.vessel.transform.right) * 11);
            axisGlobal.Add(StandardDirections.BACK_SOUTH, Util.GenericLazor(axisGlobal[StandardDirections.FORWARD_NORTH].transform, Vector3.zero, 0.05f, new Color(0.3f, 0.3f, 1, 1)));
            axisGlobal[StandardDirections.BACK_SOUTH].SetPosition(1, system.vessel.transform.InverseTransformDirection(-system.vessel.transform.forward) * 11);
            axisGlobal.Add(StandardDirections.LEFT_WEST, Util.GenericLazor(axisGlobal[StandardDirections.FORWARD_NORTH].transform, Vector3.zero, 0.05f, new Color(1, 0.3f, 0.3f, 1)));
            axisGlobal[StandardDirections.LEFT_WEST].SetPosition(1, system.vessel.transform.InverseTransformDirection(-system.vessel.transform.right) * 11);
            axisGlobal.Add(StandardDirections.UP, Util.GenericLazor(axisGlobal[StandardDirections.FORWARD_NORTH].transform, Vector3.zero, 0.05f, new Color(0.3f, 1, 0.3f, 1)));
            axisGlobal[StandardDirections.UP].SetPosition(1, system.vessel.transform.InverseTransformDirection(system.vessel.transform.up) * 11);
            axisGlobal.Add(StandardDirections.DOWN, Util.GenericLazor(axisGlobal[StandardDirections.FORWARD_NORTH].transform, Vector3.zero, 0.05f, new Color(0.3f, 1, 0.3f, 1)));
            axisGlobal[StandardDirections.DOWN].SetPosition(1, system.vessel.transform.InverseTransformDirection(-system.vessel.transform.up) * 11);
            foreach (LineRenderer l in axisGlobal.Values)
                l.enabled = false;
        }

        public void OnFixedUpdate() {
            if (toggleSpeedSrf) {
                lineSpeedSrf.transform.position = system.vessel.transform.TransformPoint(system.vessel.findLocalCenterOfMass());
                lineSpeedSrf.transform.forward = system.rotatingFrameVelocity.normalized;
                lineSpeedSrf_endpos = lineSpeedSrf.transform.position + lineSpeedSrf.transform.forward * 12;
            }
            if (toggleSpeedObt) {
                lineSpeedObt.transform.position = system.vessel.transform.TransformPoint(system.vessel.findLocalCenterOfMass());
                lineSpeedObt.transform.forward = system.nonRotatingFrameVelocity.normalized;
                lineSpeedObt_endpos = lineSpeedObt.transform.position + lineSpeedObt.transform.forward * 12;
            }
            if (toggleAxisLocal.In())
                foreach (LineRenderer l in axisLocal.Values)
                    l.enabled = true;
            else if (toggleAxisLocal.Out())
                foreach (LineRenderer l in axisLocal.Values)
                    l.enabled = false;
            
            if (toggleAxisGlobal.In()) {
                foreach (LineRenderer l in axisGlobal.Values)
                    l.enabled = true;
            } else if (toggleAxisGlobal.Out()) {
                foreach (LineRenderer l in axisGlobal.Values)
                    l.enabled = false;
            }
            if (!MapView.MapIsEnabled) {
                if (toggleAxisGlobal.set) {
                    Vector3 pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(system.vectorNorth) * 11;
                    axisGlobal[StandardDirections.FORWARD_NORTH].transform.position = system.vessel.transform.TransformPoint(system.vessel.findLocalCenterOfMass());
                    axisGlobal[StandardDirections.FORWARD_NORTH].transform.LookAt(system.vessel.transform.TransformPoint(pos), system.vessel.upAxis);
                }
                if (toggleAxisLocal.set) {
                    axisLocal[StandardDirections.FORWARD_NORTH].transform.position = system.vessel.transform.TransformPoint(system.vessel.findLocalCenterOfMass());
                    if (system.IsHorizontal) {
                        Vector3 pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(system.vessel.transform.up) * 10;
                        axisLocal[StandardDirections.FORWARD_NORTH].transform.LookAt(system.vessel.transform.TransformPoint(pos), -system.vessel.transform.forward);
                    } else {
                        Vector3 pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(system.vessel.transform.forward) * 10;
                        axisLocal[StandardDirections.FORWARD_NORTH].transform.LookAt(system.vessel.transform.TransformPoint(pos), system.vessel.transform.up);
                    }
                }
            }

            //map grid
            if (toggleMapGrid.In())
                LazorGlobals.TogglePlanetOverlay(true); //LazorGlobals.GetMapFocusBody().name, 
            else if(toggleMapGrid.Out())
                LazorGlobals.TogglePlanetOverlay(false);
        }

        public void Off() {
            lineSpeedSrf.enabled = false;
            lineSpeedObt.enabled = false;
            toggleAxisLocal.set = false;
                foreach (LineRenderer l in axisLocal.Values)
                    l.enabled = false;

                toggleAxisGlobal.set = false;
                foreach (LineRenderer l in axisGlobal.Values)
                    l.enabled = false;

        }

        public void DrawErrorMessage(string message, int lines) {
            if(mapTextLabel==null)
                mapTextLabel = Util.Label(12, Color.white, Color.white, new Color(1, 0, 0, 0.35f));
            GUIContent ctnt = new GUIContent(message);
            float min, max;
            mapTextLabel.CalcMinMaxWidth(ctnt, out min, out max);
            GUI.Label(new Rect(Screen.width / 2 - max / 2, Screen.height / 3, max + 10, 18 * lines), ctnt, mapTextLabel);
        }

        public void DrawTargetIndicator(Vector3 position, TargetIndicatorType ct, string name) {
            if (!MapView.MapIsEnabled) {
                Texture2D tex = null;
                if (ct == TargetIndicatorType.Tracking)
                    tex = targetIndicatorTracker;
                else
                    if (system.Leader.GetColor() == Color.blue && system.Leader.LazorIsHittingVessel && !system.Leader.LazorIsHittingOwnVessel)
                        tex = targetIndicatorConnection;
                    else if (!system.Leader.LazorIsHittingOwnVessel)
                        tex = targetIndicator;
                if (tex != null) {
                    Vector3 p = Camera.main.WorldToScreenPoint(position);
                    p.y = Camera.main.pixelHeight - p.y;
                    if (p.z >= 0) {
                        GUI.DrawTexture(new Rect(p.x - tex.width / 2, p.y - tex.height / 2, tex.width, tex.height), tex);
                        GUIStyle l = Util.CrosshairLabel(Color.red);
                        GUIContent ctnt = new GUIContent(name);
                        float min, max;
                        l.CalcMinMaxWidth(ctnt, out min, out max);
                        float x = max / 2;
                        GUI.Label(new Rect(p.x - x, p.y - tex.height + 7, 200, 20), ctnt, l);
                    }
                }
            } else {
                Texture2D tex = null;
                if (ct == TargetIndicatorType.Tracking)
                    tex = targetIndicatorTracker;
                else
                    tex = targetIndicator;
                Vector3 p = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(position));
                p.y = Camera.main.pixelHeight - p.y;
                if (p.z >= 0) {
                    GUI.DrawTexture(new Rect(p.x - tex.width / 2, p.y - tex.height / 2, tex.width, tex.height), tex);
                    GUIStyle l = Util.CrosshairLabel(Color.red);
                    GUIContent ctnt = new GUIContent(name);
                    float min, max;
                    l.CalcMinMaxWidth(ctnt, out min, out max);
                    float x = max / 2;
                    GUI.Label(new Rect(p.x - x, p.y - tex.height + 7, 200, 20), ctnt, l);
                }
                
            }
        }

        public void DrawCrosshair() {
            //Vector2 m = Event.current.mousePosition;
            //system.mousePos = m;
			Vector2 m = system.mousePos;
            if (system.Leader.GetColor() == Color.green)
                GUI.DrawTexture(new Rect(m.x - crossHairGreen.width / 2, m.y - crossHairGreen.height / 2,
                            crossHairGreen.width, crossHairGreen.height), crossHairGreen);
            else if (system.Leader.GetColor() == Color.blue) {
                if (system.Leader.LazorIsHittingSomething) {
                    if (system.Leader.LazorIsHittingVessel) {
                        GUI.DrawTexture(new Rect(m.x - crossHairBlueConnection.width / 2, m.y - crossHairBlueConnection.height / 2,
                                                    crossHairBlueConnection.width, crossHairBlueConnection.height), crossHairBlueConnection);
                    } else
                        GUI.DrawTexture(new Rect(m.x - crossHairBlue.width / 2, m.y - crossHairBlue.height / 2,
                                                    crossHairBlue.width, crossHairBlue.height), crossHairBlue);
                } else
                    GUI.DrawTexture(new Rect(m.x - crossHairBlue.width / 2, m.y - crossHairBlue.height / 2,
                                                    crossHairBlue.width, crossHairBlue.height), crossHairBlue);

            } else if (system.Leader.GetColor() == Color.white)
                GUI.DrawTexture(new Rect(m.x - crossHairWhite.width / 2, m.y - crossHairWhite.height / 2,
                            crossHairWhite.width, crossHairWhite.height), crossHairWhite);
            else
                GUI.DrawTexture(new Rect(m.x - crossHair.width / 2, m.y - crossHair.height / 2,
                            crossHair.width, crossHair.height), crossHair);
        }

        public void DrawOverlayLazor(IAimableLazor l) {
            //IMPACT MARKS
            if (l.LazorIsHittingSomething && (toggleImpactDist || toggleImpactAngle)) {
                Vector3 p = Camera.main.WorldToScreenPoint(l.GetHitPoint());
                p.y = Camera.main.pixelHeight - p.y;
                if (p.z >= 0) {
                    GUI.DrawTexture(new Rect(p.x - l.TexLazorCam.width / 2, p.y - l.TexLazorCam.height / 2,
                                l.TexLazorCam.width, l.TexLazorCam.height), l.TexLazorCam);
                    GUIStyle cl = Util.CrosshairLabel(l.GetColor());
                    GUIContent ctnt = new GUIContent("");
                    if(toggleImpactDist && toggleImpactAngle)
                        ctnt = new GUIContent((l.HitDistance() < 10 ? l.HitDistance().ToString("f2") : l.HitDistance() < 100 ? l.HitDistance().ToString("f1") : l.HitDistance().ToString("f0")) + " / " + (Vector3.Angle(l.GetTransform().TransformDirection(l.GetHit().normal), -l.GetTransform().TransformDirection(l.GetAim())) * 2).ToString("f1"));
                    else if(toggleImpactDist)
                        ctnt = new GUIContent(l.HitDistance() < 10 ? l.HitDistance().ToString("f2") : l.HitDistance() < 100 ? l.HitDistance().ToString("f1") : l.HitDistance().ToString("f0"));
                    else
                        ctnt = new GUIContent((Vector3.Angle(l.GetTransform().TransformDirection(l.GetHit().normal), -l.GetTransform().TransformDirection(l.GetAim())) * 2).ToString("f1"));
                    float x = (cl.CalcScreenSize(cl.CalcSize(ctnt))).x / 2;
                    GUI.Label(new Rect(p.x - x, p.y - l.TexLazorCam.height * 2, 200, 20), ctnt, cl);
                }
            }
            //EXIT MARKS
            if (toggleExitDist || toggleExitAngle) {
                Vector3 p = Camera.main.WorldToScreenPoint(l.GetOrigin());
                p.y = Camera.main.pixelHeight - p.y;
                if (p.z >= 0) {
                    GUI.DrawTexture(new Rect(p.x - l.TexLazorCam.width / 2, p.y - l.TexLazorCam.height / 2,
                                l.TexLazorCam.width, l.TexLazorCam.height), l.TexLazorCam);
                    GUIStyle cl = Util.CrosshairLabel(l.GetColor());
                    GUIContent ctnt = new GUIContent("");
                    if (toggleExitDist && toggleExitAngle)
                        ctnt = new GUIContent((system.Leader.GetTransformBase().position - l.GetOrigin()).magnitude.ToString("f2") + " / " + Vector3.Angle(l.GetTransform().TransformDirection(l.GetTransformBase().up), l.GetTransform().TransformDirection(l.GetAim())).ToString("f1"));
                    else if (toggleExitDist)
                        ctnt = new GUIContent((system.Leader.GetTransformBase().position - l.GetOrigin()).magnitude.ToString("f2"));
                    else
                        ctnt = new GUIContent(Vector3.Angle(l.GetTransform().TransformDirection(l.GetTransformBase().up), l.GetTransform().TransformDirection(l.GetAim())).ToString("f1"));
                    float x = (cl.CalcScreenSize(cl.CalcSize(ctnt))).x / 2;
                    GUI.Label(new Rect(p.x - x, p.y - l.TexLazorCam.height * 2, 200, 20), ctnt, cl);
                }
            }
        }


        public void DrawOverlay() {
            if (!MapView.MapIsEnabled) {
                if (toggleAxisGlobal.set) {
                    Vector3 pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(system.vectorNorth) * 11;
                    system.guiOverlays.DrawTextAtWorldToScreenPos(system.vessel.transform.TransformPoint(pos), "N", new Color(0.3f, 0.3f, 1, 1));

                    pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(system.vectorEast) * 11;
                    system.guiOverlays.DrawTextAtWorldToScreenPos(system.vessel.transform.TransformPoint(pos), "E", new Color(1, 0.3f, 0.3f, 1));

                    pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(-system.vectorNorth) * 11;
                    system.guiOverlays.DrawTextAtWorldToScreenPos(system.vessel.transform.TransformPoint(pos), "S", new Color(0.3f, 0.3f, 1, 1));

                    pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(-system.vectorEast) * 11;
                    system.guiOverlays.DrawTextAtWorldToScreenPos(system.vessel.transform.TransformPoint(pos), "W", new Color(1, 0.3f, 0.3f, 1));

                    pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(system.vessel.upAxis) * 11;
                    system.guiOverlays.DrawTextAtWorldToScreenPos(system.vessel.transform.TransformPoint(pos), "Ug", new Color(0.3f, 1, 0.3f, 1));

                    pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(-system.vessel.upAxis) * 11;
                    system.guiOverlays.DrawTextAtWorldToScreenPos(system.vessel.transform.TransformPoint(pos), "Dg", new Color(0.3f, 1, 0.3f, 1));
                }
                if (toggleAxisLocal.set) {

                    if (system.IsHorizontal) {
                        Vector3 pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(system.vessel.transform.up) * 10;
                        system.guiOverlays.DrawTextAtWorldToScreenPos(system.vessel.transform.TransformPoint(pos), "F", Color.blue);

                        pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(system.vessel.transform.right) * 10;
                        system.guiOverlays.DrawTextAtWorldToScreenPos(system.vessel.transform.TransformPoint(pos), "R", Color.red);

                        pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(-system.vessel.transform.up) * 10;
                        system.guiOverlays.DrawTextAtWorldToScreenPos(system.vessel.transform.TransformPoint(pos), "B", Color.blue);

                        pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(-system.vessel.transform.right) * 10;
                        system.guiOverlays.DrawTextAtWorldToScreenPos(system.vessel.transform.TransformPoint(pos), "L", Color.red);

                        pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(-system.vessel.transform.forward) * 10;
                        system.guiOverlays.DrawTextAtWorldToScreenPos(system.vessel.transform.TransformPoint(pos), "Ul", Color.green);

                        pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(system.vessel.transform.forward) * 10;
                        system.guiOverlays.DrawTextAtWorldToScreenPos(system.vessel.transform.TransformPoint(pos), "Dl", Color.green);

                    } else {
                        Vector3 pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(system.vessel.transform.forward) * 10;
                        system.guiOverlays.DrawTextAtWorldToScreenPos(system.vessel.transform.TransformPoint(pos), "F", Color.blue);

                        pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(system.vessel.transform.right) * 10;
                        system.guiOverlays.DrawTextAtWorldToScreenPos(system.vessel.transform.TransformPoint(pos), "R", Color.red);

                        pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(-system.vessel.transform.forward) * 10;
                        system.guiOverlays.DrawTextAtWorldToScreenPos(system.vessel.transform.TransformPoint(pos), "B", Color.blue);

                        pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(-system.vessel.transform.right) * 10;
                        system.guiOverlays.DrawTextAtWorldToScreenPos(system.vessel.transform.TransformPoint(pos), "L", Color.red);

                        pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(system.vessel.transform.up) * 10;
                        system.guiOverlays.DrawTextAtWorldToScreenPos(system.vessel.transform.TransformPoint(pos), "Ul", Color.green);

                        pos = system.vessel.findLocalCenterOfMass() + system.vessel.transform.InverseTransformDirection(-system.vessel.transform.up) * 10;
                        system.guiOverlays.DrawTextAtWorldToScreenPos(system.vessel.transform.TransformPoint(pos), "Dl", Color.green);
                    }
                }

                //surface velocity indicator
                if (lineSpeedSrf.enabled = toggleSpeedSrf)
                    DrawTextAtWorldToScreenPos(lineSpeedSrf_endpos, "Vs", Color.yellow);

                //orbit velocity indicator
                if (lineSpeedObt.enabled = toggleSpeedObt) {
                    DrawTextAtWorldToScreenPos(lineSpeedObt_endpos, "Vo", Color.blue);
                }
            } else {
                //surface velocity indicator on map
                if (toggleSpeedSrf)
                    PaintArrow(system.rotatingFrameVelocity.normalized, 1, Color.yellow); //surface speed
                if (toggleSpeedObt)
                    PaintArrow(system.nonRotatingFrameVelocity.normalized, 1, Color.blue); //orbit speed
                if (toggleHeading)
                    PaintArrow(system.vessel.transform.up, 1, Color.green); //current bearing


            }
        }












        public void DrawTextAtWorldToScreenPos(Vector3 pos, string text, Color color){
            Vector3 p = Camera.main.WorldToScreenPoint(pos);
            GUI.Label(new Rect(p.x, Screen.height - p.y, 200, 20), text, Util.StdLabelSmallCustom(color, TextAnchor.MiddleLeft));
        }
     
        //called from CFG_red
        public void FeedLGF(string lat, string lat2, string lon, string lon2, double newBearing, double newDistance) {
            this.lat = lat;
            this.lat2 = lat2;
            this.lon = lon;
            this.lon2 = lon2;
            toggleLGF = true;
            this.newBearing = newBearing;
            this.newDistance = newDistance;
        }

        //tells the system to activate target circle ....called from CFG_red
        public void FeedLGFTarget(Vector3 pos, double lat, double lon){
            toggleLGFtarget = true;
            posTarget = pos;
            posTargetLat = lat;
            posTargetLon = lon;
            newBearingTarget = newBearing;
            newDistanceTarget = newDistance;
        }


        public void DrawMapOverlay() {
            if (MapView.MapIsEnabled) {
                //LGF
                if (toggleLGF) {
                    Vector3 p = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(system.vessel.transform.position));
                    Vector3 m = Input.mousePosition;
                    //new bearing line
                    if (toggleLGFnewBearing) {
                        GL.PushMatrix();
                        material.SetPass(0);
                        GL.LoadOrtho();
                        GL.Begin(GL.LINES);
                        GL.Color(Color.red);
                        GL.Vertex(new Vector3(p.x / Camera.main.pixelWidth, p.y / Camera.main.pixelHeight, 0));
                        GL.Vertex(new Vector3(m.x / Camera.main.pixelWidth, m.y / Camera.main.pixelHeight, 0));
                        GL.End();
                        GL.PopMatrix();
                    }
                    if (mapTextLabel == null)
                        mapTextLabel = Util.Label(12, Color.white, Color.white, new Color(1, 0, 0, 0.35f));
                    float min, max;
                    GUIContent ctnt = new GUIContent("?");
                    PaintGrid(Color.red, m);
                    //GRID labels
                    m = Event.current.mousePosition;
                    ctnt = new GUIContent(" " + lat + ", " + lon);
                    mapTextLabel.CalcMinMaxWidth(ctnt, out min, out max);
                    GUI.Label(new Rect(m.x - (max + 7), m.y, max + 7, 16), ctnt, mapTextLabel);
                    ctnt = new GUIContent(" " + lat2 + ", " + lon2);
                    mapTextLabel.CalcMinMaxWidth(ctnt, out min, out max);
                    GUI.Label(new Rect(m.x, m.y - 16, max + 7, 16), ctnt, mapTextLabel);
                    // bearing / distance labels
                    ctnt = new GUIContent(newBearing.ToString("f0") + "°, " + newDistance.ToString("f0") + "m");
                    mapTextLabel.CalcMinMaxWidth(ctnt, out min, out max);
                    GUI.Label(new Rect(m.x + 50, m.y, max + 7, 16), ctnt, mapTextLabel);
                }
                if (toggleLGFtarget) {
                    GL.PushMatrix();
                    material.SetPass(0);
                    GL.LoadOrtho();

                    Vector3 targetLoc = system.vessel.mainBody.GetWorldSurfacePosition(posTargetLat, posTargetLon, 0);
                    //draw target triangle
                    if(Vector3.Angle(ScaledSpace.ScaledToLocalSpace(system.planetariumCamera.camera.transform.position) - targetLoc, targetLoc - system.vessel.mainBody.position) < 90)
                        PaintTriangle(targetLoc, Color.red, (float)system.vessel.mainBody.Radius * 0.015f, system.vessel.mainBody);

                    //draw line to target
                    Vector3 P = system.vessel.transform.position - system.vessel.mainBody.position;
                    Vector3 Q = targetLoc - system.vessel.mainBody.position;
                    int segments = (int)Math.Floor(Vector3.Angle(P, Q));
                    GL.Begin(GL.LINES);
                    GL.Color(Color.red);
                    Vector3 node = Vector3.zero;
                    for (int i = 0; i <= segments; i++) {
                        node = system.vessel.mainBody.position + (float)system.vessel.mainBody.Radius * (P + (((float)i / (float)segments) * (Q - P))).normalized;
                        if (Vector3.Angle(ScaledSpace.ScaledToLocalSpace(system.planetariumCamera.camera.transform.position) - node, node - system.vessel.mainBody.position) < 90) {
                            node = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(node));
                            GL.Vertex3(node.x / Camera.main.pixelWidth, node.y / Camera.main.pixelHeight, 0);
                        }
                    }
                    GL.End();
                    GL.PopMatrix();
                }
                //KPS
                if (toggleKPSlocations) {
                    CelestialBody body = LazorGlobals.GetMapFocusBody();
                    //determine what celestial body map view has focus on
                    
                    foreach (KPSmapLocation mapLoc in system.guiTargets.locationList.GetByCelestialBody(body)) {
                        Vector3 pos = body.GetWorldSurfacePosition(mapLoc.lat, mapLoc.lon, 0); //map location in world space
                        Vector3 pos_cam = ScaledSpace.ScaledToLocalSpace(system.planetariumCamera.camera.transform.position); //camera position in world space
                        float a = Vector3.Angle(pos_cam - pos, pos - body.position);
                        if (a < 90 && (pos - pos_cam).magnitude < 5000000) { //cull with normal
                            Vector3 p = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(pos)); //map location  in screen space
                            Vector3 m = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(pos + (pos - body.position).normalized * Mathf.Clamp((float)body.Radius * 0.16f, 20000, 200000))); //map location sticker position in screen space
                            if (Util.IsWithinScreen(p)) {
                                //PaintTriangle(pos, Color.blue, (float)body.Radius * 0.0075f);
                                PaintTriangle(pos, Color.blue, 5000, body);
                                if (!Util.IsWithinScreen(m))
                                    m = p;
                                else {
                                    //draw pin
                                    GL.PushMatrix();
                                    material.SetPass(0);
                                    GL.LoadOrtho();
                                    GL.Begin(GL.LINES);
                                    GL.Color(Color.blue);
                                    GL.Vertex3(p.x / Camera.main.pixelWidth, p.y / Camera.main.pixelHeight, 0);
                                    GL.Vertex3(m.x / Camera.main.pixelWidth, m.y / Camera.main.pixelHeight, 0);
                                    GL.End();
                                    GL.PopMatrix();
                                }
                                //draw label
                                GUIContent ctnt = new GUIContent(mapLoc.name);
                                if (mapTextLabelBlueSolid == null) {
                                    mapTextLabelBlueSolid = Util.Label(11, Color.white, Color.white, new Color(0, 0, 1, 1f));
                                    mapTextLabelBlueSolid.alignment = TextAnchor.MiddleLeft;
                                    mapTextLabelBlueSolid.padding = new RectOffset(1, 1, 2, 0);
                                }
                                float min, max;
                                mapTextLabelBlueSolid.CalcMinMaxWidth(ctnt, out min, out max);
                                Rect labelRect = new Rect(m.x > p.x ? m.x : m.x - max, Screen.height - m.y - (m.y > p.y ? 12 : 0), max + 1, 12);
                                GUI.Label(labelRect, ctnt, mapTextLabelBlueSolid);
                                //draw label mouse over
                                if (labelRect.Contains(Event.current.mousePosition)) {
                                    ctnt = new GUIContent(mapLoc.lat + " " + mapLoc.lon);
                                    mapTextLabelBlueSolid.CalcMinMaxWidth(ctnt, out min, out max);
                                    Rect labelRect2 = new Rect(m.x > p.x ? m.x : m.x - max, Screen.height - m.y - (m.y > p.y ? 24 : 12), max + 1, 12);
                                    GUI.Label(labelRect2, ctnt, mapTextLabelBlueSolid);
                                    ctnt = new GUIContent(Util.DegToSex(mapLoc.lat) + (mapLoc.lat > 0 ? " N" : " S") + ", "
                                                        + Util.DegToSex(mapLoc.lon) + (mapLoc.lon > 0 ? " E" : " W"));
                                    mapTextLabelBlueSolid.CalcMinMaxWidth(ctnt, out min, out max);
                                    Rect labelRect3 = new Rect(m.x > p.x ? m.x : m.x - max, Screen.height - m.y - (m.y > p.y ? 36 : 24), max + 1, 12);
                                    GUI.Label(labelRect3, ctnt, mapTextLabelBlueSolid);
                                }
                            }
                        }
                    }
                }

                //Transfer overlay
                if (toggleBlue_all) {

                    Vector3d aS = pos_origin;
                    Vector3d bS = pos_reference;
                    Vector3d cS = pos_target;
                    Vector3d P = aS - bS; //from
                    Vector3d Q = cS - bS; //to
                    Vector3d dS = bS + MovementControl.RotateAroundNormal(P, normal_reference.normalized, (phase - phaseD) * Mathf.Deg2Rad); //where the vessel will be at departure
                    Vector3d eS = system.vessel.mainBody.position + MovementControl.RotateAroundNormal(system.vessel.transform.position - system.vessel.mainBody.position, normal_reference.normalized, (angle - angleD) * Mathf.Deg2Rad); //eject angle
                    Vector3d fS = bS + MovementControl.RotateAroundNormal(P, normal_reference.normalized, (phase - phaseD2) * Mathf.Deg2Rad); //bob's estimate of departure angle

                    float scale = (float)(PlanetariumCamera.Camera.transform.position - ScaledSpace.LocalToScaledSpace(aS)).magnitude / 5;
                
                    Vector3 a = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(aS) + (aS - bS).normalized * scale); //vessel axis end point on screen
                    Vector3 b = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(bS)); //reference point on screen
                    Vector3 c = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(cS)); //arrival point on screen
                    Vector3 d = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(dS) + (dS - bS).normalized * scale); //departure point on screen
                    Vector3 e = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(eS) + (eS - system.vessel.mainBody.position).normalized * scale); //eject point on screen
                    Vector3 f = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(fS) + (fS - bS).normalized * scale); //bob's estimate point on screen
                    Vector3 mainbody = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(system.vessel.mainBody.position));

                    GL.PushMatrix();
                    material.SetPass(0);
                    GL.LoadOrtho();

                    //phase angle
                    if (phase != -1) {
                        GL.Color(Color.red);
                        GL.Begin(GL.LINES);
                        GL.Vertex3(a.x / Camera.main.pixelWidth, a.y / Camera.main.pixelHeight, 0);
                        GL.Vertex3(b.x / Camera.main.pixelWidth, b.y / Camera.main.pixelHeight, 0);
                        GL.End();
                        GL.Begin(GL.LINES);
                        GL.Vertex3(b.x / Camera.main.pixelWidth, b.y / Camera.main.pixelHeight, 0);
                        GL.Vertex3(c.x / Camera.main.pixelWidth, c.y / Camera.main.pixelHeight, 0);
                        GL.End();

                        //transfer angle
                        GL.Color(Color.blue);
                        GL.Begin(GL.LINES);
                        GL.Vertex3(d.x / Camera.main.pixelWidth, d.y / Camera.main.pixelHeight, 0);
                        GL.Vertex3(b.x / Camera.main.pixelWidth, b.y / Camera.main.pixelHeight, 0);
                        GL.End();

                        //transfer angle bob's
                        GL.Color(Color.cyan);
                        GL.Begin(GL.LINES);
                        GL.Vertex3(f.x / Camera.main.pixelWidth, f.y / Camera.main.pixelHeight, 0);
                        GL.Vertex3(b.x / Camera.main.pixelWidth, b.y / Camera.main.pixelHeight, 0);
                        GL.End();
                    }
                    //eject angle
                    if (angle != -1) {
                        GL.Color(Color.yellow);
                        GL.Begin(GL.LINES);
                        GL.Vertex3(e.x / Camera.main.pixelWidth, e.y / Camera.main.pixelHeight, 0);
                        GL.Vertex3(mainbody.x / Camera.main.pixelWidth, mainbody.y / Camera.main.pixelHeight, 0);
                        GL.End();
                    }
                    //draw angle circle
                    float radius = (float)((aS - bS).magnitude);
                    DrawCircle(P, normal_reference, bS, phase, radius, Color.red);

                    //draw trajectory
                    if(system.vessel.orbit.eccentricity<0.5)
                        DrawTrajectory(dS - bS, normal_reference, bS, 180, radius, (float)((cS - bS).magnitude));

                    //draw optimal co-planar orbit
                    DrawCircle(Q, normal_target, bS, 360, radius, Color.green);
                    GL.PopMatrix();

                    if (phase != -1) {
                        DrawTextAtScreenPos(a, b, phase.ToString("f2") + "°", Util.StdLabelSmallCustom(Color.red, TextAnchor.MiddleLeft)); //phase
                        DrawTextAtScreenPos(d, b, phaseD.ToString("f2") + "°", Util.StdLabelSmallCustom(Color.blue, TextAnchor.MiddleLeft)); //phase alignment
                        DrawTextAtScreenPos(f, b, phaseD2.ToString("f2") + "°", Util.StdLabelSmallCustom(Color.cyan, TextAnchor.MiddleLeft)); //phase alignment elliptical
                    }
                    if (angle != -1)
                        DrawTextAtScreenPos(e, mainbody, angleD.ToString("f2") + "°", Util.StdLabelSmallCustom(Color.yellow, TextAnchor.MiddleLeft)); //eject
                }
            }
        }

        public void DrawTextAtScreenPos(Vector3 pos, Vector3 origo, string text, GUIStyle style) {
            GUIContent ctnt = new GUIContent(text);
            float min, max;
            style.CalcMinMaxWidth(ctnt, out min, out max);
            Rect labelRect = new Rect(pos.x > origo.x ? pos.x : pos.x - max, Screen.height - pos.y - (pos.y > origo.y ? 20 : 0), max, 20);
            GUI.Label(labelRect, ctnt, style);
        }

        private void DrawCircle(Vector3d from, Vector3d normal, Vector3 origin, float angle, float radius, Color c) {
            Vector3d node = Vector3.zero;
            int segments = (int)Math.Floor(angle);
            GL.Begin(GL.LINES);
            GL.Color(c);
            for (int i = 0; i <= segments; i++) {
                float j = i * Mathf.Deg2Rad;
                node = from * (float)Math.Cos(j) + Vector3.Cross(normal, from) * (float)Math.Sin(j) + normal * (Vector3.Dot(normal, from) * (float)(1 - Math.Cos(j))); //Rodrigues' rotation formula
                Vector3 F = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(origin + node.normalized * radius));
                GL.Vertex3(F.x / Camera.main.pixelWidth, F.y / Camera.main.pixelHeight, 0);
            }
            GL.End();
        }

        private void DrawTrajectory(Vector3d from, Vector3d normal, Vector3 origin, float angle, float radius, float radiusEnd) {
            Vector3d node = Vector3.zero;
            int segments = (int)Math.Floor(angle);
            GL.Begin(GL.LINES);
            GL.Color(Color.blue);
            float a = (radius + radiusEnd) / 2;
            Vector3 focus = origin + (from.normalized * radius) - (from.normalized * (a-a * e));
            for (int i = 0; i <= segments; i++) {
                float j = i * Mathf.Deg2Rad;
                node = from * (float)Math.Cos(j) + Vector3.Cross(normal, from) * (float)Math.Sin(j) + normal * (Vector3.Dot(normal, from) * (float)(1 - Math.Cos(j))); //Rodrigues' rotation formula
                float r = (float)(a * (1 - e * e) / (1 + e * Math.Cos(i * Mathf.Deg2Rad))); //distance: focus to true anomaly
                Vector3 F = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(focus + node.normalized * (r)));
                GL.Vertex3(F.x / Camera.main.pixelWidth, F.y / Camera.main.pixelHeight, 0);
            }
            GL.End();
        }

        


        public bool MapSelect(out double lat, out double lon){ //wtf why is it upside down? O.o
            if (MapView.MapIsEnabled) {
                if (mapTextLabelBlue == null)
                    mapTextLabelBlue = Util.Label(12, Color.white, Color.white, new Color(0, 0, 1, 0.35f));
                float min, max;
                GUIContent ctnt = new GUIContent("?");
                Vector3 m = Input.mousePosition;
                PaintGrid(Color.blue, m);
                //GRID labels
                m = Event.current.mousePosition;
                    
                ctnt = new GUIContent(" " + latitudeDecimal.ToString("f6") + ", " + longitudeDecimal.ToString("f6"));
                mapTextLabelBlue.CalcMinMaxWidth(ctnt, out min, out max);
                GUI.Label(new Rect(m.x - (max + 7), m.y, max + 7, 16), ctnt, mapTextLabelBlue);
                ctnt = new GUIContent(" " + latitudeSexagesimal + ", " + longitudeSexagesimal);
                mapTextLabelBlue.CalcMinMaxWidth(ctnt, out min, out max);
                GUI.Label(new Rect(m.x, m.y - 16, max + 7, 16), ctnt, mapTextLabelBlue);

                m.y = Screen.height - m.y; // upside down crap
                Ray ray = system.planetariumCamera.camera.ScreenPointToRay(m);
                ray.origin = ScaledSpace.ScaledToLocalSpace(ray.origin);
                Vector3d origin = ray.origin - system.vessel.mainBody.position;
                Vector3d surfacePos;
                if (PQS.LineSphereIntersection(origin, ray.direction, system.vessel.mainBody.Radius, out surfacePos)) {
                    surfacePos += system.vessel.mainBody.position;
                    latitudeDecimal = system.vessel.mainBody.GetLatitude(surfacePos);
                    longitudeDecimal = system.vessel.mainBody.GetLongitude(surfacePos);
                    latitudeSexagesimal = Util.DegToSex(latitudeDecimal) + (latitudeDecimal > 0 ? " N" : " S");
                    longitudeSexagesimal = Util.DegToSex(longitudeDecimal) + (longitudeDecimal > 0 ? " E" : " W");
                    if (Input.GetMouseButtonDown(0)) {
                        latitudeDecimalF = latitudeDecimal;
                        longitudeDecimalF = longitudeDecimal;
                        latitudeSexagesimalF = latitudeSexagesimal;
                        longitudeSexagesimalF = longitudeSexagesimal;
                        lat = latitudeDecimal;
                        lon = longitudeDecimal;
                        return true;
                    }
                }
            } else { //!MapView.MapIsEnabled
                MapView.EnterMapView();
            }
  
            lat = latitudeDecimalF;
            lon = longitudeDecimalF;
            return false;
        }

        public void PaintGrid(Color c, Vector3 m) {
            GL.PushMatrix();
            material.SetPass(0);
            GL.LoadOrtho();
            //GRID
            GL.Begin(GL.LINES);
            GL.Color(c);
            GL.Vertex(new Vector3(0, m.y / Camera.main.pixelHeight, 0));
            GL.Vertex(new Vector3(1, m.y / Camera.main.pixelHeight, 0));
            GL.End();
            GL.Begin(GL.LINES);
            GL.Color(c);
            GL.Vertex(new Vector3(m.x / Camera.main.pixelWidth, 0, 0));
            GL.Vertex(new Vector3(m.x / Camera.main.pixelWidth, 1, 0));
            GL.End();
            GL.PopMatrix();
        }

        /*public void PaintTriangle(double lat, double lon, Color c, float size, CelestialBody body) {
            PaintTriangle(system.vessel.mainBody.GetWorldSurfacePosition(lat, lon, 0), c, size, body);
        }*/

        public void PaintTriangle(Vector3 pos, Color c, float size, CelestialBody body) {
            //float a = Vector3.Angle(ScaledSpace.ScaledToLocalSpace(system.planetariumCamera.camera.transform.position) - pos, pos - system.vessel.mainBody.position);
            //if (a < 90) { //if the targets spherical normal is pointing away then the point must be on the other side of the sphere.
                GL.PushMatrix();
                material.SetPass(0);
                GL.LoadOrtho();
                GL.Begin(GL.TRIANGLES);
                GL.Color(c);
                Vector3d north = Util.NorthVector(body, pos);
                Vector3 up = pos - body.position;
                Vector2 d1 = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(pos + north * size));
                Vector3 d2 = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(pos + Quaternion.AngleAxis(120, up) * north * size));
                Vector3 d3 = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(pos + Quaternion.AngleAxis(-120, up) * north * size));
                GL.Vertex(new Vector3(d1.x / Camera.main.pixelWidth, d1.y / Camera.main.pixelHeight, 0));
                GL.Vertex(new Vector3(d2.x / Camera.main.pixelWidth, d2.y / Camera.main.pixelHeight, 0));
                GL.Vertex(new Vector3(d3.x / Camera.main.pixelWidth, d3.y / Camera.main.pixelHeight, 0));
                GL.End();
                GL.PopMatrix();
            //}
        }

        //dist in map scale
        public void PaintArrow(Vector3 dir, float dist, Color c) {
            float scale = PlanetariumCamera.fetch.Distance / 6;
            Vector3 p = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(system.vessel.transform.position));
            Vector3 d = system.planetariumCamera.camera.WorldToScreenPoint(ScaledSpace.LocalToScaledSpace(system.vessel.transform.position) + dir * dist * scale);
            Vector3 g = (p - d).normalized;
            float arrowhead = ((float)Math.Sqrt(5/ scale) + 0.3f);
            GL.PushMatrix();
            material.SetPass(0);
            GL.LoadOrtho();
            GL.Begin(GL.LINES);//main line
            GL.Color(c);
            GL.Vertex3(p.x / Camera.main.pixelWidth, p.y / Camera.main.pixelHeight, 0);
            GL.Vertex3(d.x / Camera.main.pixelWidth, d.y / Camera.main.pixelHeight, 0);
            GL.End();
            Vector3 df = new Vector3(g.x * cosT30 - g.y * sinT30, g.x * sinT30 + g.y * cosT30, 0).normalized * arrowhead;
            Vector3 df2 = new Vector3(g.x * cosT30 - g.y * -sinT30, g.x * -sinT30 + g.y * cosT30, 0).normalized * arrowhead;
            GL.Begin(GL.LINES);
            GL.Color(c);
            GL.Vertex3(d.x / Camera.main.pixelWidth, d.y / Camera.main.pixelHeight, 0);
            GL.Vertex3((d + df * 30).x / Camera.main.pixelWidth, (d + df * 30).y / Camera.main.pixelHeight, 0);
            GL.End();
            GL.Begin(GL.LINES);
            GL.Color(c);
            GL.Vertex3(d.x / Camera.main.pixelWidth, d.y / Camera.main.pixelHeight, 0);
            GL.Vertex3((d + df2 * 30).x / Camera.main.pixelWidth, (d + df2 * 30).y / Camera.main.pixelHeight, 0);
            GL.End();
            GL.PopMatrix();
        }

        public void Dispose() {
            toggleLGF = false;
            toggleLGFnewBearing = false;
            toggleLGFtarget = false;
        }

 
    }
}
