﻿/* Author: Jim-Kyrre Benjaminsen AKA Romfarer
 * License: This work is licensed under a Creative Commons Attribution-NonCommercial-NoDerivs 3.0 Unported License. 
 * http://creativecommons.org/licenses/by-nc-nd/3.0/
*/

using System;
using UnityEngine;
using System.Collections.Generic;

namespace Romfarer {
    class GUIparameters : GUItab {

        private enum InfoModes {Surface = 0, Orbit = 1};
        private bool togglePhysical;
        private bool toggleResources;
        private InfoModes infoMode = 0;

    
        public GUIparameters(LazorSystem system) : base(system) {}

        override public string Name { get { return "Parameters"; } }
        override public string GetTabName(){
            return "Parameters";
        }

        //Draw the GUI window.
        override public void DrawGUI(string name) {
            if (!enabledSet) {
                if (system.vessel.situation == Vessel.Situations.SUB_ORBITAL || system.vessel.situation == Vessel.Situations.LANDED || system.vessel.situation == Vessel.Situations.PRELAUNCH || system.vessel.situation == Vessel.Situations.SPLASHED || system.vessel.situation == Vessel.Situations.FLYING)
                    infoMode = InfoModes.Surface;
                else
                    infoMode = InfoModes.Orbit;
                enabledSet = true;
            }
            GUI.skin = Util.LazorSkin;
            windowPos = GUILayout.Window(id, windowPos, WindowGUI, name);
            if (!docked)
                windowPos = Util.ClampToScreen(windowPos);
        }

        override public void WindowGUI(int windowID) {
            Header();

            infoMode = (InfoModes)GUILayout.SelectionGrid((int)infoMode, Enum.GetNames(typeof(InfoModes)), 2);
       
            // Flight parameters
            Vessel v = (Vessel)system.vessel;
            GUILayout.BeginVertical();

            GUILayout.BeginHorizontal(Util.Parameters_highlight);
            GUILayout.Label("► ", Util.StdLabelSmallCustom(Color.green));
            GUILayout.Label(v.vesselName, Util.StdLabelSmallCustom(Color.white));
            GUILayout.Label(" ◄", Util.StdLabelSmallCustom(Color.green));
            GUILayout.EndHorizontal();

            system.VI.X(system.VI.param_MissionTime);
            system.VI.X(system.VI.param_Crew);
            system.VI.X(system.VI.param_Situation);
            system.VI.X(system.VI.param_SoI);
			system.VI.X(system.VI.param_Acceleration);
            switch (infoMode) {
                case InfoModes.Surface:
                    system.VI.X(system.VI.param_AltitudeFT);
                    system.VI.X(system.VI.param_SurfaceSpeed);
                    system.VI.X(system.VI.param_VerticalSpeed);
                    system.VI.X(system.VI.param_Latitude);
                    //system.VI.X(system.VI.param_Longitude);
                    break;
                case InfoModes.Orbit:
                    system.VI.X(system.VI.param_Altitude);
                    system.VI.X(system.VI.param_OrbitSpeed);
                    system.VI.X(system.VI.param_Apoapsis);
                    system.VI.X(system.VI.param_TimeToApoapsis);
                    system.VI.X(system.VI.param_Periapsis);
                    system.VI.X(system.VI.param_TimeToPeriapsis);
                    system.VI.X(system.VI.param_Period);
                    system.VI.X(system.VI.param_Eccentricity);
                    system.VI.X(system.VI.param_Inclination);
                    system.VI.X(system.VI.param_MeanAnomaly);
                    system.VI.X(system.VI.param_EccentricAnomaly);
                    system.VI.X(system.VI.param_TrueAnomaly);
                    break;
            }
            GUILayout.EndVertical();

            togglePhysical = GUILayout.Toggle(togglePhysical, "Physical Parameters", Util.Button_Toggle);
            if (togglePhysical) {
                GUILayout.BeginVertical();
                system.VI.X(system.VI.param_CurrentStage);
                system.VI.X(system.VI.param_Mass);
                system.VI.X(system.VI.param_AtmosphericDensity);
                system.VI.X(system.VI.param_Pressure);
                system.VI.X(system.VI.param_DynamicPressure);
                system.VI.X(system.VI.param_GeeForce);
                system.VI.X(system.VI.param_Gravity);
                system.VI.X(system.VI.param_Temperature);
                GUILayout.EndVertical();
            }

            toggleResources = GUILayout.Toggle(toggleResources, "Resources", Util.Button_Toggle);
            if (toggleResources)
                system.VI.Y();
            Footer();
        }

        public void onFlightStateLoad(Dictionary<string, KSPParseable> parsedData) {
            onFlightStateLoad(parsedData, "GUIparameters");
        }
        public void onFlightStateSave(Dictionary<string, KSPParseable> partDataCollection) {
            onFlightStateSave(partDataCollection, "GUIparameters");
        }

    }
}
