﻿/* Author: Jim-Kyrre Benjaminsen AKA Romfarer
 * License: This work is licensed under a Creative Commons Attribution-NonCommercial-NoDerivs 3.0 Unported License. 
 * http://creativecommons.org/licenses/by-nc-nd/3.0/
*/

using System;
using System.Collections.Generic;
using System.Text;
using UnityEngine;

namespace Romfarer {
    abstract class GUItab {

        protected int id;
        protected LazorSystem system;
        protected Rect windowPos;
        protected bool docked = true; //is the window docked with the main window?
        //public bool Docked { get { return docked; } }
        protected bool enabledSet;
        protected bool enabled; //window showing?
        public bool Enabled {
            get { return enabled; }
            set {
                if (value && !enabled) system.guiController.Shuffle(this);
                else if (!value && enabled) system.guiController.Pop(this);
                enabled = value;
            }
        }
        public bool hasContent = true;
        protected bool dragging; //internal stuff, is the window being dragged?
        private static bool lockEvents; //internal stuff, are the drag/drop events locked?
        abstract public string Name { get; }

        public GUItab(LazorSystem system) {
            this.system = system;
            id = system.guiController.GenerateID();
        }
        //required
        abstract public void DrawGUI(string name);
        abstract public string GetTabName();
        abstract public void WindowGUI(int windowID);
        //optional
        virtual public void OnFixedUpdate() { }
		virtual public void OnDestroy() { }

        //button placed left of tabs
        virtual public float GUIbutton(float indent, int width = 200, int height = 100) {
            float xOffSet = indent;
            if (Enabled) {
                if (Event.current.type == EventType.Layout) {
                    if (docked) {
                        xOffSet = xOffSet + windowPos.width + 5;
                        windowPos = new Rect(system.windowPos.x - xOffSet, system.windowPos.y, width, height);
                    } else {
                        windowPos.width = width;
                        windowPos.height = height;
                    }
                }
                DrawGUI(GetTabName());
            }
            return xOffSet;
        }

        protected void Header() {
            if (GUI.Button(new Rect(windowPos.width - 20, 0, 20, 20), "", Util.Button_header_close)) { //red close button
                Enabled = false;
                enabledSet = false;
            }
            if (!docked && !dragging) //docking button.....
                GUI.enabled = true;
            else
                GUI.enabled = false;
            if (GUI.Button(new Rect(windowPos.width - 20 - 20, 0, 20, 20), "", !docked && !dragging ? Util.Button_header_minimize : GUIStyle.none)) {
                system.guiController.Shuffle(this);
                docked = true;
            }
            GUI.enabled = true;
        }

        protected void Footer() {
            Rect dragRect = new Rect(0, 0, windowPos.width, 30);
            if (!lockEvents && docked && dragRect.Contains(Event.current.mousePosition) && Event.current.button == 0 && Event.current.isMouse){ //&& Input.GetMouseButton(0) && Event.current.delta != Vector2.zero) {
                lockEvents = true;
                docked = false;
                dragging = true;
            }
            if (dragging && Input.GetMouseButtonUp(0)) {
                lockEvents = false;
                dragging = false;
            }
            GUI.DragWindow(dragRect);
        }

        protected void onFlightStateLoad(Dictionary<string, KSPParseable> parsedData, string identifier) {
            Util.LoadKSPParseableRect(parsedData, identifier + "_windowPos", ref windowPos);
            if (parsedData.ContainsKey(identifier + "_docked"))
                docked = parsedData[identifier + "_docked"].value_bool;
            if (parsedData.ContainsKey(identifier + "_Enabled"))
                Enabled = parsedData[identifier + "_Enabled"].value_bool;
        }

        protected void onFlightStateSave(Dictionary<string, KSPParseable> partDataCollection, string identifier) {
            Util.SaveKSPParseableRect(partDataCollection, identifier + "_windowPos", windowPos);
            partDataCollection.Add(identifier + "_docked", new KSPParseable((bool)docked, KSPParseable.Type.BOOL));
            partDataCollection.Add(identifier + "_Enabled", new KSPParseable((bool)Enabled, KSPParseable.Type.BOOL));
        }

    }
}
