﻿/* Author: Jim-Kyrre Benjaminsen AKA Romfarer
 * License: This work is licensed under a Creative Commons Attribution-NonCommercial-NoDerivs 3.0 Unported License. 
 * http://creativecommons.org/licenses/by-nc-nd/3.0/
*/

using System;
using System.Collections.Generic;
using System.Text;
using UnityEngine;

namespace Romfarer {

    class DockingCamera {
        //private GameObject camGameObject;
        //private GameObject camGameObject2;
        //private GameObject camGameObject3;
        //public Camera cam;
        //public Camera cam2;
        //public Camera cam3;
        //protected RenderTexture renTex;

		private Rect vSmall = new Rect(5, 30, 266, 266);
		public Rect vMedium = new Rect(5, 30, 410, 410);
        public Rect vLarge = new Rect(5, 30, 635, 635);
		private Rect vCurrent = new Rect(5, 30, 266, 266);
        protected Texture bg = Util.getTexture(Color.black);
        public Rect windowPos;

        private Ray ray;
        private RaycastHit hit;
        private Texture2D overlay;
        private String hitDistance = "∞";
        private bool isHitting;
        private bool lateral;
        private PartModule pm_cam;
        private PartModule pm_dock;
        private Vector3 relativeVel;
        private bool locked;
        private double vel_x;
        private double vel_y;
        private double ang_x;
        private double ang_y;
        private double dist_x;
        private double dist_y;
        private double ang_z; //angle between up vectors of docking ports
        private UtilAverage vel_xA = new UtilAverage(20);
        private UtilAverage vel_yA = new UtilAverage(20);
        private Material material;
        private Material material2;
		//private CameraModes cameraMode = CameraModes.Normal;
		//private int camHeight = 200;
		//private int camWidth = 200;
		private CamSize camSize = CamSize.SMALL;
		//Material material_lighten;


		private enum CamSize{
			SMALL = 0,
			MEDIUM = 1,
			LARGE = 2
		}

		private LazorCam lazorCam;
		private LazorCamGui lazorGui;

        public DockingCamera(PartModule pm_cam, PartModule pm_dock) {
            this.pm_cam = pm_cam;
            this.pm_dock = pm_dock;
			//lazorCam = new LazorCam(256, 256, 24, RenderTextureFormat.Default, RenderTextureReadWrite.sRGB, 40, 32771);
			lazorCam = new LazorCam(512, 512, 24, RenderTextureFormat.Default, RenderTextureReadWrite.sRGB, 40, 32771);
			lazorGui = new LazorCamGui(lazorCam);
            material = new Material(Shader.Find("Particles/Additive"));
			material2 = new Material(Shader.Find("Sprite/Vertex Colored"));  //Multiply (Double)
			overlay = Util.LoadTexture(256, 256, "dockingcam.png");
			windowPos = vSmall;
			lateral = pm_cam.part.name == "dockingPortLateral";
			//SetFilters();
        }

		public void SetNewRenderTexture(int width, int height, int depth, RenderTextureFormat format, RenderTextureReadWrite rw, float fov, int cullingMask){
			/*lazorCam.DestroyCams();
			lazorCam.SetNewRenderTexture(width, height, depth, RenderTextureFormat.Default, RenderTextureReadWrite.sRGB, fov, cullingMask);
			SetFilters();*/
		}

		private void SetFilters(){
			lazorCam.cam_Main.gameObject.AddComponent<CameraFilterSepiatone>();
			lazorCam.cam_Main.gameObject.AddComponent<CameraFilterGrayscale>();
			lazorCam.cam_Far.gameObject.AddComponent<CameraFilterDummy>();
			lazorCam.cam_Space.gameObject.AddComponent<CameraFilterDummy>();
		}

        public void Update() {
            if (FlightGlobals.fetch.VesselTarget is ModuleDockingNode) {
                Vector3 dirToTarget = FlightGlobals.fetch.VesselTarget.GetTransform().position - pm_dock.part.transform.position;
				ray = new Ray(lazorCam.position, dirToTarget);
                relativeVel = (FlightGlobals.fetch.VesselTarget as ModuleDockingNode).GetObtVelocity() - (pm_dock as ModuleDockingNode).GetObtVelocity();
                locked = true;

				dist_x = dirToTarget.magnitude * Math.Sin(Util.AngleAroundNormal(dirToTarget, lazorCam.forward, lazorCam.up) * Mathf.Deg2Rad);
				vel_x = relativeVel.magnitude * Math.Sin(Util.AngleAroundNormal(relativeVel, lazorCam.forward, lazorCam.up) * Mathf.Deg2Rad);
				ang_x = Util.AngleAroundNormal(-(FlightGlobals.fetch.VesselTarget as ModuleDockingNode).GetFwdVector(), lazorCam.forward, lazorCam.up);
                
				dist_y = dirToTarget.magnitude * Math.Sin(Util.AngleAroundNormal(dirToTarget, lazorCam.forward, -lazorCam.right) * Mathf.Deg2Rad);
				vel_y = relativeVel.magnitude * Math.Sin(Util.AngleAroundNormal(relativeVel, lazorCam.forward, -lazorCam.right) * Mathf.Deg2Rad);
				ang_y = Util.AngleAroundNormal(-(FlightGlobals.fetch.VesselTarget as ModuleDockingNode).GetFwdVector(), lazorCam.forward, -lazorCam.right);

                vel_xA.Average(vel_x);
                vel_yA.Average(vel_y);

				ang_z = Util.AngleAroundNormal((FlightGlobals.fetch.VesselTarget as ModuleDockingNode).GetTransform().up, lazorCam.up, -lazorCam.forward);

            } else {
				ray = new Ray(lazorCam.position, lazorCam.forward);
                locked = false;
            }

            isHitting = Physics.Raycast(ray.origin, ray.direction, out hit, 1000, 557059);
            if (isHitting) {
                hitDistance = hit.distance.ToString("f2");
            } else
                hitDistance = "∞";
        }



        public void WindowGUI(int windowID) {
			windowPos.width = vCurrent.width + 10;
			windowPos.height = vCurrent.height + 30;
			float offH = vCurrent.width;

			if(GUI.Button(new Rect(windowPos.width - 20, 0, 20, 20), "", Util.Button_header_close)) { //red close button
				(pm_cam as LazorSystemDockingCamera).on = false;
				pm_cam.Events["ActivateLazorCamera"].active = true;
				pm_cam.Events["DeactivateLazorCamera"].active = false;
			}

			Vector3 aim = Vector3.zero;
			Vector3 origin = Vector3.zero;
			Vector3 n1 = Vector3.zero;
			Vector3 n2 = Vector3.zero;

			if(lateral) {
				aim = -pm_cam.part.transform.forward;
				n1 = n2 = pm_cam.part.transform.up;
				origin = pm_cam.part.transform.position - pm_cam.part.transform.forward;
			} else {
				aim = pm_cam.part.transform.up;
				n1 = n2 = -pm_cam.part.transform.forward;
				origin = pm_cam.part.transform.position + pm_cam.part.transform.up;
			}
			Component target = null;
			if(LazorSystemControllerGlobal.ActiveLazorSystem != null)
				target =  LazorSystemControllerGlobal.ActiveLazorSystem.Target;
			else if(FlightGlobals.fetch.VesselTarget is ModuleDockingNode)
				target =  FlightGlobals.fetch.VesselTarget as ModuleDockingNode;

			Rect expandButton = new Rect(vCurrent.width - 25, vCurrent.height, 20, 20);
			Rect a = new Rect(expandButton.x, expandButton.y, 20, 20);
			Matrix4x4 matrix = GUI.matrix;
			switch (camSize){
				case CamSize.SMALL:
					lazorGui.CameraGuiDocking(MultiMaterial.Lighten, 256, 256, aim, origin, n1, n2);
					//Graphics.DrawTexture(new Rect(5, 30, 256, 256), renTex, material_lighten);
					Graphics.DrawTexture(new Rect(5, 30, 256, 256), overlay);
					GUIUtility.RotateAroundPivot(-90, new Vector2(a.x + 10, a.y + 10)); 
					if (GUI.Button(a, "", Util.Divider_gray_button)) {
						camSize = CamSize.MEDIUM;
						vCurrent = vMedium;
						SetNewRenderTexture(384, 384, 24, RenderTextureFormat.Default, RenderTextureReadWrite.sRGB, 40, 32771);
						
					}
					break;
				case CamSize.MEDIUM:
					lazorGui.CameraGuiDocking(MultiMaterial.Lighten, 400, 400, aim, origin, n1, n2);
					//Graphics.DrawTexture(new Rect(5, 30, 400, 400), renTex, material_lighten);
					Graphics.DrawTexture(new Rect(5, 30, 400, 400), overlay);
					GUIUtility.RotateAroundPivot(-90, new Vector2(a.x + 10, a.y + 10)); 
					if (GUI.Button(a, "", Util.Divider_gray_button)) {
						camSize = CamSize.LARGE;
						vCurrent = vLarge;
						//SetNewRenderTexture(625, 625, 24, RenderTextureFormat.Default, RenderTextureReadWrite.sRGB, 40, 32771);
						SetNewRenderTexture(512, 512, 24, RenderTextureFormat.Default, RenderTextureReadWrite.sRGB, 40, 32771);
					}
					GUIUtility.RotateAroundPivot(180, new Vector2(a.x + 10, a.y + 10)); 
					a.x -= 10;
					a.y += 10;
					if (GUI.Button(a, "", Util.Divider_gray_button)) {
						camSize = CamSize.SMALL;
						vCurrent = vSmall;
						SetNewRenderTexture(256, 256, 24, RenderTextureFormat.Default, RenderTextureReadWrite.sRGB, 40, 32771);
						
					}
					break;
				case CamSize.LARGE:
					lazorGui.CameraGuiDocking(MultiMaterial.Lighten, 625, 625, aim, origin, n1, n2);
					//Graphics.DrawTexture(new Rect(5, 30, 625, 625), renTex, material_lighten);
					Graphics.DrawTexture(new Rect(5, 30, 625, 625), overlay);
					GUIUtility.RotateAroundPivot(90, new Vector2(a.x + 10, a.y + 10)); 
					if (GUI.Button(a, "", Util.Divider_gray_button)) {
						camSize = CamSize.MEDIUM;
						vCurrent = vMedium;
						SetNewRenderTexture(384, 384, 24, RenderTextureFormat.Default, RenderTextureReadWrite.sRGB, 40, 32771);
						
					}
					break;
			}

			GUI.matrix = matrix;

			//GUILayout.Space(100);
            //GUILayout.Space(245);







            //angular velocity
			float indentA = offH/2-53;
			float indentB = offH/2-15;
			Rect label_wX_ = new Rect(offH/2, 40, indentA, 20);
            GUI.Label(label_wX_, "ω X ", Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));
			Rect label_wX = new Rect(offH/2, 40, indentB, 20);
            GUI.Label(label_wX, (-pm_cam.part.vessel.angularVelocity.z).ToString("f3"), Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));

			Rect label_wY_ = new Rect(offH/2, 51, indentA, 20);
            GUI.Label(label_wY_, "ω Y ", Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));
			Rect label_wY = new Rect(offH/2, 51, indentB, 20);
            GUI.Label(label_wY, (-pm_cam.part.vessel.angularVelocity.x).ToString("f3"), Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));

			Rect label_wZ_ = new Rect(offH/2, 62, indentA, 20);
            GUI.Label(label_wZ_, "ω Z ", Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));
			Rect label_wZ = new Rect(offH/2, 62, indentB, 20);
            GUI.Label(label_wZ, (-pm_cam.part.vessel.angularVelocity.y).ToString("f3"), Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));

            //distance
			Rect label_dist_ = new Rect(offH/2, offH - 36, indentA, 20);
            GUI.Label(label_dist_, "d ", Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));
			Rect label_dist = new Rect(offH/2, offH - 36, indentB, 20);
            GUI.Label(label_dist, hitDistance, Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));

            Rect label_lock = new Rect(8, 30, 100, 20);
            GUI.Label(label_lock, locked?"LOCKED":"LOCK PENDING", Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleLeft));
            if (locked) {
                //relative velocity
				Rect label_vel_ = new Rect(offH/2, 84, indentA, 20);
                GUI.Label(label_vel_, "v ", Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));
				Rect label_vel = new Rect(offH/2, 84, indentB, 20);
                GUI.Label(label_vel, relativeVel.magnitude.ToString("f3"), Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));

				Rect label_vel_x_ = new Rect(offH/2, 95, indentA, 20);
                GUI.Label(label_vel_x_, "v X ", Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));
				Rect label_vel_x = new Rect(offH/2, 95, indentB, 20);
                GUI.Label(label_vel_x, vel_x.ToString("f3"), Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));

				Rect label_vel_y_ = new Rect(offH/2, 106, indentA, 20);
                GUI.Label(label_vel_y_, "v Y ", Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));
				Rect label_vel_y = new Rect(offH/2, 106, indentB, 20);
                GUI.Label(label_vel_y, vel_y.ToString("f3"), Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));

				float startLine = offH - 80 ;
                //angle
				Rect label_ang_x_ = new Rect(offH/2, startLine, indentA, 20);
                GUI.Label(label_ang_x_, "° X ", Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));
				Rect label_ang_x = new Rect(offH/2, startLine, indentB, 20);
                GUI.Label(label_ang_x, ang_x.ToString("f3"), Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));
				startLine += 11;

				Rect label_ang_y_ = new Rect(offH/2, startLine, indentA, 20);
                GUI.Label(label_ang_y_, "° Y ", Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));
				Rect label_ang_y = new Rect(offH/2, startLine, indentB, 20);
                GUI.Label(label_ang_y, ang_y.ToString("f3"), Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));
				startLine += 11;

				Rect label_ang_z_ = new Rect(offH/2, startLine, indentA, 20);
                GUI.Label(label_ang_z_, "° Z ", Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));
				Rect label_ang_z = new Rect(offH/2, startLine, indentB, 20);
                GUI.Label(label_ang_z, ang_z.ToString("f2"), Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));
				startLine += 33;

                //distance
				Rect label_dist_x_ = new Rect(offH/2, startLine, indentA, 20);
                GUI.Label(label_dist_x_, "d X ", Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));
				Rect label_dist_x = new Rect(offH/2, startLine, indentB, 20);
                GUI.Label(label_dist_x,  dist_x.ToString("f3"), Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));
				startLine += 11;

				Rect label_dist_y_ = new Rect(offH/2, startLine, indentA, 20);
                GUI.Label(label_dist_y_, "d Y ", Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));
				Rect label_dist_y = new Rect(offH/2, startLine, indentB, 20);
                GUI.Label(label_dist_y, dist_y.ToString("f3"), Util.StdLabelSmallCustom(Color.black, TextAnchor.MiddleRight));

                GL.PushMatrix();
                material.SetPass(0);
                GL.LoadPixelMatrix();

                //WHITE CROSS
                GL.Begin(GL.LINES); //horizontal angle
                GL.Color(Color.white);
				float yoff = Screen.height - (windowPos.y + offH + 25);
				float y = Mathf.Clamp(yoff + (offH/2) - (float)ang_y, yoff, yoff + offH);
                GL.Vertex(new Vector3(windowPos.x, y, 0));
				GL.Vertex(new Vector3(windowPos.x + (offH/2)-25, y, 0));

				GL.Vertex(new Vector3(windowPos.x + (offH/2)+25, y, 0));
                GL.Vertex(new Vector3(windowPos.x + windowPos.width, y, 0));
                GL.End();
                GL.Begin(GL.LINES); //vertical angle
                GL.Color(Color.white);
				float x = Mathf.Clamp(windowPos.x + (offH/2) - (float)ang_x, windowPos.x, windowPos.x + offH);
                GL.Vertex(new Vector3(x, yoff, 0));
				GL.Vertex(new Vector3(x, yoff + (offH/2)-25, 0));

				GL.Vertex(new Vector3(x, yoff + (offH/2)+25, 0));
				GL.Vertex(new Vector3(x, yoff + offH, 0));
                GL.End();

				//RED CROSS
                material2.SetPass(0);
                GL.Begin(GL.LINES); //velocity
                GL.Color(Color.red);
				float vel_xc = windowPos.x + (offH/2) + (float)vel_xA * 50;
				float vel_yc = yoff + (offH/2) + (float)vel_yA * 50;
				GL.Vertex(new Vector3(Mathf.Clamp(vel_xc - 25, windowPos.x, windowPos.x + offH),
				                      Mathf.Clamp(vel_yc, yoff, yoff + offH), 0));
				GL.Vertex(new Vector3(Mathf.Clamp(vel_xc + 25, windowPos.x, windowPos.x + offH),
				                      Mathf.Clamp(vel_yc, yoff, yoff + offH), 0));

				GL.Vertex(new Vector3(Mathf.Clamp(vel_xc, windowPos.x, windowPos.x + offH),
				                      Mathf.Clamp(vel_yc - 25, yoff, yoff + offH), 0));
				GL.Vertex(new Vector3(Mathf.Clamp(vel_xc, windowPos.x, windowPos.x + offH),
				                      Mathf.Clamp(vel_yc + 25, yoff, yoff + offH), 0));
                GL.End();

                //RED TRIANGLE, zangle
                GL.Begin(GL.TRIANGLES);
                GL.Color(Color.red);
				float cx = windowPos.x + (offH/2);
				float cy = yoff + (offH/2);
                float ang_zF = Util.TwoPiToSignedAngle((float)ang_z + 90);
                double cx1 = cx + 35 * Math.Cos(ang_zF * Mathf.Deg2Rad);
                double cy1 = cy + 35 * Math.Sin(ang_zF * Mathf.Deg2Rad);
                double cx2 = cx + 40 * Math.Cos((ang_zF - 5) * Mathf.Deg2Rad);
                double cy2 = cy + 40 * Math.Sin((ang_zF - 5) * Mathf.Deg2Rad);
                double cx3 = cx + 40 * Math.Cos((ang_zF + 5) * Mathf.Deg2Rad);
                double cy3 = cy + 40 * Math.Sin((ang_zF + 5) * Mathf.Deg2Rad);
				GL.Vertex(new Vector3(Mathf.Clamp((float)cx1, windowPos.x, windowPos.x + offH),
				                      Mathf.Clamp((float)cy1, yoff, yoff + offH), 0));
				GL.Vertex(new Vector3(Mathf.Clamp((float)cx2, windowPos.x, windowPos.x + offH),
				                      Mathf.Clamp((float)cy2, yoff, yoff + offH), 0));
				GL.Vertex(new Vector3(Mathf.Clamp((float)cx3, windowPos.x, windowPos.x + offH),
				                      Mathf.Clamp((float)cy3, yoff, yoff + offH), 0));
                GL.End();
                GL.PopMatrix();
            }


            GUI.DragWindow(new Rect(0, 0, windowPos.width, 30));
        }
    }



































    class GUItargetCam : GUItab {
		
        private Material matShaderCurrent;
		private CameraModes cameraMode = CameraModes.Normal;
		private LazorCam lazorCam;
		private LazorCamGui lazorGui;
		private int camHeight = 200;
		private int camWidth = 200;


        public GUItargetCam(LazorSystem system) : base(system) {
			lazorCam = new LazorCam(256, 256, 24, RenderTextureFormat.Default, RenderTextureReadWrite.sRGB, 60, 32771);
			lazorGui = new LazorCamGui(lazorCam);
            matShaderCurrent = CameraFilter.normal;
        }

        public static Camera FindCamera(string name) {
            foreach (Camera c in Camera.allCameras)
                if (c.name == name)
                    return c;
            return null;
        }

        override public string Name { get { return "Target Cam"; } }
        override public string GetTabName() {
            return "Target Cam";
        }

        override public float GUIbutton(float indent, int width = 200, int height = 100) {
			return base.GUIbutton(indent, camWidth, camHeight);
        }

        //Draw the GUI window.
        override public void DrawGUI(string name) {
            GUI.skin = Util.LazorSkin;
            windowPos = GUILayout.Window(id, windowPos, WindowGUI, name, GUILayout.MinWidth(200));
            if (!docked)
                windowPos = Util.ClampToScreen(windowPos);
        }

		//called directly from Targets
		public void OnTargetSwitch() {
			lazorGui.ResetOffset();
			//posOffset = Vector3.zero;
		}

        override public void WindowGUI(int windowID) {
            Header();
			lazorGui.CameraGui(cameraMode, out camWidth, out camHeight, system.Leader.GetAim(), system.Leader.GetOrigin() + system.Leader.GetAim() * 0.4f, system.LocalUp, -system.LocalForward, system.Target);
        
            GUILayout.Space(5);
            cameraMode = (CameraModes)GUILayout.SelectionGrid((int)cameraMode, Enum.GetNames(typeof(CameraModes)), 2);
            GUILayout.Space(5);

			GUILayout.BeginHorizontal();
			GUILayout.Label("Zoom");
			lazorGui.sliderZoom = GUILayout.HorizontalSlider(lazorGui.sliderZoom, 0, 12);
			GUILayout.EndHorizontal();
          
            // Flight parameters
            if (system.Target != null) {
                if (system.Target is Vessel) {
                    Vessel v = (Vessel)system.Target;
                    GUILayout.BeginVertical();
                    GUILayout.BeginHorizontal(Util.Parameters_highlight);
                    GUILayout.Label("► ", Util.StdLabelSmallCustom(Color.green));
                    GUILayout.Label(v.vesselName, Util.StdLabelSmallCustom(Color.white));
                    GUILayout.Label(" ◄", Util.StdLabelSmallCustom(Color.green));
                    GUILayout.EndHorizontal();
                    system.VIT.X(system.VIT.param_Distance);
                    system.VIT.X(system.VIT.param_MissionTime);
                    system.VIT.X(system.VIT.param_Crew);
                    system.VIT.X(system.VIT.param_Situation);
                    system.VIT.X(system.VIT.param_SoI);
                    system.VIT.X(system.VIT.param_Altitude);
                    system.VIT.X(system.VIT.param_OrbitSpeed);
                    system.VIT.X(system.VIT.param_CurrentStage);
                    system.VIT.X(system.VIT.param_Pressure);
                    system.VIT.X(system.VIT.param_Apoapsis);
                    system.VIT.X(system.VIT.param_TimeToApoapsis);
                    system.VIT.X(system.VIT.param_Periapsis);
                    system.VIT.X(system.VIT.param_TimeToPeriapsis);
                    system.VIT.X(system.VIT.param_Period);
                    system.VIT.X(system.VIT.param_Eccentricity);
                    system.VIT.X(system.VIT.param_Inclination);
                    GUILayout.EndVertical();
                }
                if (system.Target is CelestialBody) {
                    //CelestialBody cb = system.VIT.lp.cb = (CelestialBody)system.Target;
                    CelestialBody cb = (CelestialBody)system.Target;
                    system.VIT.lp.cb = (CelestialBody)system.Target;
                    GUILayout.BeginVertical();
                    GUILayout.BeginHorizontal(Util.Parameters_highlight);
                    GUILayout.Label("► ", Util.StdLabelSmallCustom(Color.green));
                    GUILayout.Label(cb.theName, Util.StdLabelSmallCustom(Color.white));
                    GUILayout.Label(" ◄", Util.StdLabelSmallCustom(Color.green));
                    GUILayout.EndHorizontal();
                    system.VIT.X(system.VIT.param_CB_Distance);
                    system.VIT.X(system.VIT.param_CB_Radius);
                    system.VIT.X(system.VIT.param_CB_Mass);
                    system.VIT.X(system.VIT.param_CB_GravParam);
                    system.VIT.X(system.VIT.param_CB_ASLGee);
                    system.VIT.X(system.VIT.param_CB_HasAtmosphere);
                    if (cb.atmosphere) {
                        system.VIT.X(system.VIT.param_CB_AtmosAlt);
                        system.VIT.X(system.VIT.param_CB_ASLPressure);
                        system.VIT.X(system.VIT.param_CB_ASLDensity);
                    } else if (cb.theName == "the Sun") {
                        system.VIT.X(system.VIT.param_CB_ASLPressure);
                    }
                    system.VIT.X(system.VIT.param_CB_ASLTemperature);
                    system.VIT.X(system.VIT.param_CB_HasOxygen);
                    system.VIT.X(system.VIT.param_CB_HasOcean);
                    system.VIT.X(system.VIT.param_CB_IsTidallyLocked);
                    system.VIT.X(system.VIT.param_CB_IsRotating);
                    if (cb.rotates) {
                        system.VIT.X(system.VIT.param_CB_RotationPeriod);
                        system.VIT.X(system.VIT.param_CB_IsRotatingInverse);
                    }
                    if (cb.theName != "the Sun"){ //cb.orbit != null) { //its null for the Sun
                        system.VIT.X(system.VIT.param_CB_OrbitPeriod);
                        system.VIT.X(system.VIT.param_CB_OrbitVelocity);
                        system.VIT.X(system.VIT.param_CB_OrbitEccentricity);
                        system.VIT.X(system.VIT.param_CB_OrbitInclination);
                        system.VIT.X(system.VIT.param_CB_MeanAnomaly);
                        system.VIT.X(system.VIT.param_CB_EccentricAnomaly);
                        system.VIT.X(system.VIT.param_CB_TrueAnomaly);
                    }
                    GUILayout.EndVertical();
                }

                if (system.Target is Collider || system.Target is BoxCollider || system.Target is MeshCollider) {
                    Collider c = (Collider)system.Target;
                    GUILayout.BeginVertical();
                    GUILayout.BeginHorizontal(Util.Parameters_highlight);
                    GUILayout.Label("► ", Util.StdLabelSmallCustom(Color.green));
                    GUILayout.Label(c.name, Util.StdLabelSmallCustom(Color.white));
                    GUILayout.Label(" ◄", Util.StdLabelSmallCustom(Color.green));
                    GUILayout.EndHorizontal();
                    GUILayout.BeginHorizontal(Util.Parameters_normal);
                    Util.DistanceFormatted((c.transform.position - system.vessel.transform.position).magnitude, Color.green, "Distance: ");
                    GUILayout.EndHorizontal();
                    GUILayout.EndVertical();
                }
            }

            Footer();
        }



        public void onFlightStateLoad(Dictionary<string, KSPParseable> parsedData) {
            onFlightStateLoad(parsedData, "GUItargetCam");
        }

        public void onFlightStateSave(Dictionary<string, KSPParseable> partDataCollection) {
            onFlightStateSave(partDataCollection, "GUItargetCam");
        }



    }
}



