﻿/* Author: Jim-Kyrre Benjaminsen AKA Romfarer
 * License: This work is licensed under a Creative Commons Attribution-NonCommercial-NoDerivs 3.0 Unported License. 
 * http://creativecommons.org/licenses/by-nc-nd/3.0/
*/

using System;
using System.Collections.Generic;
using System.Text;
using UnityEngine;

namespace Romfarer {

    class Lazor : IAimableLazor{

        //private LazorSystem system;
		private Part part;

        GameObject lazorObject;
        private LineRenderer lazor;
        private Vector3 lazorOriginPoint;
        private Ray ray;
        public RaycastHit hit;
        private int nReflections = 1; //the number of reflections  
        private int nPoints; //the number of points at the line renderer  
        private Vector3 inDirection; //reflection direction  
        public float range = 500;
        private Color color;
		private bool internalStateOn; //the state set in On() and Off()
		public bool lazorStateInitiated, safeMode; 

		private bool lazorIsHittingSomething;
        public bool LazorIsHittingSomething { get { return lazorIsHittingSomething; } }
		private bool lazorIsHittingVessel;
		public bool LazorIsHittingVessel { get { return lazorIsHittingVessel; } }
		private bool lazorIsHittingOwnVessel;
		public bool LazorIsHittingOwnVessel { get { return lazorIsHittingOwnVessel; } }

        public bool lazorOn { get { return internalStateOn; } }
        private Vessel vesselHit;
		public Vessel VesselHit { get { return vesselHit; } } //if the lazor is hitting a vessel this is the vessel, can be null
        private Texture2D texLazorCam;
        public Texture2D TexLazorCam { get { return texLazorCam; } }
        public Texture2D textureColor;
        public Vector3 horizontalAim = Vector3.zero;
        public bool isInverse; //counterparts on planes are inverse
        private Transform transform_base; //the transform the line renderer is connected to

		private Transform revolver;
		private bool isLazorAimable;
        public Lazor() {}
		public void Initialize_WithPart(Part part, Transform revolver, Transform t, Vector3 origin, Color color) {
			this.part = part;
			this.revolver = revolver;
			this.isLazorAimable = true;
			Initialize(null, t, origin, color);
		}
        public void Initialize(LazorSystem system, Transform t, Vector3 origin, Color color) {
            transform_base = t;
            lazorOriginPoint = origin;

            // Main lazor
            lazorObject = new GameObject("Lazor");
            lazorObject.transform.parent = t;
            lazorObject.transform.localEulerAngles = Vector3.zero;
            lazor = lazorObject.AddComponent<LineRenderer>();
            lazor.transform.parent = t;
            lazor.useWorldSpace = false;
            lazor.transform.localPosition = lazorOriginPoint;
            lazor.transform.localEulerAngles = Vector3.zero;
            lazor.material = new Material(Shader.Find("Particles/Additive"));
            lazor.SetWidth(0.015f, 0.015f);
            lazor.SetVertexCount(2);
            lazor.SetPosition(0, Vector3.zero);
            lazor.SetPosition(1, t.up * 100);
            lazor.enabled = false;
            this.color = color;
            texLazorCam = Util.GetCrossHairHitPoint(color);
            textureColor = Util.LightTexture(Util.getTexture(new Color(color.r, color.g, color.b, 0.9f)), 0.6f);

            //check if inverse
			if (system != null){
				part = system;
				if(system.IsHorizontal)
	                if (Vector3.Dot(t.right, system.LocalUp) > 0)
	                    isInverse = true;
			}
			Vector3 gUp = (part.vessel.transform.position - part.vessel.mainBody.position).normalized;
            float angle = 90 - Vector3.Angle(lazor.transform.up, gUp);
            horizontalAim = Quaternion.Euler(isInverse ? -angle : angle, 0, 0) * lazor.transform.up;
        }


        // IColorable
        public Color GetColor() {
            return color;
        }

        // IColorable
        public void SetColor(Color color) {
            this.color = color;
        }

		private float alpha;
        public void SetRange(float range) {
            this.range = range;
			//this.alpha = 0.15f + range * 0.00001f;
			this.alpha = 0.25f + range / LazorGlobals.VIEW_DISTANCE;
        }

        public void On() {
            if (!lazorStateInitiated) {
                lazor.SetPosition(1, lazor.transform.InverseTransformDirection(transform_base.up) * 100);
                lazorStateInitiated = true;
            }
            lazor.enabled = true;
            internalStateOn = true;
			if(part is LazorAimable)
				part.enabled = true;
        }

        public void Off() {
            if (lazor != null) 
                lazor.enabled = false; //0.15 thing
            internalStateOn = false;
			if(part is LazorAimable)
				part.enabled = false;
        }

        virtual public void SetLocalEulerAngles(Vector3 euler) { //Transform of your aimable object
            lazor.transform.localEulerAngles = euler;
        }

        virtual public void SetEulerAngles(Vector3 euler) { //Transform of your aimable object
            lazor.transform.eulerAngles = euler;
        }

        virtual public Transform GetTransform() { //Transform of your aimable object
            return lazor.transform;
        }

        virtual public Transform GetTransformBase() { //Transform of your aimable object
            return transform_base;
        }

        public void ResetOrientation() {
            Aim(transform_base.up);
        }

        public void SetOrigin(Vector3 o) {
            lazor.transform.position = o;
        }

        // Interface: IAimable
        public void SetControlledStatus(bool cs) {
            //controlledStatus = cs;
        }
        // Interface: IAimable
        public Vector3 GetOrigin() {
            return lazor.transform.position;
        }

        // Interface: IAimable
        public Vector3 GetAim() {
            return lazor.transform.up;
        }

        // Interface: IAimable
        public void Aim(Vector3 dir) {
            lazor.transform.up = dir;
			if(isLazorAimable)
				Util.RotateAroundAxis(GetAim(), Vector3.up, revolver, part.transform);
        }

        //hit stuff
        public RaycastHit GetHit() {
            return hit;
        }

        public Vector3 GetHitPoint() {
            return hit.point;
        }

		public float HitDistance() {
			if (lazorIsHittingSomething)
				return hit.distance;
			else
				return float.PositiveInfinity;
		}

        public Component GetHitVesselOrCollider() {
            if (lazorIsHittingSomething && lazorOn) {
                if (lazorIsHittingVessel)
                    return hit.collider.attachedRigidbody.GetComponent<Part>().vessel;
                else
                    return hit.collider;
            } else
				return null;
        }

		public Part GetHitPart() {
			if (lazorOn)
				return hit.collider.attachedRigidbody.GetComponent<Part>();
			return null;
		}

		virtual public void onUpdate() {
			//if (lazor == null) 
			//	return;
			lazor.enabled = internalStateOn; // Turn modules on/off
			lazorIsHittingOwnVessel = false;
			Color c = new Color(color.r, color.g, color.b, alpha);
			lazor.SetColors(c, c);
			nPoints = nReflections;
			ray = new Ray(lazor.transform.position, lazor.transform.up);
			if (Physics.Raycast(ray.origin, ray.direction, out hit, range, 557059)) {
				lazorIsHittingSomething = true;
				if (hit.collider.attachedRigidbody != null) {
					Part hitPart = hit.collider.attachedRigidbody.GetComponent<Part>();
					if (hitPart != null) {
						vesselHit = hitPart.vessel;
						lazorIsHittingVessel = VesselHit != null;
						if (vesselHit == part.vessel) {
							lazorIsHittingOwnVessel = true;
							if(safeMode){
								lazor.enabled = false;
								return;
							}
						}
					} else {
						lazorIsHittingOwnVessel = false;
						lazorIsHittingVessel = false;
						vesselHit = null;
					}
				} else {
					lazorIsHittingOwnVessel = false;
					lazorIsHittingVessel = false;
					vesselHit = null;
				}
				lazor.SetVertexCount(3);
				inDirection = Vector3.Reflect(ray.direction, hit.normal);
				ray = new Ray(hit.point, inDirection);
				lazor.SetPosition(1, lazor.transform.InverseTransformPoint(hit.point));
				lazor.SetPosition(2, lazor.transform.InverseTransformPoint(ray.GetPoint(range - hit.distance)));
			} else {
				lazorIsHittingOwnVessel = false;
				lazorIsHittingVessel = false;
				vesselHit = null;
				lazorIsHittingSomething = false;
				lazor.SetVertexCount(2);
				lazor.SetPosition(1, lazor.transform.InverseTransformPoint(ray.GetPoint(range)));
			}
		}
		/*
        virtual public void onUpdate() {
            //if (lazor == null) 
			//	return;
            lazor.enabled = internalStateOn; // Turn modules on/off
            lazorIsHittingOwnVessel = false;
            Color c = new Color(color.r, color.g, color.b, alpha);
            lazor.SetColors(c, c);
            nPoints = nReflections;
            ray = new Ray(lazor.transform.position, lazor.transform.up);
            for (int i = 1; i <= nReflections; i++) {
                if (i == 1) {
                    if (Physics.Raycast(ray.origin, ray.direction, out hit, range, 557059)) {
                        lazorIsHittingSomething = true;
                        if (hit.collider.attachedRigidbody != null) {
                            Part hitPart = hit.collider.attachedRigidbody.GetComponent<Part>();
                            if (hitPart != null) {
                                vesselHit = hitPart.vessel;
                                if (VesselHit != null)
                                    lazorIsHittingVessel = true;
                                else {
                                    lazorIsHittingVessel = false;
                                }
                                if (vesselHit == part.vessel) {
                                    lazorIsHittingOwnVessel = true;
									if(safeMode)
                                    	lazor.enabled = false;
                                    return;
                                }
                            } else {
                                lazorIsHittingOwnVessel = false;
                                lazorIsHittingVessel = false;
                                vesselHit = null;
                            }
                        } else {
                            lazorIsHittingOwnVessel = false;
                            lazorIsHittingVessel = false;
                            vesselHit = null;
                        }

                        lazor.SetVertexCount(i + 2);
                        inDirection = Vector3.Reflect(ray.direction, hit.normal);
                        ray = new Ray(hit.point, inDirection);
                        lazor.SetPosition(i, lazor.transform.InverseTransformPoint(hit.point));
                        lazor.SetPosition(i + 1, lazor.transform.InverseTransformPoint(ray.GetPoint(range - hit.distance)));
                    } else {
                        lazorIsHittingOwnVessel = false;
                        lazorIsHittingVessel = false;
                        vesselHit = null;
                        lazorIsHittingSomething = false;
                        lazor.SetVertexCount(2);
                        lazor.SetPosition(1, lazor.transform.InverseTransformPoint(ray.GetPoint(range)));
                    }
                }
            }
        }*/

    }





    //**************************************************************************************************************
    //**************************************************************************************************************
    //**************************************************************************************************************
    //**************************************************************************************************************
    //**************************************************************************************************************











    class LazorAimable : Part, IModule {

        private Lazor lazor;
        public Lazor Lazor { get { return lazor; } }
        public String lazorColor = "none";
		private bool initiated;
		private Vector3 STANDARD_ORIGIN = new Vector3(0, 0.08f, 0); //the position of the lazor origin point, in local space
		private LazorSystem system; //the system controlling this lazor

        public void ActivateIcon() {
            try {
                stackIcon.SetIcon(this, "icons.png", 65, 0);
            } catch (Exception) { }
        }

        protected override void onPartStart() {
            ActivateIcon();
        }

        protected override void onPartFixedUpdate(){
			if(!initiated) {
				enabled = false;
				return;
			}
            if (Lazor.lazorOn)
				Lazor.onUpdate();
        }
	
        public void Initialize(LazorSystem system) {
			this.system = system;
			if(!initiated) {
				Color color = Color.white;
				switch(lazorColor) {
					case "green":
						color = Color.green;
						break;
					case "blue":
						color = Color.blue;
						break;
					case "red":
						color = Color.red;
						break;
					case "yellow":
						color = Color.yellow;
						break;
					case "magenta":
						color = Color.magenta;
						break;
				}
				lazor = new Lazor();
				lazor.Initialize_WithPart(this, transform.Find("model/lazor/Lazor"), transform, STANDARD_ORIGIN, color);
				lazor.safeMode = true;
				initiated = true;
			}
        }

		//Called when switching active lazor system. 
		public void Claim(LazorSystem system) {
			this.system = system;
		}

    }
}

