/* Author: Jim-Kyrre Benjaminsen AKA Romfarer
 * License: This work is licensed under a Creative Commons Attribution-NonCommercial-NoDerivs 3.0 Unported License. 
 * http://creativecommons.org/licenses/by-nc-nd/3.0/
*/

using System;
using UnityEngine;

namespace Romfarer {
	public class PartMissile : Part {
		
		//.cgf MODULE LazorWarhead
		public float blastRadius = 50;
		public float destructionRadius = 0;

		//.cfg MODULE LazorControl
		public bool receiveControl;
		public float maxThrust = 0;
		public float ejectionForce = 0;
		public float burnTime = 0;

		//internal
		public double activationTime = 0;
		private bool iconSetEditor;
		private bool iconSet;
		private LazorSystemMissile missileModule;
		public Transform transform_missile;
		
		private Explosion explosion_effect;

		
		[KSPAction("Deploy", KSPActionGroup.None)]
		public void Fire(KSPActionParam param) {
			force_activate();
		}
		
		protected override void onEditorUpdate() {
			if (!iconSetEditor) {
				try {
					base.stackIcon.SetIcon(this, "icons.png", 260, 0);
				} catch (Exception) { }
				iconSetEditor = true;
			}
		}
		
		protected override void onPartFixedUpdate() {
			if (FlightGlobals.ActiveVessel == vessel) {
				if (!iconSet) {
					try {
						base.stackIcon.SetIcon(this, "icons.png", 260, 0);
					} catch (Exception) { }
					iconSet = true;
				}
			} else
				iconSet = false;
		}
		
		protected override void onPartStart(){
			try {
				base.stackIcon.SetIcon(this, "icons.png", 260, 0);
			} catch(Exception) {
			}
		}


		protected override void onFlightStart(){
			transform_missile = transform.Find("model/missile");
			if(!Modules.Contains("LazorControlT2")) {
				AddModule("LazorControlT2");
				Modules["LazorControlT2"].Fields.SetValue<float>("maxTorque ", 1);
				Modules["LazorControlT2"].Fields.SetValue<float>("maxThrust", 35);
				Modules["LazorControlT2"].Fields.SetValue<float>("ejectionForce", 5);
				Modules["LazorControlT2"].Fields.SetValue<float>("burnTime", 45);
			}
			maxThrust = Modules["LazorControlT2"].Fields.GetValue<float>("maxThrust");
			ejectionForce = Modules["LazorControlT2"].Fields.GetValue<float>("ejectionForce");
			burnTime = Modules["LazorControlT2"].Fields.GetValue<float>("burnTime");
			Modules["LazorControlT2"].Fields.SetValue<bool>("receiveControl", false);

			if(Modules.Contains("LazorWarhead")) {
				blastRadius = Modules["LazorWarhead"].Fields.GetValue<float>("blastRadius");
				destructionRadius = Modules["LazorWarhead"].Fields.GetValue<float>("destructionRadius");
			}
			//bug
			if(!Modules.Contains("LazorSystemMissile")) {
				missileModule = AddModule("LazorSystemMissile") as LazorSystemMissile;
				Modules["LazorSystemMissile"].Fields.SetValue<Vector3>("cameraPosition", new Vector3(0, 0.92f, 0));
			}
			Modules["LazorSystemMissile"].Events.Send("Initialize");

			explosion_effect = gameObject.AddComponent<Explosion>() as Explosion;
			explosion_effect.Initialize(transform, blastRadius);
		}

		private bool launched;
		protected override bool onPartActivate() {
			decouple();
			Transform attach = transform.Find("model/missile/decoupler");
			attach.gameObject.SetActive(false);
			rigidbody.AddForce(transform.forward * ejectionForce);
			activationTime = vessel.missionTime;
			Modules["LazorControlT2"].Fields.SetValue<bool>("receiveControl", true);
			vessel.distancePackThreshold = 95000;
			launched = true;
			return true;
		}

		protected override void onPack(){
			if (launched)
				GameObject.Destroy (vessel);
		}

		protected override void onActiveFixedUpdate() {
			if (vessel.missionTime < activationTime + burnTime)
				rigidbody.AddForce(this.transform.up * maxThrust);
			else
				deactivate();
		}
		
		protected override void onPartDeactivate() {
			Modules["LazorControlT2"].Fields.SetValue<bool>("receiveControl", false);
		}

		protected override void onPartExplode() {
			explosion_effect.Explode(transform.position);
			float power = 1000.0f;
			Vector3 explosionPos = transform.position;

			Collider[] colliders = Physics.OverlapSphere(explosionPos, blastRadius );
			foreach (Collider hit in colliders) {
				if (hit.attachedRigidbody != null) {
					Part hitPart = hit.collider.attachedRigidbody.GetComponent<Part>();
					hitPart.vessel.GoOffRails();
					hit.attachedRigidbody.AddExplosionForce(power, explosionPos, blastRadius, 0.0f);
				}
			}

			colliders = Physics.OverlapSphere(explosionPos, destructionRadius);
			foreach (Collider hit in colliders) {
				if (hit.attachedRigidbody != null) {
					hit.collider.attachedRigidbody.GetComponent<Part>().explode();
				}
			}
		}
		

	}































	
	class LazorSystemMissile : PartModule{

		[KSPField]
		public Vector3 cameraPosition = Vector3.zero;

		private int camHeight = 200;
		private int camWidth = 200;
		private Rect windowPos;
		public bool on;
		private LazorCam lazorCam;
		private LazorCamGui lazorGui;
		public float zoom{ get { return lazorGui.sliderZoom; } }
		private MovementControl mc = new MovementControl();
		public AimingDevice aimer;
		Vector3 originAtLense = Vector3.zero;
		private DoubleSwitch toggleFix;
		private PartMissile partMissile;
		private Transform transform_missile;
		private PIDclamp p1 = new PIDclamp("Missile p1", 0.102f, 0.055f, 0.06f, 1f);
		private PIDclamp p2 = new PIDclamp("Missile p2", 0.102f, 0.055f, 0.06f, 1f);
		//GameObject debug;


		[KSPEvent(guiActive = true, guiName = "Activate Camera")]
		public void ActivateLazorMissileCamera() {
			Events["DeactivateLazorMissileCamera"].active = true;
			Events["ActivateLazorMissileCamera"].active = false;
			on = true;
		}
		
		[KSPEvent(guiActive = true, guiName = "Deactivate Camera")]
		public void DeactivateLazorMissileCamera() {
			Events["DeactivateLazorMissileCamera"].active = false;
			Events["ActivateLazorMissileCamera"].active = true;
			on = false;
		}
		
		public override void OnStart(PartModule.StartState state) {
			if (vessel != null) {
				Events["DeactivateLazorMissileCamera"].active = false;
				Events["ActivateLazorMissileCamera"].active = true;
			}
		}
		
		public override void OnAwake() {
			if (vessel != null) {
				Events["DeactivateLazorMissileCamera"].active = false;
			}
		}
		

		[KSPEvent]
		public void Initialize(){
			lazorCam = new LazorCam(256, 256, 24, RenderTextureFormat.Default, RenderTextureReadWrite.sRGB, 60, 557059); //32771
			lazorGui = new LazorCamGui(lazorCam);
			partMissile = (part as PartMissile);
			transform_missile = partMissile.transform_missile;
			aimer = new AimingDevice(partMissile.transform, 
			                         partMissile.transform,
			                         partMissile.transform.up, 
			                         partMissile.transform.position, 
			                         -partMissile.transform.forward, 
			                         0.1f, 1.0f, 150f);
			toggleFix.set = false;
			aimer.Fix(false);
			RenderingManager.AddToPostDrawQueue(0, new Callback(CamGUI));
			part.OnJustAboutToBeDestroyed += Dispose;

			//debug = Util.Cross();
		}

		
		private void Dispose() {
			on = false;
			RenderingManager.RemoveFromPostDrawQueue(0, new Callback(CamGUI));
		}

		public override void OnUpdate(){
			if(on) {
				Vector3 normal = -FlightGlobals.getGeeForceAtPosition(vessel.transform.position);
				if(toggleFix && LazorSystemControllerGlobal.ActiveLazorSystemTarget != null){
					aimer.SetLockDirection ((LazorSystemControllerGlobal.ActiveLazorSystemTarget.transform.position - transform.position).normalized, normal);
				} else if(toggleFix && FlightGlobals.fetch.VesselTarget != null){
					aimer.SetLockDirection ((FlightGlobals.fetch.VesselTarget.GetTransform().position - transform.position).normalized, normal);
				}else{
					aimer.SetLockDirection(partMissile.transform.up, normal);
				}
				aimer.CorrectDeadzone();
				originAtLense = part.transform.TransformPoint(cameraPosition);
				//debug.transform.position = originAtLense;
			}
		}

		public override void OnFixedUpdate() {
			if(!LazorSystemControllerGlobal.ActiveVesselHasLazorSystem() && FlightGlobals.fetch.VesselTarget != null) {
				Vector3 torqueOut = Controllers.MissileControl(partMissile, (Component)FlightGlobals.fetch.VesselTarget,  partMissile.transform.up, p1, p2, 0f);
				BaseEventData data = new BaseEventData(BaseEventData.Sender.USER);
				data.Set<Vector3>("vector", torqueOut);
				part.Modules.Send("AddTorque", data);
			}
		}
		
		public void CamGUI() {
			UnityEngine.GUI.skin = Util.LazorSkin;
			if (!FlightDriver.Pause && on) {
				windowPos = Util.ClampToScreen(GUILayout.Window(GetInstanceID(), windowPos, WindowGUI, "Missile Cam"));
				windowPos.width = camWidth;
			}
		}

		public void WindowGUI(int windowID){
			if(GUI.Button(new Rect(windowPos.width - 20, 0, 20, 20), "", Util.Button_header_close)) { //red close button
				on = false;
				Events["ActivateLazorMissileCamera"].active = true;
				Events["DeactivateLazorMissileCamera"].active = false;
			}
			if(part != null && !part.packed) {

				lazorGui.CameraGui(CameraModes.Normal, out camWidth, out camHeight, aimer.forward, originAtLense, aimer.up, aimer.up, null);
				
				GUI.enabled = part.isConnected;
				if(GUILayout.Button("Deploy"))
					part.force_activate();
				GUI.enabled = true;

				if(GUILayout.Button("Detonate"))
					part.explode();
				
				toggleFix.set = GUILayout.Toggle(toggleFix, "Lock cam on target");

				GUILayout.BeginVertical();

				GUILayout.BeginHorizontal(Util.Parameters_normal);
				Util.DistanceFormatted((transform.position - FlightGlobals.ActiveVessel.transform.position).magnitude, LazorGlobals.COLOR_GREENISH, "d' vessel - missile: ");
				GUILayout.EndHorizontal();
				if(LazorSystemControllerGlobal.ActiveLazorSystemTarget != null) {
					GUILayout.BeginHorizontal(Util.Parameters_normal);
					Util.DistanceFormatted((LazorSystemControllerGlobal.ActiveLazorSystemTarget.transform.position - transform.position).magnitude, LazorGlobals.COLOR_GREENISH, "d'  missile - target: ");
					GUILayout.EndHorizontal();
				} else if(FlightGlobals.fetch.VesselTarget != null) {
					GUILayout.BeginHorizontal(Util.Parameters_normal);
					Util.DistanceFormatted((FlightGlobals.fetch.VesselTarget.GetTransform().position - transform.position).magnitude, LazorGlobals.COLOR_GREENISH, "d' missile - target: ");
					GUILayout.EndHorizontal();
				}
				GUILayout.BeginHorizontal(Util.Parameters_normal);
				GUILayout.Label("Velocity: ", Util.StdLabelSmallCustom(LazorGlobals.COLOR_GREENISH));
				GUILayout.Label((part.vessel.GetObtVelocity() - FlightGlobals.ActiveVessel.GetObtVelocity()).magnitude.ToString("f2"), Util.StdLabelSmallCustom(Color.white));
				GUILayout.Label("m/s", Util.StdLabelSmallCustom(LazorGlobals.COLOR_GREENISH));
				GUILayout.EndHorizontal();

				GUILayout.BeginHorizontal(Util.Parameters_normal);
				GUILayout.Label("Depletion T-: ", Util.StdLabelSmallCustom(LazorGlobals.COLOR_GREENISH));
				double burntime = partMissile.burnTime - (partMissile.vessel.missionTime - partMissile.activationTime);
				GUILayout.Label(Util.SecondsAsTimeString(burntime>0?burntime:0), Util.StdLabelSmallCustom(Color.white));
				GUILayout.EndHorizontal();

				GUILayout.EndVertical();
			}else
				on = false;

			//cameraPosition = Util.MovePoint(cameraPosition, 0.001f);
			//GUILayout.Label("vessel - missile: " + transform.position - FlightGlobals.ActiveVessel.transform.position);
			//GUILayout.Label("missile - target: " + LazorGlobals.ActiveLazorSystemTarget.transform.position - transform.position);


			GUI.DragWindow(new Rect(0, 0, windowPos.width, 30));
		}


	}
	











	
	public class Explosion : MonoBehaviour {

		private ParticleEmitter emitter_debris;
		private ParticleEmitter emitter_explosion;
		private Material smoke;
		private Transform parent;
		private float radius; //90, 350, 2500

		public void Initialize(Transform parent, float radius) {
			this.parent = parent;
			transform.parent = parent;
			this.radius = radius;

			//Debris
			GameObject go_debris = new GameObject("LazorEffect Debris");
			emitter_debris = go_debris.AddComponent("EllipsoidParticleEmitter") as ParticleEmitter;
			ParticleAnimator animator_debris = go_debris.AddComponent<ParticleAnimator>();
			go_debris.AddComponent<ParticleRenderer>();
			(go_debris.renderer as ParticleRenderer).uvAnimationXTile = 2;
			(go_debris.renderer as ParticleRenderer).uvAnimationYTile = 2;
			Material mat = new Material(Shader.Find("Particles/Additive"));
			mat.mainTexture = Util.LoadTexture(64, 64, "particle_debris.png");
			go_debris.renderer.material = mat;
			emitter_debris.emit = false;
			emitter_debris.minSize = 4 + radius/200; //radius * 0.05f;  //4    3 + 
			emitter_debris.maxSize = 5 + radius/200; //radius * 0.1f;  //8
			emitter_debris.minEnergy = 1;
			emitter_debris.maxEnergy = 4;
			emitter_debris.rndVelocity = Vector3.one * (150 + radius/90 * 40); //1.6f * radius; //150
			emitter_debris.useWorldSpace = false;
			emitter_debris.rndAngularVelocity = 500;
			animator_debris.rndForce = new Vector3(400,10,400);
			animator_debris.sizeGrow = -0.1f;

			//Smoke
			GameObject go_explosion = new GameObject("LazorEffect Explosion");
			emitter_explosion = go_explosion.AddComponent("EllipsoidParticleEmitter") as ParticleEmitter;
			ParticleAnimator animator_explosion = go_explosion.AddComponent<ParticleAnimator>();
			go_explosion.AddComponent<ParticleRenderer>();
			smoke = new Material(Shader.Find("Particles/Additive"));
			smoke.mainTexture = Util.LoadTexture(1024, 1024, "particle_smoke.png");
			go_explosion.renderer.material = smoke;
			(go_explosion.renderer as ParticleRenderer).uvAnimationXTile = 4;
			(go_explosion.renderer as ParticleRenderer).uvAnimationYTile = 4;
			emitter_explosion.emit = false;
			emitter_explosion.minSize = radius; //120
			emitter_explosion.maxSize = radius * 1.2f; //180
			emitter_explosion.minEnergy = 0.5f;
			emitter_explosion.maxEnergy = 2.5f;
			emitter_explosion.rndVelocity = Vector3.one * 50;
			emitter_explosion.useWorldSpace = false;
			animator_explosion.rndForce = new Vector3(10,10,10);
			animator_explosion.sizeGrow = -0.1f;
			Color[] colors = animator_explosion.colorAnimation;
			colors[0] = new Color(1f, 1f, 1f, 1f);
			colors[1] = new Color(1f, 1f, 1f, 0.9f);
			colors[2] = new Color(1f, 1f, 1f, 0.8f);
			colors[3] = new Color(1f, 1f, 1f, 0.7f);
			colors[4] = new Color(1f, 1f, 1f, 0.5f);
			animator_explosion.colorAnimation = colors;
		}
		
		public void Explode(Vector3 position){
			emitter_debris.transform.position = position;
			emitter_explosion.transform.position = position;
			emitter_debris.Emit(500);	
			emitter_explosion.Emit(10);

			/*MonoBehaviour.print("pp  "+parent.position);
			MonoBehaviour.print("p   "+position);
			MonoBehaviour.print("tp  "+transform.position);
			MonoBehaviour.print("got "+gameObject.transform.position);
			MonoBehaviour.print("emd "+emitter_debris.transform.position);
			MonoBehaviour.print("eme "+emitter_explosion.transform.position);
			MonoBehaviour.print("eme "+emitter_explosion.gameObject.transform.position);*/

			GameObject o = new GameObject("LazorMissile Spark Handler");
			int sparks = UnityEngine.Random.Range((int)Math.Sqrt(radius), 20 + (int)Math.Sqrt(radius));
			for(int i = 0; i < sparks; i++){
				Spark spark = o.AddComponent<Spark>();
				spark.Initialize(smoke, transform, 16 + radius/90);
				spark.Explode(UnityEngine.Random.insideUnitSphere * (25000 + (radius/90 * 1000))); //30000
				//MonoBehaviour.print("s   "+spark.transform.position);
			}
		}

		
	}
	
	
	
	
	
	
	
	
	
	
	
	
	public class Spark : MonoBehaviour{
		private ParticleEmitter emitter_spark;
		private GameObject go_spark;
		private float emitTime = 0;
		private float flyTime = 5;
		
		public void Initialize(Material mat, Transform parent, float size){
			transform.position = parent.position;
			go_spark = new GameObject("LazorEffect Spark");
			go_spark.transform.position = parent.position;
			go_spark.AddComponent<Rigidbody>();
			go_spark.rigidbody.mass = 1;
			//go_spark.AddComponent<BoxCollider>();
			emitter_spark = go_spark.AddComponent("EllipsoidParticleEmitter") as ParticleEmitter;
			ParticleAnimator animator_spark = go_spark.AddComponent<ParticleAnimator>();
			go_spark.AddComponent<ParticleRenderer>();
			go_spark.renderer.material = mat;
			(go_spark.renderer as ParticleRenderer).uvAnimationXTile = 4;
			(go_spark.renderer as ParticleRenderer).uvAnimationYTile = 4;
			emitter_spark.emit = false;
			emitter_spark.minSize = size; //20
			emitter_spark.maxSize = size * 1.25f; //25
			emitter_spark.minEnergy = 1f;
			emitter_spark.maxEnergy = 1.2f;
			emitter_spark.minEmission = 50;
			emitter_spark.maxEmission = 50;
			animator_spark.rndForce = new Vector3(10,10,10);
			animator_spark.sizeGrow = -0.75f;
		}
		
		public void Explode(Vector3 force){
			emitter_spark.emit = true;
			go_spark.rigidbody.AddForce(force);
			emitTime = Time.time;
		}

		public void FixedUpdate(){
			emitter_spark.minSize *= 0.98f;
			emitter_spark.maxSize *= 0.98f;
		}

		public void Update(){
			if(emitTime+flyTime<Time.time){
				UnityEngine.Object.Destroy(go_spark);
				UnityEngine.Object.Destroy(this);
			}
		}
	}






}






