/* Author: Jim-Kyrre Benjaminsen AKA Romfarer
 * License: This work is licensed under a Creative Commons Attribution-NonCommercial-NoDerivs 3.0 Unported License. 
 * http://creativecommons.org/licenses/by-nc-nd/3.0/
*/

//#define LAZORDEBUG

using System;
using System.Linq;
using System.Collections.Generic;
using UnityEngine;
using System.Text.RegularExpressions;



namespace Romfarer
{



	[KSPAddon(KSPAddon.Startup.SpaceCentre, true)]
	public class RuntimeLogic : MonoBehaviour {


		public static RuntimeLogic Instance;
		public static string ApplicationPath = "Romfarer/";
		private bool inFlight { get { return HighLogic.LoadedSceneIsFlight && FlightGlobals.fetch != null && FlightGlobals.ready; } }
		//private DoubleSwitch timeWarp;

		//launch clamps
		/*private bool launchClampsDetected;
		private bool launching;
		private Dictionary<Guid, List<Part>> destroyOnLiftoff = new Dictionary<Guid, List<Part>>();
*/

		//DEBUG
		//private bool DEBUG = true;
		private bool DEBUG_gui_on;
		private bool DEBUG_hierarchy;
		private bool DEBUG_log;
		private DoubleSwitch DEBUG_joints;
		private List<Joint> joints = new List<Joint>();
		private Rect windowPos_DEBUG = new Rect(160,0,500,100);
		private Vector2 scrollArea = Vector2.zero;

		private bool showSpaceCenterGUI = true;


		public void Awake() {
			UnityEngine.GameObject.DontDestroyOnLoad(gameObject);
		}

		public void Start() {
			if (Instance != null) {
				GameObject.DestroyImmediate (gameObject);
				return;
			}
			Instance = this;

			MonoBehaviour.print("LazorSystem RuntimeLogic Started");

			//find the application sub path within the GameData folder
			string path = AssemblyLoader.GetPathByType (typeof(RuntimeLogic));
			string subPath = path.Substring (Application.dataPath.Length + 1);
			//print("path:::    " + AssemblyLoader.GetPathByType (typeof(RuntimeLogic)));
			//print("datapath:::    " + Application.dataPath);
			//print("subpath:::    " + subPath);

			string subPathCompiled = "";
			char[] delimiters = new char[]{'\\','/'};
			string[] dirs = subPath.Split (delimiters);
			int i = 0;
			foreach (string s in dirs) {
				if(++i<2)
					subPathCompiled = UrlDir.PathCombine (subPathCompiled, s);
				//print(s);
				//print("x -> " + subPathCompiled);
			}
			subPathCompiled = UrlDir.PathCombine (subPathCompiled, "X");
			subPathCompiled = subPathCompiled.Substring(0, subPathCompiled.Length-1);
		
			ApplicationPath = subPathCompiled.Replace("\\","/");

			/*string pattern = "(.*\\n?)(?=PluginData.Romfarer)";
			Match match = Regex.Match (subPath, pattern, RegexOptions.IgnoreCase);
			if (match.Success) {
				ApplicationPath = match.ToString().Replace("\\","/");

			} */                                                      
			print ("LazorSystemRuntime app.path: "+ GameDatabase.Instance.PluginDataFolder);
			print ("LazorSystemRuntime plugin.path: "+ ApplicationPath);


			/*foreach (AssemblyLoader.LoadedAssembly a in AssemblyLoader.loadedAssemblies) {
				foreach (System.Type t in a.assembly.GetTypes())
				{
					if (t.ToString() == typeof(LazorSystem).ToString()) {
						print ("xxx -> " + a.dllName);
						print ("xxx -> " + a.path);
						print ("xxx -> " + a.url);
					}
				}
			}*/




			Romfarer.Events.onVesselChange += OnVesselChange;
			//add LazorEVA partmodule to EVA prefab
			foreach (Part p in Resources.FindObjectsOfTypeAll(typeof(Part))){
				if(p.name == "kerbalEVA_RD"){
					p.gameObject.AddComponent<LazorEVA>();
				}
			}
			GameEvents.onGameSceneLoadRequested.Add(GameEvents_onGameSceneLoadRequested);
			GameEvents.onPartRemove.Add(GameEvents_onPartRemove);
			GameEvents.onPartCouple.Add(GameEvents_onPartCouple);
			GameEvents.onTimeWarpRateChanged.Add(GameEvents_onTimeWarpRateChanged);
			GameEvents.onFlightReady.Add(GameEvents_onFlightReady);


			GameEvents.onGUIAstronautComplexSpawn.Add(GameEvents_onGUIAstronautComplexSpawn);
			GameEvents.onGUIAstronautComplexDespawn.Add(GameEvents_onGUIAstronautComplexDespawn);
			GameEvents.onGUIRnDComplexSpawn.Add(GameEvents_onGUIRnDComplexSpawn);
			GameEvents.onGUIRnDComplexDespawn.Add(GameEvents_onGUIRnDComplexDespawn);
		}


		private void GameEvents_onGUIAstronautComplexSpawn(){
			showSpaceCenterGUI = false;
		}
		private void GameEvents_onGUIAstronautComplexDespawn(){
			showSpaceCenterGUI = true;
		}

		private void GameEvents_onGUIRnDComplexSpawn(){
			showSpaceCenterGUI = false;
		}
		private void GameEvents_onGUIRnDComplexDespawn(){
			showSpaceCenterGUI = true;
		}

		public void OnDestroy() {
			GameEvents.onGameSceneLoadRequested.Remove(GameEvents_onGameSceneLoadRequested);
			GameEvents.onPartRemove.Remove(GameEvents_onPartRemove);
			GameEvents.onPartCouple.Remove(GameEvents_onPartCouple);
			GameEvents.onTimeWarpRateChanged.Remove(GameEvents_onTimeWarpRateChanged);
			GameEvents.onFlightReady.Remove(GameEvents_onFlightReady);
		}

		public void GameEvents_onTimeWarpRateChanged() {
			if(TimeWarp.CurrentRate != 1)
				LazorGlobals.SetLoadDistance();
			else 
				LazorGlobals.SetLoadDistance(LazorGlobals.MAX_DISTANCE, LazorGlobals.VIEW_DISTANCE);

		}

		private void GameEvents_onGameSceneLoadRequested(GameScenes scene) {
			Util.ResetGUI();
		}

		private void GameEvents_onPartCouple(GameEvents.FromToAction<Part, Part> action) {
			if(inFlight && action.from.vessel != action.to.vessel) {
				RuntimeLogic.Instance.Log("(f" + Time.frameCount + ") COUPLE " + action.from + " " + action.to, false);
				LazorSystemControllerGlobal.MergeLazorControllers(action.from.vessel, action.to.vessel);
			}
		}

		private void GameEvents_onPartRemove(GameEvents.HostTargetAction<Part, Part> action) {
			if(inFlight){
				//RuntimeLogic.Instance.Log("(f" + Time.frameCount + ") test " + action.target.vessel.vesselName + " " + action.target.parent.vessel.vesselName, false);
				RuntimeLogic.Instance.Log("(f" + Time.frameCount + ") REMOVE " + action.host + " " + action.target + " " + action.target.parent, false);
				LazorSystemControllerGlobal.EnableSystemLock (action.target.parent.vessel, (sys) => {return sys.sysLock_WaitDisconnect;});
				/*LazorSystemController lsc = LazorSystemControllerGlobal.GetController(action.target.parent.vessel);
				if(lsc != null){
					RuntimeLogic.Instance.Log("(f" + Time.frameCount + ") REMOVE " + action.host + " " + action.target + " " + action.target.parent, false);
					//lsc.Enable_SysLock_WaitDisconnect();
					lsc.EnableSystemLock ((sys)=>{return sys.sysLock_WaitDisconnect;});
				}*/
			}
		}

		private void GameEvents_onFlightReady(){
			//MonoBehaviour.print("LazorSystem RuntimeLogic FlightStart sequence initiated.");
			//LazorGlobals.manager = (OrbitPhysicsManager)GameObject.FindObjectOfType(typeof(OrbitPhysicsManager));
			LazorGlobals.Initialize();
			LazorGlobals.SetLoadDistance(LazorGlobals.MAX_DISTANCE, LazorGlobals.VIEW_DISTANCE);
		}

		public void Update(){
			//inFlight.set = HighLogic.LoadedSceneIsFlight && FlightGlobals.fetch != null && FlightGlobals.ready; 
			/*if(inFlight.In())
				flightStart();
			else if(inFlight.Out()) {}*/

			if(inFlight) {
				LazorGlobals.LastVessel = FlightGlobals.ActiveVessel;
				//timeWarp.set = TimeWarp.CurrentRate != 1;
				/*
				if(timeWarp.In())
					SetLoadDistance(2500, 2250, 350, 5000);
				else if(timeWarp.Out())
					SetLoadDistance(LazorGlobals.MAX_DISTANCE, LazorGlobals.VIEW_DISTANCE, LazorGlobals.LANDEDPARTPACK_DISTANCE, LazorGlobals.PARTPACK_DISTANCE);
*/

				/*
				//Destroy launch clamps
				if(!launching && FlightGlobals.ActiveVessel.situation == Vessel.Situations.PRELAUNCH){
					launching = true;
					List<Part> clamps = new List<Part>();
					foreach (Part p in FlightGlobals.ActiveVessel.parts)
						if (p.Modules.Contains("LaunchClamp"))
							clamps.Add(p);
					if (clamps.Count > 0){
						launchClampsDetected = true;
						destroyOnLiftoff.Add (FlightGlobals.ActiveVessel.id, clamps);
					}
				}
				if(launchClampsDetected && FlightGlobals.ActiveVessel.situation == Vessel.Situations.FLYING){
					print("[LazorSystem : Destroying launch clamps]");
					foreach (Part p in destroyOnLiftoff[FlightGlobals.ActiveVessel.id])
						p.explode();
					destroyOnLiftoff.Clear();
					launchClampsDetected = false;
				}*/
			}
		}


		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// EVENTS
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////




		private void OnVesselChange(){
			MonoBehaviour.print("LazorSystem : RuntimeLogic : OnVesselChange()");
			if (FlightGlobals.ActiveVessel.isEVA && LazorSystemControllerGlobal.LastVesselWithLazor != null && FlightGlobals.ActiveVessel.rootPart.Modules["LazorEVA"].Fields.GetValue<string>("myVessel") == "-1") {
				print("[Adding memory to " + FlightGlobals.ActiveVessel.vesselName + "]");
				PartModule mem = FlightGlobals.ActiveVessel.rootPart.Modules["LazorEVA"];
				mem.Fields.SetValue("myVessel", LazorSystemControllerGlobal.LastVesselWithLazor.id.ToString());
			}
			/*destroyOnLiftoff.Clear();
			launchClampsDetected = false;
			launching = false;*/

			//add docking cam to all docking ports
			foreach (Part p in FlightGlobals.ActiveVessel.parts) {
				if (p.Modules.Contains("ModuleDockingNode")) {
					if (!p.Modules.Contains("LazorSystemDockingCamera")) {
						MonoBehaviour.print("[Adding docking camera to " + p.name + "]");
						PartModule pm = p.AddModule("LazorSystemDockingCamera");
					}
				}
			}
		}

		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// METHODS
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////


		/*private void SetLoadDistance(float loadDistance, float unloadDistance, float landedPartPackTreshold, float partPackTreshold){
			if (LazorGlobals.GLOBALCONFIG_CurrentLoadDistance != LazorGlobals.LoadDistance.Default) {
				MonoBehaviour.print ("--== LazorSystem RuntimeLogic SetLoadDistance ==--");
				if (LazorGlobals.manager == null) {
					LazorGlobals.manager = (OrbitPhysicsManager)GameObject.FindObjectOfType (typeof(OrbitPhysicsManager));
					MonoBehaviour.print ("[LazorSystem RuntimeLogic : New physics manager loaded]");
				}
				MonoBehaviour.print ("LazorSystem RuntimeLogic : loadDistance " + loadDistance + ", unloadDistance " + unloadDistance + ", landedPartPackTreshold " + landedPartPackTreshold + ", partPackTreshold " + partPackTreshold);
				//MonoBehaviour.print ("--== orig: " + Vessel.loadDistance + " " + Vessel.unloadDistance + " " + LazorGlobals.manager.distantLandedPartPackThreshold + " " + LazorGlobals.manager.distantPartPackThreshold);
				Vessel.loadDistance = loadDistance; //2500
				Vessel.unloadDistance = unloadDistance; //2250
				LazorGlobals.manager.distantLandedPartPackThreshold = landedPartPackTreshold; //95000; //350
				LazorGlobals.manager.distantPartPackThreshold = partPackTreshold; //95000; //5000
				//floatingOrigin = (FloatingOrigin)GameObject.FindObjectOfType(typeof(FloatingOrigin));
	            //if(floatingOrigin!=null)
	            //    floatingOrigin.threshold = 1000000000000;
				//MonoBehaviour.print ("--== new: " + Vessel.loadDistance + " " + Vessel.unloadDistance + " " + LazorGlobals.manager.distantLandedPartPackThreshold + " " + LazorGlobals.manager.distantPartPackThreshold);
			}
		}*/

		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// GUI
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////

		private bool settingsLoaded;

		/*private bool waitASecond;
		private float waitASecond_time = float.MaxValue;
		private bool WaitASecond() {
			if(!waitASecond) {
				waitASecond = true;
				waitASecond_time = Time.realtimeSinceStartup;
				MonoBehaviour.print("blabla "+Time.time + " "+Time.realtimeSinceStartup);
				return false;
			}else
				return Time.realtimeSinceStartup > waitASecond_time + 5;
		}*/
		public void OnGUI(){
			//Romfarer settings interface in space center
			if (HighLogic.LoadedScene == GameScenes.SPACECENTER){// && WaitASecond()) {
				if (!settingsLoaded) {
					LazorGlobals.LoadConfig();
					settingsLoaded = true;
				}
				GUI.skin = Util.LazorSkin;
				if(showSpaceCenterGUI)
					windowPos_SpaceCenter = Util.ClampToScreen (GUILayout.Window (GetInstanceID(), windowPos_SpaceCenter, GUIwindow_SpaceCenter, "Lazor Powered"));

				if (Event.current.type == EventType.Repaint) {
					windowPos_SpaceCenter.height = 10;
					windowPos_SpaceCenter.x = 0;
				}
			} else {
				settings_loading = false;
				settings_lazorsystem = false;
				//waitASecond = false;
			}

			//DEBUG
			#if LAZORDEBUG
			if(HighLogic.LoadedSceneIsFlight && FlightGlobals.ready && FlightGlobals.ActiveVessel != null && FlightGlobals.ActiveVessel.loaded) {
				GUI.skin = Util.LazorSkin;
				windowPos_DEBUG = GUILayout.Window(FlightGlobals.ActiveVessel.GetInstanceID(), windowPos_DEBUG, GUIwindow_DEBUG, "", GUIStyle.none);
				if (Event.current.type == EventType.Repaint)
					windowPos_DEBUG.height = 100;
			}
			#endif
		}



		private Rect windowPos_SpaceCenter = new Rect(0,0,150,10);
		private bool settings_loading;
		private bool settings_lazorsystem;

		private void GUIwindow_SpaceCenter(int windowID) {
			settings_loading = GUILayout.Toggle(settings_loading, "View distance", Util.Button_Toggle);
			if(settings_loading) {
				GUILayout.Space(5);
				//GUILayout.Label("Loading distance: ", Util.StdLabelSmallCustom(Color.yellow));
				LazorGlobals.GLOBALCONFIG_CurrentLoadDistance = (LazorGlobals.LoadDistance)GUILayout.SelectionGrid((int)LazorGlobals.GLOBALCONFIG_CurrentLoadDistance, Enum.GetNames(typeof(LazorGlobals.LoadDistance)), 4);

				GUILayout.BeginHorizontal(Util.Space_Compact);
				GUILayout.Label("View distance: ", Util.StdLabelSmallCustom(Color.green));
				GUILayout.Label(LazorGlobals.MAX_DISTANCE + " m");
				GUILayout.EndHorizontal();
			}

			settings_lazorsystem = GUILayout.Toggle(settings_lazorsystem, "Lazor System Settings", Util.Button_Toggle);
			if(settings_lazorsystem) {
				GUILayout.Space(5);
				LazorGlobals.GLOBALCONFIG_SyncTargets = GUILayout.Toggle(LazorGlobals.GLOBALCONFIG_SyncTargets, "Sync targeting");
				LazorGlobals.GLOBALCONFIG_ElectricGenerator = GUILayout.Toggle(LazorGlobals.GLOBALCONFIG_ElectricGenerator, "Electric Generator");
			}

			GUILayout.Space(5);
			if(GUILayout.Button("Save"))
				LazorGlobals.SaveConfig();



			/*GUILayout.Space(10);
			if (GUILayout.Button ("test")) {

				MonoBehaviour.print ("parts "+ Resources.FindObjectsOfTypeAll(typeof(Part)).Length);
				MonoBehaviour.print ("vessel "+ Resources.FindObjectsOfTypeAll(typeof(Vessel)).Length);
				foreach(Vessel v in Resources.FindObjectsOfTypeAll(typeof(Vessel)))
					MonoBehaviour.print (v.name);

				foreach (Part p in Resources.FindObjectsOfTypeAll(typeof(Part))){
					if(p.name == "kerbalEVA_RD"){
						MonoBehaviour.print ("FOUND part");
						//p.AddModule("LazorEVA");
						p.gameObject.AddComponent<LazorEVA>();
					}
				}
			}*/
		}




		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
		// DEBUG
		//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////




		private void GUIwindow_DEBUG(int windowID){
			DEBUG_gui_on = GUILayout.Toggle(DEBUG_gui_on, "Romfarer Debug");
			if(DEBUG_gui_on) {
				GUILayout.BeginHorizontal();
				DEBUG_hierarchy = GUILayout.Toggle(DEBUG_hierarchy, "Hierarchy");
				DEBUG_joints.set = GUILayout.Toggle(DEBUG_joints, "Joints");
				DEBUG_log = GUILayout.Toggle(DEBUG_log, "Log");
				GUILayout.FlexibleSpace();
				GUILayout.EndHorizontal();

				if(DEBUG_hierarchy) {
					scrollArea = GUILayout.BeginScrollView(scrollArea, false, false, GUILayout.MinHeight(800), GUILayout.ExpandHeight(false));
					Part p = FlightGlobals.ActiveVessel.rootPart;
					GUILayout.Label(p.name + " " + p.children.Count);
					Element("   |", p);
					GUILayout.EndScrollView();
				}

				if(DEBUG_joints.In()) {
					joints = UnityEngine.Object.FindObjectsOfType(typeof(Joint)).Cast<Joint>().ToList();
				} else if(DEBUG_joints.Out()) {}
				if(DEBUG_joints) {
					scrollArea = GUILayout.BeginScrollView(scrollArea, false, false, GUILayout.MinHeight(800), GUILayout.ExpandHeight(false));
					PrintJoints();
					GUILayout.EndScrollView();
				}

				if(DEBUG_log){
					GUILayout.BeginHorizontal();
					if(GUILayout.Button("clear log"))
						log = "Log:\n";
					if(GUILayout.Button("-----"))
						log += "-----------------------------\n";
					GUILayout.FlexibleSpace();
					GUILayout.EndHorizontal();

					scrollArea = GUILayout.BeginScrollView(scrollArea, false, false, GUILayout.MinHeight(800), GUILayout.ExpandHeight(false));
					GUILayout.Label(log);
					GUILayout.EndScrollView();
				}
				GUILayout.BeginHorizontal();
				GUILayout.BeginVertical();
				if(LazorSystemControllerGlobal.ActiveLazorSystem != null)
					LazorSystemControllerGlobal.ActiveLazorSystem.debugAssist.TestButtons();
				GUILayout.EndVertical();
				GUILayout.FlexibleSpace();
				GUILayout.EndHorizontal();
			}

		}

		private void Element(string a, Part p){
			using(List<Part>.Enumerator enumerator = p.children.GetEnumerator()) {
				while(enumerator.MoveNext()){
					GUILayout.Label(a+enumerator.Current.name + " (" + enumerator.Current.children.Count + ") " + 
						(enumerator.Current.attachJoint==null?"null":enumerator.Current.attachJoint.Target.partName));
				//GUILayout.Label(a+enumerator.Current.name + " (" + enumerator.Current.children.Count + ") " +  //XXX
				//	                (enumerator.Current.attachJoint==null?"null":enumerator.Current.attachJoint.connectedBody.name));
					Element(a + "   |", enumerator.Current);
				}
			}
		}

		private void PrintJoints(){
			joints.RemoveAll(x => x == null);
			joints.Sort(delegate(Joint a, Joint b){return (a.transform.position - FlightGlobals.ActiveVessel.rootPart.transform.position).magnitude.CompareTo(
					(b.transform.position - FlightGlobals.ActiveVessel.rootPart.transform.position).magnitude); });
			foreach(Joint j in joints){
				if(j != null)
					GUILayout.Label(j.name + " "+j.GetType()+" "+(j.transform.position - FlightGlobals.ActiveVessel.rootPart.transform.position).magnitude);
			}
		}

		private string log = "Log:\n";
		public void Log(string line, bool print){
			/*if (DEBUG) {
				log += line + "\n";
				if (print)
					MonoBehaviour.print (line);
			}*/
		}

	}
}

