﻿/* Author: Jim-Kyrre Benjaminsen AKA Romfarer
 * License: This work is licensed under a Creative Commons Attribution-NonCommercial-NoDerivs 3.0 Unported License. 
 * http://creativecommons.org/licenses/by-nc-nd/3.0/
*/

using System;
using System.Collections.Generic;
using System.Text;
using UnityEngine;

namespace Romfarer {
    class UtilAverage {

        private int size = 0;
        private LinkedList<double> linkedList = new LinkedList<double>();
        private double total = 0;


        public UtilAverage(int size) {
            this.size = size;
        }

        public double value {
            get {
                if (linkedList.Count == 0)
                    return 0;
                return total / linkedList.Count; 
            }
        }

        public static implicit operator double(UtilAverage a) {
            return a.value;
        }

        public double Average(double value) {
            linkedList.AddFirst(value);
            if (linkedList.Count > size) {
                total -= linkedList.Last.Value;
                linkedList.RemoveLast();
            }
            total += value;
            if (linkedList.Count == 0)
                return 0;
            return total / linkedList.Count;
        }

        public void Reset() {
            total = 0;
            linkedList.Clear();
        }
    }


    class UtilAverageVector {

        private int size = 0;
        private LinkedList<Vector3> linkedList = new LinkedList<Vector3>();
        private Vector3 total = Vector3.zero;

        //private LinkedList<double> linkedList_mag = new LinkedList<double>();
        //private double total_mag;

        public UtilAverageVector(int size) {
            this.size = size;
        }

        public Vector3 value {
            get {
                if (linkedList.Count == 0) {
                    return Vector3.zero;
                } //return total.normalized * ((float)total_mag / linkedList.Count);
                return total.normalized * (total.magnitude / linkedList.Count);
            }
        }

        public static implicit operator Vector3(UtilAverageVector v) {
            return v.value;
        }

        public Vector3 Average(Vector3 v) {
            linkedList.AddFirst(v);
            //linkedList_mag.AddFirst(v.magnitude);
            if (linkedList.Count > size) {
                total -= linkedList.Last.Value;
                linkedList.RemoveLast();

                //total_mag -= linkedList_mag.Last.Value;
                //linkedList_mag.RemoveLast();
            }
            total += v;
            //total_mag += v.magnitude;
            return value;
        }

        public void Reset() {
            total = Vector3.zero;
            linkedList.Clear();

            //total_mag = 0;
            //linkedList_mag.Clear();
        }
    }
}
